@@ -1,5 +1,5 @@
 /*
- *    Copyright 2012-2016 Kai Pastor
+ *    Copyright 2012-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -37,19 +37,43 @@ class QXmlStreamReader;
 class QXmlStreamWriter;
 // IWYU pragma: no_forward_declare QPointF
 
-typedef void* projPJ;
+#ifdef ACCEPT_USE_OF_DEPRECATED_PROJ_API_H
+using ProjTransformData = void;
+#else
+using ProjTransformData = struct PJconsts;
+#endif
 
 namespace OpenOrienteering {
 
 
-#if defined(Q_OS_ANDROID)
-
 /**
- * Registers a file finder function needed by Proj.4 on Android.
+ * A utility which encapsulates PROJ API variants and resource management.
  */
-extern "C" void registerProjFileHelper();
-
-#endif
+struct ProjTransform
+{
+	ProjTransform() noexcept = default;
+	ProjTransform(const ProjTransform&) = delete;
+	ProjTransform(ProjTransform&& other) noexcept;
+	ProjTransform(const QString& crs_spec);
+	~ProjTransform();
+	
+	ProjTransform& operator=(const ProjTransform& other) = delete;
+	ProjTransform& operator=(ProjTransform&& other) noexcept;
+	
+	bool isValid() const noexcept;
+	bool isGeographic() const;
+	
+	QPointF forward(const LatLon& lat_lon, bool* ok) const;
+	LatLon inverse(const QPointF& projected, bool* ok) const;
+	
+	QString errorText() const;
+	
+private:
+	ProjTransform(ProjTransformData* pj) noexcept;
+	
+	ProjTransformData* pj = nullptr;
+	
+};
 
 
 
@@ -66,7 +90,7 @@ extern "C" void registerProjFileHelper();
  * Conversions between projected coordinates and geographic coordinates (here:
  * latitude/longitude for the WGS84 datum) are made based on a specification
  * of the coordinate reference system of the projected coordinates. The actual
- * geographic transformation is done by the PROJ.4 library for geographic 
+ * geographic transformation is done by the PROJ library for geographic 
  * projections. 
  *
  * If no (valid) specification is given, the projected coordinates are regarded
@@ -80,7 +104,6 @@ class Georeferencing : public QObject  // clazy:exclude=copyable-polymorphic
 {
 Q_OBJECT
 
-friend class NativeFileImport;
 friend QDebug operator<<(QDebug dbg, const Georeferencing& georef);
 
 public:
@@ -97,7 +120,7 @@ public:
 	
 	
 	/**
-	 * A shared PROJ.4 specification of a WGS84 geographic CRS.
+	 * A shared PROJ specification of a WGS84 geographic CRS.
 	 */
 	static const QString geographic_crs_spec;
 	
@@ -147,7 +170,7 @@ public:
 	 * Note: Since QObjects may not be copied, this is better understood as
 	 * creating a new object with the same settings.
 	 */
-	Georeferencing(const Georeferencing& georeferencing);
+	Georeferencing(const Georeferencing& other);
 	
 	/** 
 	 * Cleans up memory allocated by the georeferencing 
@@ -197,7 +220,7 @@ public:
 	/**
 	 * Returns true if the "projected CRS" is actually geographic.
 	 * 
-	 * \see pj_is_latlong(projPJ pj) in PROJ.4
+	 * \see proj_angular_output(PJ *, enum PJ_DIRECTION) in PROJ
 	 */
 	bool isGeographic() const;
 	
@@ -210,7 +233,7 @@ public:
 	/**
 	 * Sets the georeferencing state.
 	 * 
-	 * This is only neccessary to decrease the state to Local, as otherwise it
+	 * This is only necessary to decrease the state to Local, as otherwise it
 	 * will be automatically changed when setting the respective values.
 	 */
 	void setState(State value);
@@ -268,7 +291,7 @@ public:
 	 * 
 	 * This will also affect the grivation value and the transformations.
 	 */
-	void setDeclination(double declination);
+	void setDeclination(double value);
 	
 	
 	/**
@@ -307,7 +330,7 @@ public:
 	 * 
 	 * This will also affect the declination value and the transformations.
 	 */
-	void setGrivation(double grivation);
+	void setGrivation(double value);
 	
 	
 	/**
@@ -320,7 +343,7 @@ public:
 	 * 
 	 * This will <b>not</b> update the map and geographic coordinates of the reference point.
 	 */
-	void setMapRefPoint(MapCoord point);
+	void setMapRefPoint(const MapCoord& point);
 	
 	
 	/**
@@ -334,7 +357,7 @@ public:
 	 * This may trigger changes of the geographic coordinates of the reference
 	 * point, the convergence, the grivation and the transformations.
 	 */
-	void setProjectedRefPoint(QPointF point, bool update_grivation = true);
+	void setProjectedRefPoint(const QPointF& point, bool update_grivation = true);
 	
 	
 	/**
@@ -361,7 +384,7 @@ public:
 	/** 
 	 * Returns the specification of the coordinate reference system (CRS) of the
 	 * projected coordinates
-	 * @return a PROJ.4 specification of the CRS
+	 * @return a PROJ specification of the CRS
 	 */
 	QString getProjectedCRSSpec() const { return projected_crs_spec; }
 	
@@ -374,7 +397,7 @@ public:
 	 * declination and grivation.
 	 * 
 	 * @param id  an identifier
-	 * @param spec the PROJ.4 specification of the CRS
+	 * @param spec the PROJ specification of the CRS
 	 * @param params parameter values (ignore for empty spec)
 	 * @return true if the specification is valid or empty, false otherwise
 	 */
@@ -511,9 +534,9 @@ public:
 	void setTransformationDirectly(const QTransform& transform);
 	
 	
-	QTransform mapToProjected() const;
+	const QTransform& mapToProjected() const;
 	
-	QTransform projectedToMap() const;
+	const QTransform& projectedToMap() const;
 	
 	
 signals:
@@ -565,12 +588,11 @@ private:
 	QString projected_crs_id;
 	QString projected_crs_spec;
 	std::vector< QString > projected_crs_parameters;
-	projPJ projected_crs;
+	
+	ProjTransform proj_transform;
 	
 	LatLon geographic_ref_point;
 	
-	projPJ geographic_crs;
-	
 };
 
 /**
@@ -614,7 +636,7 @@ double Georeferencing::roundDeclination(double value)
 inline
 bool Georeferencing::isValid() const
 {
-	return state == Local || projected_crs;
+	return state == Local || proj_transform.isValid();
 }
 
 inline
