@@ -36,7 +36,7 @@ namespace OpenOrienteering {
 class MapColor;
 
 /**
- * The MapColorCmyk class provides a datatype for storing and transfering 
+ * The MapColorCmyk class provides a datatype for storing and transferring 
  * opaque CMYK colors. 
  * 
  * Components (c, m, y, k) are floats in the range [0.0; 1.0].
@@ -85,7 +85,7 @@ bool operator!=(const MapColorCmyk& lhs, const MapColorCmyk& rhs);
 
 
 /**
- * The MapColorRgb class provides a datatype for storing and transfering 
+ * The MapColorRgb class provides a datatype for storing and transferring 
  * opaque RGB colors. 
  * 
  * Components (r, g, b) are floats in the range [0.0; 1.0].
@@ -271,6 +271,36 @@ public:
 	 */
 	void setSpotColorName(const QString& spot_color_id);
 	
+	/**
+	 * Returns the spot color halftone screen frequency in lines per inch.
+	 * 
+	 * This value is undefined for spot color methods other than SpotColor.
+	 */
+	double getScreenFrequency() const { return screen_frequency; }
+	
+	/**
+	 * Sets the spot color halftone screen frequency in lines per inch.
+	 * 
+	 * A negative or zero value indicates that the halftone screen is undefined.
+	 * 
+	 * This function does nothing for spot color methods other than SpotColor.
+	 */
+	void setScreenFrequency(double value);
+	
+	/**
+	 * Returns the spot color halftone screen angle in lines per inch.
+	 * 
+	 * This value is undefined for spot color methods other than SpotColor.
+	 */
+	double getScreenAngle() const { return screen_angle; }
+	
+	/**
+	 * Sets the spot color halftone screen angle in lines per inch.
+	 * 
+	 * This function does nothing for spot color methods other than SpotColor.
+	 */
+	void setScreenAngle(double value);
+	
 	/** 
 	 * Sets the given components (i.e. screens and/or overprint) for the color,
 	 * and sets the spot color method to CustomColor.
@@ -438,6 +468,8 @@ protected:
 	char flags;
 	
 	QString spot_color_name;
+	double screen_frequency = -1;  ///< Halftone screen frequency in lines per inch.
+	double screen_angle     =  0;  ///< Halftone screen angle in degrees.
 	SpotColorComponents components;
 };
 
