@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2016  Kai Pastor
+ *    Copyright 2012-2018  Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -26,7 +26,6 @@
 #include <Qt>
 #include <QtMath>
 #include <QColor>
-#include <QDebug>
 #include <QHash>
 #include <QImage>
 #include <QLatin1String>
@@ -60,8 +59,12 @@
 
 // ### A namespace which collects various string constants of type QLatin1String. ###
 
-namespace literal
-{
+namespace OpenOrienteering {
+
+namespace {
+
+namespace literal {
+
 	static const QLatin1String scale("scale");
 	static const QLatin1String resolution("resolution");
 	static const QLatin1String templates_visible("templates_visible");
@@ -86,20 +89,20 @@ namespace literal
 	static const QLatin1String print_area("print_area");
 	static const QLatin1String center_area("center_area");
 	static const QLatin1String single_page("single_page");
-}
+	
+}  // namespace literal
 
 
+}  // namespace
 
-namespace OpenOrienteering {
 
 // #### MapPrinterPageFormat ###
 
 MapPrinterPageFormat::MapPrinterPageFormat(QSizeF page_rect_size, qreal margin, qreal overlap) 
- :
 #ifdef QT_PRINTSUPPORT_LIB
-   paper_size(QPrinter::Custom)
+ : page_size(QPageSize::Custom)
 #else
-   paper_size(-1)
+ : page_size(QStringLiteral("Custom"))
 #endif
  , orientation(Portrait)
  , page_rect(QRectF(QPointF(margin, margin), page_rect_size))
@@ -113,7 +116,7 @@ MapPrinterPageFormat::MapPrinterPageFormat(QSizeF page_rect_size, qreal margin,
 #ifdef QT_PRINTSUPPORT_LIB
 
 MapPrinterPageFormat::MapPrinterPageFormat(const QPrinter& printer, qreal overlap)
- : paper_size(printer.paperSize())
+ : page_size(printer.pageLayout().pageSize().id())
  , orientation((printer.orientation() == QPrinter::Portrait) ? MapPrinterPageFormat::Portrait : MapPrinterPageFormat::Landscape)
  , page_rect(printer.pageRect(QPrinter::Millimeter))
  , paper_dimensions(printer.paperSize(QPrinter::Millimeter))
@@ -137,7 +140,7 @@ MapPrinterPageFormat MapPrinterPageFormat::fromDefaultPrinter()
 
 bool operator==(const MapPrinterPageFormat& lhs, const MapPrinterPageFormat& rhs)
 {
-	return  lhs.paper_size  == rhs.paper_size &&
+	return  lhs.page_size == rhs.page_size &&
 	        lhs.orientation == rhs.orientation &&
 	        fabs(lhs.h_overlap - rhs.h_overlap) < 0.05 &&
 	        fabs(lhs.v_overlap - rhs.v_overlap) < 0.05 &&
@@ -163,6 +166,17 @@ MapPrinterOptions::MapPrinterOptions(unsigned int scale, int resolution, MapPrin
 	// nothing
 }
 
+bool operator==(const MapPrinterOptions& lhs, const MapPrinterOptions& rhs)
+{
+	return     lhs.mode                  == rhs.mode
+	        && lhs.color_mode            == rhs.color_mode
+	        && lhs.resolution            == rhs.resolution
+	        && lhs.scale                 == rhs.scale
+	        && lhs.show_templates        == rhs.show_templates
+	        && lhs.show_grid             == rhs.show_grid
+	        && lhs.simulate_overprinting == rhs.simulate_overprinting;
+}
+
 
 
 // ### MapPrinterConfig ###
@@ -204,7 +218,7 @@ MapPrinterConfig::MapPrinterConfig(const Map& map, QXmlStreamReader& xml)
 		else if (mode == literal::separations)
 			options.mode = MapPrinterOptions::Separations;
 		else
-			qDebug() << "Unsupported map printing mode:" << mode;
+			qDebug("Unsupported map printing mode: %s", mode.toUtf8().constData());
 	}
 	QStringRef color_mode = printer_config_element.attribute<QStringRef>(literal::color_mode);
 	if (!color_mode.isEmpty())
@@ -214,7 +228,7 @@ MapPrinterConfig::MapPrinterConfig(const Map& map, QXmlStreamReader& xml)
 		else if (color_mode == literal::device_cmyk)
 			options.color_mode = MapPrinterOptions::DeviceCmyk;
 		else
-			qDebug() << "Unsupported map color mode:" << color_mode;
+			qDebug("Unsupported map color mode: %s", color_mode.toUtf8().constData());
 	}
 	
 	while (xml.readNextStartElement())
@@ -224,14 +238,21 @@ MapPrinterConfig::MapPrinterConfig(const Map& map, QXmlStreamReader& xml)
 			XmlElementReader page_format_element(xml);
 			QString value;
 			
-#ifdef QT_PRINTSUPPORT_LIB
 			value = page_format_element.attribute<QString>(literal::paper_size);
+#ifdef QT_PRINTSUPPORT_LIB
 			const QHash< int, const char* >& paper_size_names = MapPrinter::paperSizeNames();
-			for (int i = 0; i < paper_size_names.count(); ++i)
+			for (int i = 0; i <= static_cast<int>(QPageSize::PageSizeId::LastPageSize); ++i)
 			{
-				if (value == QLatin1String(paper_size_names[i]))
-					page_format.paper_size = i;
+				auto const page_size = static_cast<QPageSize::PageSizeId>(i);
+				if (value == QPageSize::key(page_size)
+				    || (paper_size_names.contains(i) && value == QLatin1String(paper_size_names[i])) )
+				{
+					page_format.page_size = page_size;
+					break;
+				}
 			}
+#else
+			page_format.page_size = value;
 #endif
 			
 			value = page_format_element.attribute<QString>(literal::orientation);
@@ -296,7 +317,7 @@ void MapPrinterConfig::save(QXmlStreamWriter& xml, const QLatin1String& element_
 		break;
 	default:
 		// Do not fail on saving
-		qDebug() << "Unsupported map printing mode:" << options.mode;
+		qDebug("Unsupported map printing mode: %d", int(options.mode));
 	}
 
 	switch (options.color_mode)
@@ -310,14 +331,17 @@ void MapPrinterConfig::save(QXmlStreamWriter& xml, const QLatin1String& element_
 		break;
 	default:
 		// Do not fail on saving
-		qDebug() << "Unsupported map color mode:" << options.color_mode;
+		qDebug("Unsupported map color mode: %d", int(options.color_mode));
 	}
 
 	{
 		XmlElementWriter page_format_element(xml, literal::page_format);
 #ifdef QT_PRINTSUPPORT_LIB
 		page_format_element.writeAttribute(literal::paper_size,
-		  MapPrinter::paperSizeNames()[page_format.paper_size]);
+		  QPageSize::key(page_format.page_size));
+#else
+		if (!page_format.page_size.isEmpty())
+			page_format_element.writeAttribute(literal::paper_size, page_format.page_size);
 #endif
 		page_format_element.writeAttribute(literal::orientation,
 		  (page_format.orientation == MapPrinterPageFormat::Portrait) ? literal::portrait : literal::landscape );
@@ -339,6 +363,17 @@ void MapPrinterConfig::save(QXmlStreamWriter& xml, const QLatin1String& element_
 }
 
 
+bool operator==(const MapPrinterConfig& lhs, const MapPrinterConfig& rhs)
+{
+	return     lhs.printer_name           == rhs.printer_name
+	        && lhs.print_area             == rhs.print_area
+	        && lhs.page_format            == rhs.page_format
+	        && lhs.options                == rhs.options
+	        && lhs.center_print_area      == rhs.center_print_area
+	        && lhs.single_page_print_area == rhs.single_page_print_area;
+}
+
+
 
 #ifdef QT_PRINTSUPPORT_LIB
 
@@ -356,21 +391,18 @@ const QPrinterInfo* MapPrinter::imageTarget()
 	return &image_target;
 }
 
+
+// QPageSize (::key(), ::name()) made this list mostly obsolete.
+// But we keep it in v0.9 for loading maps where we used names
+// different from QPageSize::key().
+/// @todo Remove MapPrinter::paperSizeNames() in Mapper 1.0.
+
+// static
 const QHash< int, const char* >& MapPrinter::paperSizeNames()
 {
 	static QHash< int, const char* > names;
 	if (names.empty())
 	{
-		names[QPrinter::A0]        = "A0";
-		names[QPrinter::A1]        = "A1";
-		names[QPrinter::A2]        = "A2";
-		names[QPrinter::A3]        = "A3";
-		names[QPrinter::A4]        = "A4";
-		names[QPrinter::A5]        = "A5";
-		names[QPrinter::A6]        = "A6";
-		names[QPrinter::A7]        = "A7";
-		names[QPrinter::A8]        = "A8";
-		names[QPrinter::A9]        = "A9";
 		names[QPrinter::B0]        = "B0";
 		names[QPrinter::B1]        = "B1";
 		names[QPrinter::B2]        = "B2";
@@ -385,34 +417,25 @@ const QHash< int, const char* >& MapPrinter::paperSizeNames()
 		names[QPrinter::C5E]       = "C5E";
 		names[QPrinter::DLE]       = "DLE";
 		names[QPrinter::Executive] = "Executive";
-		names[QPrinter::Folio]     = "Folio";
-		names[QPrinter::Ledger]    = "Ledger";
-		names[QPrinter::Legal]     = "Legal";
-		names[QPrinter::Letter]    = "Letter";
-		names[QPrinter::Tabloid]   = "Tabloid";
 		names[QPrinter::Comm10E]   = "US Common #10 Envelope";
-		names[QPrinter::Custom]    = "Custom";
 	}
 	return names;
 }
 
 
 MapPrinter::MapPrinter(Map& map, const MapView* view, QObject* parent)
-: QObject(parent),
-  MapPrinterConfig(map.printerConfig()),
-  map(map),
-  view(view),
-  target(nullptr)
+: QObject(parent)
+, MapPrinterConfig(map.printerConfig())
+, map(map)
+, view(view)
 {
 	scale_adjustment = map.getScaleDenominator() / qreal(options.scale);
 	updatePaperDimensions();
 	connect(&map.getGeoreferencing(), &Georeferencing::transformationChanged, this, &MapPrinter::mapScaleChanged);
 }
 
-MapPrinter::~MapPrinter()
-{
-	// Do not remove.
-}
+MapPrinter::~MapPrinter() = default;
+
 
 void MapPrinter::saveConfig() const
 {
@@ -441,9 +464,9 @@ void MapPrinter::setTarget(const QPrinterInfo* new_target)
 		if (old_target == imageTarget() || new_target == imageTarget())
 		{
 			// No page margins. Will emit pageFormatChanged( ).
-			setCustomPaperSize(page_format.page_rect.size());
+			setCustomPageSize(page_format.page_rect.size());
 		}
-		else if (page_format.paper_size != QPrinter::Custom)
+		else if (page_format.page_size != QPageSize::Custom)
 		{
 			updatePaperDimensions();
 			emit pageFormatChanged(page_format);
@@ -458,19 +481,19 @@ std::unique_ptr<QPrinter> MapPrinter::makePrinter() const
 	std::unique_ptr<QPrinter> printer;
 	if (!target)
 	{
-		printer.reset(new QPrinter(QPrinter::HighResolution));
+		printer = std::make_unique<QPrinter>(QPrinter::HighResolution);
 	}
 	else if (isPrinter())
 	{
-		printer.reset(new QPrinter(*target, QPrinter::HighResolution));
+		printer = std::make_unique<QPrinter>(*target, QPrinter::HighResolution);
 	}
 	else if (options.color_mode == MapPrinterOptions::DeviceCmyk)
 	{
-		printer.reset(new AdvancedPdfPrinter(*target, QPrinter::HighResolution));
+		printer = std::make_unique<AdvancedPdfPrinter>(*target, QPrinter::HighResolution);
 	}
 	else
 	{
-		printer.reset(new QPrinter(*target, QPrinter::HighResolution));
+		printer = std::make_unique<QPrinter>(*target, QPrinter::HighResolution);
 		printer->setOutputFormat(QPrinter::PdfFormat);
 	}
 	
@@ -489,19 +512,19 @@ std::unique_ptr<QPrinter> MapPrinter::makePrinter() const
 	
 	printer->setDocName(::OpenOrienteering::MapPrinter::tr("- Map -"));
 	printer->setFullPage(true);
-	if (page_format.paper_size == QPrinter::Custom)
+	if (page_format.page_size == QPageSize::Custom)
 	{
 		printer->setPaperSize(page_format.paper_dimensions, QPrinter::Millimeter);
 		printer->setOrientation(QPrinter::Portrait);
 	}
 	else
 	{
-		printer->setPaperSize(QPrinter::PaperSize(page_format.paper_size));
+		printer->setPageSize(QPageSize{page_format.page_size});
 		printer->setOrientation((page_format.orientation == MapPrinterPageFormat::Portrait) ? QPrinter::Portrait : QPrinter::Landscape);
 	}
 	printer->setResolution(options.resolution);
 	
-	if (page_format.paper_size == QPrinter::Custom || !isPrinter())
+	if (page_format.page_size == QPageSize::Custom || !isPrinter())
 	{
 		printer->setPageMargins(0.0, 0.0, 0.0, 0.0, QPrinter::Millimeter);
 	}
@@ -525,7 +548,7 @@ void MapPrinter::setPrintArea(const QRectF& area)
 		print_area = area;
 		
 		if (target == imageTarget() && print_area.size() != page_format.paper_dimensions)
-			setCustomPaperSize(print_area.size() * scale_adjustment);
+			setCustomPageSize(print_area.size() * scale_adjustment);
 		
 		updatePageBreaks();
 		
@@ -534,23 +557,23 @@ void MapPrinter::setPrintArea(const QRectF& area)
 }
 
 // slot
-void MapPrinter::setPaperSize(const int size)
+void MapPrinter::setPageSize(QPageSize::PageSizeId size)
 {
-	if (page_format.paper_size != size)
+	if (page_format.page_size != size)
 	{
-		if (size == QPrinter::Custom)
+		if (size == QPageSize::Custom)
 		{
-			setCustomPaperSize(page_format.paper_dimensions);
+			setCustomPageSize(page_format.paper_dimensions);
 		}
 		else
 		{
-			if ( page_format.paper_size == QPrinter::Custom &&
+			if ( page_format.page_size == QPageSize::Custom &&
 			     page_format.paper_dimensions.width() > page_format.paper_dimensions.height() )
 			{
 				// After QPrinter::Custom, determine orientation from actual dimensions.
 				page_format.orientation = MapPrinterPageFormat::Landscape;
 			}
-			page_format.paper_size = size;
+			page_format.page_size = size;
 			updatePaperDimensions();
 		}
 		emit pageFormatChanged(page_format);
@@ -558,13 +581,13 @@ void MapPrinter::setPaperSize(const int size)
 }
 
 // slot
-void MapPrinter::setCustomPaperSize(const QSizeF dimensions)
+void MapPrinter::setCustomPageSize(const QSizeF& dimensions)
 {
-	if ((page_format.paper_size != QPrinter::Custom || 
+	if ((page_format.page_size != QPageSize::Custom || 
 	     page_format.paper_dimensions != dimensions) &&
 	     ! dimensions.isEmpty())
 	{
-		page_format.paper_size = QPrinter::Custom;
+		page_format.page_size = QPageSize::Custom;
 		page_format.orientation = MapPrinterPageFormat::Portrait;
 		page_format.paper_dimensions = dimensions;
 		updatePaperDimensions();
@@ -575,7 +598,7 @@ void MapPrinter::setCustomPaperSize(const QSizeF dimensions)
 // slot
 void MapPrinter::setPageOrientation(const MapPrinterPageFormat::Orientation orientation)
 {
-	if (page_format.paper_size == QPrinter::Custom)
+	if (page_format.page_size == QPageSize::Custom)
 	{
 		// do nothing
 		emit pageFormatChanged(page_format);
@@ -604,7 +627,7 @@ void MapPrinter::setOverlap(qreal h_overlap, qreal v_overlap)
 
 void MapPrinter::updatePaperDimensions()
 {
-	if (target == imageTarget() && page_format.paper_size == QPrinter::Custom)
+	if (target == imageTarget() && page_format.page_size == QPageSize::Custom)
 	{
 		// No margins, no need to query QPrinter.
 		page_format.page_rect = QRectF(QPointF(0.0, 0.0), page_format.paper_dimensions);
@@ -619,14 +642,14 @@ void MapPrinter::updatePaperDimensions()
 	if (!printer->isValid() || target == imageTarget() || target == pdfTarget())
 		printer->setOutputFormat(QPrinter::PdfFormat);
 	  
-	if (page_format.paper_size == QPrinter::Custom)
+	if (page_format.page_size == QPageSize::Custom)
 	{
 		printer->setPaperSize(page_format.paper_dimensions, QPrinter::Millimeter);
 		page_format.orientation = (printer->orientation() == QPrinter::Portrait) ? MapPrinterPageFormat::Portrait : MapPrinterPageFormat::Landscape; 
 	}
 	else
 	{
-		printer->setPaperSize(QPrinter::PaperSize(page_format.paper_size));
+		printer->setPageSize(static_cast<QPagedPaintDevice::PageSize>(page_format.page_size));
 		printer->setOrientation((page_format.orientation == MapPrinterPageFormat::Portrait) ? QPrinter::Portrait : QPrinter::Landscape);
 	}
 	
@@ -634,7 +657,7 @@ void MapPrinter::updatePaperDimensions()
 	page_format.paper_dimensions = page_format.page_rect.size();
 	
 	if ( target != imageTarget() && target != pdfTarget() &&
-		 page_format.paper_size != QPrinter::Custom )
+		 page_format.page_size != QPageSize::Custom )
 	{
 		qreal left, top, right, bottom;
 		printer->getPageMargins(&left, &top, &right, &bottom, QPrinter::Millimeter);
@@ -733,6 +756,87 @@ bool MapPrinter::isOutputEmpty() const
 	);
 }
 
+QSizeF MapPrinter::getPrintAreaPaperSize() const
+{
+	return getPrintArea().size() * scale_adjustment;
+}
+
+QSizeF MapPrinter::getPageRectPrintAreaSize() const
+{
+	return page_format.page_rect.size() / scale_adjustment;
+}
+
+
+bool MapPrinter::engineMayRasterize() const
+{
+#ifdef Q_OS_WIN
+	/*
+	 * For Windows, non-opaque drawing to QWin32PrintEngine will trigger internal
+	 * rasterization in Qt (implemented in QAlphaPaintEngine), in order to do care
+	 * for the printing system's incapability to handle alpha. However, this is not
+	 * desired in our map printing where raster output is an explicit option.
+	 * 
+	 * QAlphaPaintEngines rasterizes
+	 * - with the device's logical resolution but at least 300 dpi,
+	 *   cf. QAlphaPaintEnginePrivate::drawAlphaImage(const QRectF &rect)
+	 * - in tiles of 2048x2048 pixels,
+	 *   cf. QAlphaPaintEnginePrivate::drawAlphaImage(const QRectF &rect)
+	 * - when the painter is not opaque,
+	 *   cf. QAlphaPaintEngine::updateState(const QPaintEngineState &state)
+	 * - when the pen is not opaque,
+	 *   cf. QAlphaPaintEngine::updateState(const QPaintEngineState &state)
+	 * - when the brush is not opaque,
+	 *   cf. QAlphaPaintEngine::updateState(const QPaintEngineState &state)
+	 */
+	return !rasterModeSelected()
+	       && target != pdfTarget()
+	       && target != imageTarget();
+#else
+	/*
+	 * For macOS (Cocoa), QMacPrintEngine does not rasterize.
+	 * 
+	 * For Linux (CUPS), QCupsPrintEngine is based on QPdfEngine
+	 * (via QPdfPrintEngine) which does not rasterize. (However, the print
+	 * queue may rasterize, even when *spooling* to a PDF file. The vector
+	 * output is preserved when redirecting the output to a file in
+	 * Print... > Preview... > Print > Output file [...].)
+	 */
+	return false;
+#endif
+}
+
+bool MapPrinter::engineWillRasterize() const
+{
+#ifdef Q_OS_WIN
+	if (!engineMayRasterize())
+		return false;
+	
+	if (!view)
+		return map.hasAlpha();
+	
+	const auto visibility = view->effectiveMapVisibility();
+	return visibility.hasAlpha()
+	       || (visibility.visible && map.hasAlpha());
+#else
+	Q_ASSERT(!engineMayRasterize());
+	return false;
+#endif
+}
+
+bool MapPrinter::hasAlpha(const Template* temp) const
+{
+	if (temp->getTemplateState() != Template::Loaded)
+		return false;
+	
+	if (!view)
+		return temp->hasAlpha();
+	
+	const auto visibility = view->getTemplateVisibility(temp);
+	return visibility.hasAlpha()
+	       || (visibility.visible && temp->hasAlpha());
+}
+
+
 void MapPrinter::updatePageBreaks()
 {
 	Q_ASSERT(print_area.left() <= print_area.right());
@@ -800,7 +904,7 @@ void MapPrinter::takePrinterSettings(const QPrinter* printer)
 	
 	if (f != page_format)
 	{
-		page_format.paper_size = f.paper_size;
+		page_format.page_size = f.page_size;
 		page_format.orientation = f.orientation;
 		page_format.paper_dimensions = f.paper_dimensions;
 		page_format.page_rect = f.page_rect;
@@ -816,48 +920,44 @@ void MapPrinter::takePrinterSettings(const QPrinter* printer)
 	}
 }
 
-// local
-void drawBuffer(QPainter* device_painter, const QImage* page_buffer, qreal pixel2units)
-{
-	Q_ASSERT(page_buffer);
-	
-	device_painter->save();
-	device_painter->scale(pixel2units, pixel2units);
-	device_painter->setRenderHint(QPainter::SmoothPixmapTransform, false);
-	device_painter->drawImage(0, 0, *page_buffer);
-	device_painter->restore();
-}
 
-void MapPrinter::drawPage(QPainter* device_painter, float units_per_inch, const QRectF& page_extent, bool white_background, QImage* page_buffer) const
+void MapPrinter::drawPage(QPainter* device_painter, const QRectF& page_extent, QImage* page_buffer) const
 {
-	device_painter->save();
-	
-	device_painter->setRenderHint(QPainter::Antialiasing);
-	device_painter->setRenderHint(QPainter::SmoothPixmapTransform);
-	
-	QPainter* painter = device_painter;
-	
 	// Logical units per mm
-	qreal units_per_mm = units_per_inch / 25.4;
-	// Image pixels per mm
-	qreal pixel_per_mm = options.resolution / 25.4;
-	// Scaling from pixels to logical units
-	qreal pixel2units = units_per_inch / options.resolution;
-	// The current painter's resolution
-	qreal scale = units_per_mm;
+	const qreal units_per_mm = options.resolution / 25.4;
+	
+	const auto saved_hints  = device_painter->renderHints();
+	const auto render_hints = saved_hints
+	                          | QPainter::Antialiasing
+	                          | QPainter::SmoothPixmapTransform;
+	
+	// Determine transformation and clipping for page extent and region
+	const auto page_extent_transform = [this, units_per_mm, page_extent]() {
+		// Translate for top left page margin 
+		auto transform = QTransform::fromScale(units_per_mm, units_per_mm);
+		transform.translate(page_format.page_rect.left(), page_format.page_rect.top());
+		// Convert native map scale to print scale
+		transform.scale(scale_adjustment, scale_adjustment);
+		// Translate and clip for margins and print area
+		transform.translate(-page_extent.left(), -page_extent.top());
+		return transform;
+	}();
+	
+	const auto page_region_used = page_extent.intersected(print_area);
+	
 	
 	/*
 	 * Analyse need for page buffer
 	 * 
-	 * Painting raster images with opacity is not supported on print devices.
-	 * This can only be solved by merging the images before sending them to
+	 * Painting with opacity is not supported on some print devices.
+	 * This can only be solved by merging raster images before sending them to
 	 * the printer. For the map itself, this may result in loss of sharpness
 	 * and increase in data volume.
 	 * 
-	 * In vector mode, a page buffer (raster image) is used to collect background
-	 * templates and raster foreground templates. After this buffer is sent to
-	 * to the printer, map, grid, and
-	 * non-raster foreground templates are drawn on top of it.
+	 * In vector mode, a page buffer (raster image) is used to collect
+	 * background templates and even lower foreground templates until only
+	 * opaque templates are left in the foreground. Map, grid, and the remaining
+	 * foreground templates are drawn on top of it.
 	 * 
 	 * In raster mode, all map features are drawn to in regular order to
 	 * temporary images first.
@@ -865,45 +965,41 @@ void MapPrinter::drawPage(QPainter* device_painter, float units_per_inch, const
 	 * When the target is an image, use the temporary image to enforce the given
 	 * resolution.
 	 */
-	bool use_buffer_for_map = (options.mode == MapPrinterOptions::Raster || target == imageTarget());
-	bool use_buffer_for_background = use_buffer_for_map && options.show_templates;
-	bool use_buffer_for_foreground = use_buffer_for_map && options.show_templates;
-	if (view && options.show_templates)
+	const bool use_buffer_for_map = rasterModeSelected() || target == imageTarget() || engineWillRasterize();
+	bool use_page_buffer = use_buffer_for_map;
+	
+	auto first_front_template = map.getFirstFrontTemplate();
+	if (options.show_templates && engineMayRasterize())
 	{
-		if (!use_buffer_for_background)
+		// When we don't use a buffer for the map, i.e. when we draw in vector mode,
+		// we may need to put non-opaque foreground templates to the background
+		// in order to avoid rasterization 
+		if (!use_buffer_for_map)
 		{
-			for (int i = 0; i < map.getFirstFrontTemplate() && !use_buffer_for_background; ++i)
+			first_front_template = map.getNumTemplates();
+			while (first_front_template > map.getFirstFrontTemplate()
+			       && !hasAlpha(map.getTemplate(first_front_template-1)))
 			{
-				if (map.getTemplate(i)->isRasterGraphics())
-				{
-					auto visibility = view->getTemplateVisibility(map.getTemplate(i));
-					use_buffer_for_background = visibility.visible && visibility.opacity < 1.0f;
-				}
+				--first_front_template;
 			}
 		}
-		if (!use_buffer_for_foreground)
+		
+		for (int i = 0; i < first_front_template && !use_page_buffer; ++i)
 		{
-			for (int i = map.getFirstFrontTemplate(); i < map.getNumTemplates() && !use_buffer_for_foreground; ++i)
-			{
-				if (map.getTemplate(i)->isRasterGraphics())
-				{
-					auto visibility = view->getTemplateVisibility(map.getTemplate(i));
-					use_buffer_for_foreground = visibility.visible && visibility.opacity < 1.0f;
-				}
-			}
+			use_page_buffer = hasAlpha(map.getTemplate(i));
 		}
 	}
 	
-	// Prepare buffer if required
-	bool use_page_buffer = use_buffer_for_map ||
-	                       use_buffer_for_background ||
-	                       use_buffer_for_foreground;
-	QImage scoped_buffer;
+	/*
+	 * Use a local "page painter", redirected to a local buffer if needed.
+	 */
+	auto* page_painter = device_painter;
+	QImage local_page_buffer;
+	QPainter local_page_painter;
 	if (use_page_buffer && !page_buffer)
 	{
-		scale = pixel_per_mm;
-		int w = qCeil(page_format.paper_dimensions.width() * scale);
-		int h = qCeil(page_format.paper_dimensions.height() * scale);
+		int w = qCeil(page_format.paper_dimensions.width() * units_per_mm);
+		int h = qCeil(page_format.paper_dimensions.height() * units_per_mm);
 #if defined (Q_OS_MACOS)
 		if (device_painter->device()->physicalDpiX() == 0)
 		{
@@ -912,113 +1008,86 @@ void MapPrinter::drawPage(QPainter* device_painter, float units_per_inch, const
 			// the corresponding QPaintEngine must handle the resolution mapping"
 			// which doesn't seem to happen here.
 			qreal corr = device_painter->device()->logicalDpiX() / 72.0;
-			w = qCeil(page_format.paper_dimensions.width() * scale * corr);
-			h = qCeil(page_format.paper_dimensions.height() * scale * corr);
+			w = qCeil(page_format.paper_dimensions.width() * units_per_mm * corr);
+			h = qCeil(page_format.paper_dimensions.height() * units_per_mm * corr);
 		}
 #endif
 		
-		scoped_buffer = QImage(w, h, QImage::Format_RGB32);
-		if (scoped_buffer.isNull())
+		local_page_buffer = QImage(w, h, QImage::Format_RGB32);
+		if (local_page_buffer.isNull())
 		{
 			// Allocation failed
-			device_painter->restore();
 			device_painter->end(); // Signal error
 			return;
 		}
+		local_page_buffer.fill(QColor(Qt::white));
+		local_page_painter.begin(&local_page_buffer);
 		
-		page_buffer = &scoped_buffer;
-		painter = new QPainter(page_buffer);
-		painter->setRenderHints(device_painter->renderHints());
+		page_buffer = &local_page_buffer;
+		page_painter = &local_page_painter;
 	}
 	
-	/*
-	 * Prepare the common background
-	 */
-	if (use_page_buffer)
-	{
-		page_buffer->fill(QColor(Qt::white));
-	}
-	else if (white_background)
-	{
-		painter->fillRect(QRect(0, 0, painter->device()->width(), painter->device()->height()), Qt::white);
-	}
-	
-	/*
-	 * One-time setup of transformation and clipping
-	 */
-	// Translate for top left page margin 
-	painter->scale(scale, scale);
-	painter->translate(page_format.page_rect.left(), page_format.page_rect.top());
-	
-	// Convert native map scale to print scale
-	if (scale_adjustment != 1.0)
-	{
-		scale *= scale_adjustment;
-		painter->scale(scale_adjustment, scale_adjustment);
-	}
-	
-	// Translate and clip for margins and print area
-	painter->translate(-page_extent.left(), -page_extent.top());
-	
-	QTransform transform = painter->transform();
-	
-	QRectF page_region_used(page_extent.intersected(print_area));
-	painter->setClipRect(page_region_used, Qt::ReplaceClip);
-	
 	/*
 	 * Draw templates in the background
 	 */
 	if (options.show_templates)
 	{
-		map.drawTemplates(painter, page_region_used, 0, map.getFirstFrontTemplate() - 1, view, false);
-		if (vectorModeSelected() && use_buffer_for_foreground)
-		{
-			for (int i = map.getFirstFrontTemplate(); i < map.getNumTemplates(); ++i)
-			{
-				if (map.getTemplate(i)->isRasterGraphics())
-					map.drawTemplates(painter, page_region_used, i, i, view, false);
-			}
-		}
+		// This is the first output, so it can be drawn directly to page_painter.
+		page_painter->save();
+		
+		page_painter->setRenderHints(render_hints);
+		page_painter->setTransform(page_extent_transform, /*combine*/ true);
+		page_painter->setClipRect(page_region_used, Qt::ReplaceClip);
+		
+		map.drawTemplates(page_painter, page_region_used, 0, first_front_template - 1, view, false);
+		
+		page_painter->restore();
 	}
 	
-	if (use_buffer_for_background && !use_buffer_for_map)
+	if (local_page_painter.isActive() && !use_buffer_for_map)
 	{
 		// Flush the buffer, reset painter
-		delete painter;
-		drawBuffer(device_painter, page_buffer, pixel2units);
+		local_page_painter.end();
 		
-		painter = device_painter;
-		painter->setTransform(transform);
-		painter->setClipRect(page_region_used, Qt::ReplaceClip);
+		device_painter->setRenderHint(QPainter::SmoothPixmapTransform, false);
+		device_painter->drawImage(0, 0, local_page_buffer);
+		
+		page_painter = device_painter;
 	}
 	
+	QImage local_buffer;
+	
 	/*
 	 * Draw the map
 	 */
 	if (!view || view->effectiveMapVisibility().visible)
 	{
-		QImage map_buffer;
-		QPainter* map_painter = painter;
+		QPainter* map_painter = page_painter;
+		
+		QPainter map_buffer_painter;
 		if (use_buffer_for_map)
 		{
 			// Draw map into a temporary buffer first which is printed with the map's opacity later.
 			// This prevents artifacts with overlapping objects.
-			if (painter == device_painter)
+			local_buffer = QImage(page_buffer->size(), QImage::Format_ARGB32_Premultiplied);
+			if (local_buffer.isNull())
 			{
-				map_buffer = *page_buffer; // Use existing buffer
+				// Allocation failed
+				device_painter->end(); // Signal error
+				return;
 			}
-			else
-			{
-				map_buffer = QImage(page_buffer->size(), QImage::Format_ARGB32_Premultiplied);
-			}
-			map_buffer.fill(QColor(Qt::transparent));
-			
-			map_painter = new QPainter(&map_buffer);
-			map_painter->setRenderHints(painter->renderHints());
-			map_painter->setTransform(painter->transform());
+			local_buffer.fill(QColor(Qt::transparent));
+			map_buffer_painter.begin(&local_buffer);
+			map_painter = &map_buffer_painter;
 		}
 		
-		RenderConfig config = { map, page_region_used, scale, RenderConfig::NoOptions, 1.0 };
+		page_painter->save();  // Modified via map_painter, or when drawing the map_buffer
+		
+		map_painter->setRenderHints(render_hints);
+		map_painter->setTransform(page_extent_transform, /*combine*/ true);
+		map_painter->setClipRect(page_region_used, Qt::ReplaceClip);
+		
+		RenderConfig config = { map, page_region_used, units_per_mm * scale_adjustment, RenderConfig::NoOptions, 1.0 };
 		
 		if (rasterModeSelected() && options.simulate_overprinting)
 		{
@@ -1026,76 +1095,123 @@ void MapPrinter::drawPage(QPainter* device_painter, float units_per_inch, const
 		}
 		else
 		{
-			if (vectorModeSelected() && view)
+			if (view && !map_buffer_painter.isActive())
 				config.opacity = view->effectiveMapVisibility().opacity;
 		
 			map.draw(map_painter, config);
 		}
 			
-		if (map_painter != painter)
+		if (map_buffer_painter.isActive())
 		{
 			// Flush the buffer
-			delete map_painter;
-			map_painter = nullptr;
+			map_buffer_painter.end();
 			
-			// Print buffer with map opacity
-			painter->save();
-			painter->resetTransform();
+			// Draw buffer with map opacity
 			if (view)
-				painter->setOpacity(view->effectiveMapVisibility().opacity);
-			painter->setRenderHint(QPainter::SmoothPixmapTransform, false);
-			painter->drawImage(0, 0, map_buffer);
-			painter->restore();
+				page_painter->setOpacity(view->effectiveMapVisibility().opacity);
+			page_painter->setRenderHint(QPainter::SmoothPixmapTransform, false);
+			page_painter->drawImage(0, 0, local_buffer);
 		}
+		
+		page_painter->restore();
 	}
 	
 	/*
 	 * Draw the grid
 	 */
 	if (options.show_grid)
-		map.drawGrid(painter, print_area, false); // Maybe replace by page_region_used?
+	{
+		page_painter->save();
+		
+		page_painter->setRenderHints(render_hints);
+		page_painter->setTransform(page_extent_transform, /*combine*/ true);
+		page_painter->setClipRect(page_region_used, Qt::ReplaceClip);
+		
+		const auto& map_grid = map.getGrid();
+		if (map_grid.hasAlpha() && !use_buffer_for_map && engineMayRasterize())
+		{
+			const auto rgba = map_grid.getColor();
+			const auto alpha = qAlpha(rgba);
+			const auto opaque = [alpha](auto component) {
+				return 255 - alpha * (255 - component) / 255;
+			};
+			auto grid_copy = map_grid;
+			grid_copy.setColor(qRgb(opaque(qRed(rgba)), opaque(qGreen(rgba)), opaque(qBlue(rgba))));
+			grid_copy.draw(page_painter, print_area, &map, scale_adjustment); // Maybe replace by page_region_used?
+		}
+		else
+		{
+			map_grid.draw(page_painter, print_area, &map, scale_adjustment); // Maybe replace by page_region_used?
+		}
+		
+		page_painter->restore();
+	}
 	
 	/*
 	 * Draw the foreground templates
 	 */
 	if (options.show_templates)
 	{
-		if (vectorModeSelected() && use_buffer_for_foreground)
+		QPainter* painter = page_painter;
+		
+		QPainter local_buffer_painter;
+		if (!vectorModeSelected() && use_buffer_for_map)
 		{
-			for (int i = map.getFirstFrontTemplate(); i < map.getNumTemplates(); ++i)
+			if (local_buffer.isNull())
 			{
-				if (!map.getTemplate(i)->isRasterGraphics())
-					map.drawTemplates(painter, page_region_used, i, i, view, false);
+				local_buffer = QImage(page_buffer->size(), QImage::Format_ARGB32_Premultiplied);
 			}
+			if (local_buffer.isNull())
+			{
+				// Allocation failed
+				device_painter->end(); // Signal error
+				return;
+			}
+			local_buffer.fill(QColor(Qt::transparent));
+			local_buffer_painter.begin(&local_buffer);
+			painter = &local_buffer_painter;
 		}
-		else
+		
+		page_painter->save();  // Modified via painter, or when drawing the local_buffer
+		
+		painter->setRenderHints(render_hints);
+		painter->setTransform(page_extent_transform, /*combine*/ true);
+		painter->setClipRect(page_region_used, Qt::ReplaceClip);
+		
+		map.drawTemplates(painter, page_region_used, first_front_template, map.getNumTemplates() - 1, view, false);
+		
+		if (local_buffer_painter.isActive())
 		{
-			if (use_buffer_for_foreground && !use_buffer_for_map)
-			{
-				page_buffer->fill(QColor(Qt::transparent));
-				painter = new QPainter(page_buffer);
-				painter->setRenderHints(device_painter->renderHints());
-				painter->setTransform(transform);
-				painter->setClipRect(page_region_used, Qt::ReplaceClip);
-			}
-			map.drawTemplates(painter, page_region_used, map.getFirstFrontTemplate(), map.getNumTemplates() - 1, view, false);
+			// Flush the buffer
+			local_buffer_painter.end();
+			
+			// Draw buffer with map opacity
+			page_painter->setRenderHint(QPainter::SmoothPixmapTransform, false);
+			page_painter->drawImage(0, 0, local_buffer);
 		}
+		
+		page_painter->restore();
 	}
-
+	
+	// Release the second buffer's memory first: The following drawImage()
+	// needs memory to allocate a large QPixmap with QWin32PrintEngine.
+	local_buffer = {};
+	
 	/*
 	 * Cleanup: If a temporary buffer has been used, paint it on the device painter
 	 */
-	if (painter != device_painter)
+	if (local_page_painter.isActive())
 	{
-		delete painter;
-		painter = nullptr;
-		device_painter->resetTransform();
-		drawBuffer(device_painter, page_buffer, pixel2units);
+		local_page_painter.end();
+		
+		device_painter->setRenderHint(QPainter::SmoothPixmapTransform, false);
+		device_painter->drawImage(0, 0, local_page_buffer);
 	}
-	device_painter->restore();
+	
+	device_painter->setRenderHints(saved_hints);
 }
 
-void MapPrinter::drawSeparationPages(QPrinter* printer, QPainter* device_painter, float dpi, const QRectF& page_extent) const
+void MapPrinter::drawSeparationPages(QPrinter* printer, QPainter* device_painter, const QRectF& page_extent) const
 {
 	Q_ASSERT(printer->colorMode() == QPrinter::GrayScale);
 	
@@ -1104,7 +1220,7 @@ void MapPrinter::drawSeparationPages(QPrinter* printer, QPainter* device_painter
 	device_painter->setRenderHint(QPainter::Antialiasing);
 	
 	// Translate for top left page margin 
-	qreal scale = dpi / 25.4; // Dots per mm
+	qreal scale = options.resolution / 25.4; // Dots per mm
 	device_painter->scale(scale, scale);
 	device_painter->translate(page_format.page_rect.left(), page_format.page_rect.top());
 	
@@ -1149,8 +1265,6 @@ bool MapPrinter::printMap(QPrinter* printer)
 	QSizeF extent_size = page_format.page_rect.size() / scale_adjustment;
 	QPainter painter(printer);
 	
-	auto resolution = float(options.resolution);
-	
 #if defined(Q_OS_WIN)
 	// Workaround for Wine
 	if (printer->resolution() == 0)
@@ -1158,42 +1272,14 @@ bool MapPrinter::printMap(QPrinter* printer)
 		return false;
 	}
 	
-	/* Non-opaque drawing of vector data to the raw printer will trigger
-	 * internal rasterization in Qt.
-	 */
-	auto engine_will_rasterize = [](const Map& map, const MapView* view, const MapPrinterOptions& options)->bool {
-		if (!view)
-			return false;
-		
-		if (options.mode != MapPrinterOptions::Raster)
-		{
-			auto visibility = view->effectiveMapVisibility();
-			if (visibility.visible && visibility.opacity < 1.0)
-				return true;
-		}
-			
-		for (int i = 0; i < map.getNumTemplates(); ++i)
-		{
-			auto temp = map.getTemplate(i);
-			if (temp->isRasterGraphics())
-				continue;
-			auto visibility = view->getTemplateVisibility(temp);
-			if (visibility.visible && visibility.opacity < 1.0)
-				return true;
-		}
-		
-		return false;
-	};
-
-	if (printer->paintEngine()->type() == QPaintEngine::Windows
-	    && !engine_will_rasterize(map, view, options) )
+	if (printer->paintEngine()->type() == QPaintEngine::Windows)
 	{
 		/* QWin32PrintEngine will (have to) do rounding when passing coordinates
 		 * to GDI, using the device's reported logical resolution.
 		 * We establish an MM_ISOTROPIC transformation from a higher resolution
 		 * to avoid the loss of precision due to this rounding.
-		 * However, output of rasterization produced by Qt fails when the
-		 * MM_ISOTROPIC transformation is active.
+		 * This transformation must only be used when we reliably avoid to
+		 * trigger rasterization in Qt (cf. engineMayRasterize etc.)
 		 */
 		
 		QWin32PrintEngine* engine = static_cast<QWin32PrintEngine*>(printer->printEngine());
@@ -1222,15 +1308,16 @@ bool MapPrinter::printMap(QPrinter* printer)
 			SetWindowExtEx(dc, hires_width, hires_height, nullptr);
 			SetViewportExtEx(dc, phys_width, phys_height, nullptr);
 			SetViewportOrgEx(dc, -phys_off_x, -phys_off_y, nullptr);
-			resolution *= ((double)hires_width / phys_width);
+			const auto hires_scale = static_cast<qreal>(hires_width) / phys_width;
+			painter.scale(hires_scale, hires_scale);
 		}
 	}
 	else if (printer->paintEngine()->type() == QPaintEngine::Picture)
 	{
 		// Preview: work around for offset, maybe related to QTBUG-5363
 		painter.translate(
-		    -page_format.page_rect.left()*resolution / 25.4,
-		    -page_format.page_rect.top()*resolution / 25.4   );
+		    -page_format.page_rect.left() * options.resolution / 25.4,
+		    -page_format.page_rect.top() * options.resolution / 25.4   );
 	}
 #endif
 	
@@ -1276,11 +1363,11 @@ bool MapPrinter::printMap(QPrinter* printer)
 			QRectF page_extent = QRectF(QPointF(hpos, vpos), extent_size);
 			if (separationsModeSelected())
 			{
-				drawSeparationPages(printer, &painter, resolution, page_extent);
+				drawSeparationPages(printer, &painter, page_extent);
 			}
 			else
 			{
-				drawPage(&painter, resolution, page_extent, false);
+				drawPage(&painter, page_extent);
 			}
 			
 			need_new_page = true;
@@ -1308,6 +1395,8 @@ void MapPrinter::cancelPrintMap()
 	cancel_print_map = true;
 }
 
+
+
 #endif  // QT_PRINTSUPPORT_LIB
 
 
