@@ -24,11 +24,11 @@
 #include <algorithm>
 #include <iterator>
 #include <new>
+#include <utility>
 
 #include <Qt>
 #include <QtGlobal>
 #include <QChar>
-#include <QCoreApplication>
 #include <QHash>
 #include <QLatin1Char>
 #include <QLatin1String>
@@ -40,7 +40,7 @@
 #include "core/symbols/symbol.h"
 
 
-// ### Local utilites ###
+// ### Local utilities ###
 
 namespace {
 
