@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2017 Kai Pastor
+ *    Copyright 2012-2018 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -157,7 +157,7 @@ void ObjectRenderables::draw(int map_color, const QColor& color, QPainter* paint
 		if (!state.activate(painter, current_clip, config, color, initial_clip))
 			continue;
 		
-		for (const auto renderable : config_renderables.second)
+		for (const auto* renderable : config_renderables.second)
 		{
 			if (renderable->intersects(config.bounding_box))
 			{
@@ -289,7 +289,7 @@ void MapRenderables::draw(QPainter *painter, const RenderConfig &config) const
 				if (!state.activate(painter, current_clip, config, color, initial_clip))
 				    continue;
 				
-				for (const auto renderable : renderables.second)
+				for (const auto* renderable : renderables.second)
 				{
 #ifdef Q_OS_ANDROID
 					const QRectF& extent = renderable->getExtent();
@@ -559,7 +559,7 @@ void MapRenderables::drawColorSeparation(QPainter* painter, const RenderConfig&
 				
 				// For each renderable that uses the current painter configuration...
 				// Render the renderable
-				for (const auto renderable : renderables.second)
+				for (const auto* renderable : renderables.second)
 				{
 					if (renderable->intersects(config.bounding_box))
 					{
@@ -603,7 +603,7 @@ void MapRenderables::removeRenderablesOfObject(const Object* object, bool mark_a
 					// ... because here it gets expensive
 					for (const auto& renderables : *obj->second)
 					{
-						for (const auto renderable : renderables.second)
+						for (const auto* renderable : renderables.second)
 						{
 							extent = extent.isValid() ? extent.united(renderable->getExtent()) : renderable->getExtent();
 						}
@@ -627,7 +627,7 @@ void MapRenderables::clear(bool mark_area_as_dirty)
 			{
 				for (const auto& renderables : *object.second)
 				{
-					for (const auto renderable : renderables.second)
+					for (const auto* renderable : renderables.second)
 					{
 						map->setObjectAreaDirty(renderable->getExtent());
 					}
@@ -714,12 +714,14 @@ bool PainterConfig::activate(QPainter* painter, const QPainterPath*& current_cli
 	QBrush brush(config.testFlag(RenderConfig::Highlighted) ? highlightedColor(color) : color);
 	if (mode == PainterConfig::PenOnly)
 	{
-#ifdef Q_OS_ANDROID
-		if (pen_width * config.scaling < 0.1)
-			return false;
-#endif
-		if (config.testFlag(RenderConfig::ForceMinSize) && pen_width * config.scaling <= 1.0)
-			actual_pen_width = 0.0; // Forces cosmetic pen
+		if (pen_width > 0)
+		{
+			auto const width_px = pen_width * config.scaling;
+			if (config.testFlag(RenderConfig::Screen) && width_px < 0.125)
+				return false;
+			if (config.testFlag(RenderConfig::ForceMinSize) && width_px < 1)
+				actual_pen_width = 0.0; // Forces cosmetic pen
+		}
 		painter->setPen(QPen(brush, actual_pen_width));
 		painter->setBrush(QBrush(Qt::NoBrush));
 	}
