@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2017 Kai Pastor
+ *    Copyright 2012-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -27,7 +27,6 @@
 #include <memory>
 
 #include <QtMath>
-#include <QIODevice>
 #include <QLatin1String>
 #include <QStringRef>
 #include <QXmlStreamReader> // IWYU pragma: keep
@@ -68,55 +67,7 @@ AreaSymbol::FillPattern::FillPattern() noexcept
 	// nothing else
 }
 
-#ifndef NO_NATIVE_FILE_FORMAT
 
-bool AreaSymbol::FillPattern::load(QIODevice* file, int version, Map* map)
-{
-	flags = Option::Default;
-	qint32 itype;
-	file->read((char*)&itype, sizeof(qint32));
-	type = (Type)itype;
-	file->read((char*)&angle, sizeof(float));
-	if (version >= 4)
-	{
-		bool is_rotatable;
-		file->read((char*)&is_rotatable, sizeof(bool));
-		setRotatable(is_rotatable);
-	}
-	file->read((char*)&line_spacing, sizeof(int));
-	if (version >= 3)
-	{
-		file->read((char*)&line_offset, sizeof(int));
-		file->read((char*)&offset_along_line, sizeof(int));
-	}
-	
-	if (type == LinePattern)
-	{
-		int color_index;
-		file->read((char*)&color_index, sizeof(int));
-		line_color = (color_index >= 0) ? map->getColor(color_index) : nullptr;
-		file->read((char*)&line_width, sizeof(int));
-	}
-	else
-	{
-		file->read((char*)&point_distance, sizeof(int));
-		bool have_point;
-		file->read((char*)&have_point, sizeof(bool));
-		if (have_point)
-		{
-			point = new PointSymbol();
-			if (!point->load(file, version, map))
-				return false;
-			if (version < 21)
-				point->setRotatable(true);
-		}
-		else
-			point = nullptr;
-	}
-	return true;
-}
-
-#endif
 
 void AreaSymbol::FillPattern::save(QXmlStreamWriter& xml, const Map& map) const
 {
@@ -147,7 +98,7 @@ void AreaSymbol::FillPattern::save(QXmlStreamWriter& xml, const Map& map) const
 	}
 }
 
-void AreaSymbol::FillPattern::load(QXmlStreamReader& xml, const Map& map, SymbolDictionary& symbol_dict)
+void AreaSymbol::FillPattern::load(QXmlStreamReader& xml, const Map& map, SymbolDictionary& symbol_dict, int version)
 {
 	Q_ASSERT (xml.name() == QLatin1String("pattern"));
 	
@@ -173,7 +124,7 @@ void AreaSymbol::FillPattern::load(QXmlStreamReader& xml, const Map& map, Symbol
 		while (xml.readNextStartElement())
 		{
 			if (xml.name() == QLatin1String("symbol"))
-				point = static_cast<PointSymbol*>(Symbol::load(xml, map, symbol_dict));
+				point = static_cast<PointSymbol*>(Symbol::load(xml, map, symbol_dict, version).release());
 			else
 				xml.skipCurrentElement();
 		}
@@ -186,7 +137,7 @@ bool AreaSymbol::FillPattern::equals(const AreaSymbol::FillPattern& other, Qt::C
 {
 	if (type != other.type)
 		return false;
-	if (qAbs(angle - other.angle) > 1e-05f)
+	if (qAbs(angle - other.angle) > 1e-05)
 		return false;
 	if (flags != other.flags)
 		return false;
@@ -241,7 +192,7 @@ void AreaSymbol::FillPattern::colorDeleted(const MapColor* color)
 	switch (type)
 	{
 	case FillPattern::PointPattern:
-		point->colorDeleted(color);
+		point->colorDeletedEvent(color);
 		break;
 	case FillPattern::LinePattern:
 		if (line_color == color)
@@ -288,7 +239,7 @@ void AreaSymbol::FillPattern::createLine<AreaSymbol::FillPattern::LinePattern>(
         MapCoordF first, MapCoordF second,
         qreal,
         LineSymbol* line,
-        float,
+        qreal,
         const AreaRenderable&,
         ObjectRenderables& output ) const
 {
@@ -303,7 +254,7 @@ void AreaSymbol::FillPattern::createLine<AreaSymbol::FillPattern::PointPattern>(
         MapCoordF first, MapCoordF second,
         qreal delta_offset,
         LineSymbol*,
-        float rotation,
+        qreal rotation,
         const AreaRenderable& outline,
         ObjectRenderables& output ) const
 {
@@ -321,15 +272,17 @@ void AreaSymbol::FillPattern::createLine<AreaSymbol::FillPattern::PointPattern>(
 template <int T>
 void AreaSymbol::FillPattern::createRenderables(
         const AreaRenderable& outline,
-        float delta_rotation,
+        qreal delta_rotation,
         const MapCoord& pattern_origin,
         const QRectF& point_extent,
         LineSymbol* line,
         qreal rotation,
         ObjectRenderables& output ) const
 {
-	auto extent = outline.getExtent();
-	extent.adjust(-point_extent.right(), -point_extent.bottom(), -point_extent.left(), -point_extent.top());
+	// Canvas is the entire rectangle which will be filled with renderables.
+	// It will be clipped by the outline, later.
+	auto canvas = outline.getExtent();
+	canvas.adjust(-point_extent.right(), -point_extent.bottom(), -point_extent.left(), -point_extent.top());
 	
 	MapCoordF first, second;
 	
@@ -357,22 +310,22 @@ void AreaSymbol::FillPattern::createRenderables(
 		// Special case: vertical lines
 		delta_along_line_offset = -delta_along_line_offset;
 		
-		double first_offset = offset + ceil((extent.left() - offset) / line_spacing_f) * line_spacing_f;
-		for (double cur = first_offset; cur < extent.right(); cur += line_spacing_f)
+		double first_offset = offset + ceil((canvas.left() - offset) / line_spacing_f) * line_spacing_f;
+		for (double cur = first_offset; cur < canvas.right(); cur += line_spacing_f)
 		{
-			first = MapCoordF(cur, extent.top());
-			second = MapCoordF(cur, extent.bottom());
+			first = MapCoordF(cur, canvas.top());
+			second = MapCoordF(cur, canvas.bottom());
 			createLine<T>(first, second, delta_along_line_offset, line, delta_rotation, outline, output);
 		}
 	}
 	else if (qAbs(rotation - 0) < 0.0001)
 	{
 		// Special case: horizontal lines
-		double first_offset = offset + ceil((extent.top() - offset) / line_spacing_f) * line_spacing_f;
-		for (double cur = first_offset; cur < extent.bottom(); cur += line_spacing_f)
+		double first_offset = offset + ceil((canvas.top() - offset) / line_spacing_f) * line_spacing_f;
+		for (double cur = first_offset; cur < canvas.bottom(); cur += line_spacing_f)
 		{
-			first = MapCoordF(extent.left(), cur);
-			second = MapCoordF(extent.right(), cur);
+			first = MapCoordF(canvas.left(), cur);
+			second = MapCoordF(canvas.right(), cur);
 			createLine<T>(first, second, delta_along_line_offset, line, delta_rotation, outline, output);
 		}
 	}
@@ -393,28 +346,28 @@ void AreaSymbol::FillPattern::createRenderables(
 		if (rotation < M_PI/2)
 		{
 			// Start with the upper left corner
-			offset_x += (-extent.top()) / tan(rotation);
-			offset_y -= extent.left() * tan(rotation);
-			auto start_x = offset_x + ceil((extent.x() - offset_x) / dist_x) * dist_x;
-			auto start_y = extent.top();
-			auto end_x = extent.left();
-			auto end_y = offset_y + ceil((extent.y() - offset_y) / dist_y) * dist_y;
+			offset_x += (-canvas.top()) / tan(rotation);
+			offset_y -= canvas.left() * tan(rotation);
+			auto start_x = offset_x + ceil((canvas.x() - offset_x) / dist_x) * dist_x;
+			auto start_y = canvas.top();
+			auto end_x = canvas.left();
+			auto end_y = offset_y + ceil((canvas.y() - offset_y) / dist_y) * dist_y;
 			
 			do
 			{
 				// Correct coordinates
-				if (start_x > extent.right())
+				if (start_x > canvas.right())
 				{
-					start_y += ((start_x - extent.right()) / dist_x) * dist_y;
-					start_x = extent.right();
+					start_y += ((start_x - canvas.right()) / dist_x) * dist_y;
+					start_x = canvas.right();
 				}
-				if (end_y > extent.bottom())
+				if (end_y > canvas.bottom())
 				{
-					end_x += ((end_y - extent.bottom()) / dist_y) * dist_x;
-					end_y = extent.bottom();
+					end_x += ((end_y - canvas.bottom()) / dist_y) * dist_x;
+					end_y = canvas.bottom();
 				}
 				
-				if (start_y > extent.bottom())
+				if (start_y > canvas.bottom())
 					break;
 				
 				// Create the renderable(s)
@@ -430,28 +383,28 @@ void AreaSymbol::FillPattern::createRenderables(
 		else
 		{
 			// Start with left lower corner
-			offset_x += (-extent.bottom()) / tan(rotation);
-			offset_y -= extent.x() * tan(rotation);
-			auto start_x = offset_x + ceil((extent.x() - offset_x) / dist_x) * dist_x;
-			auto start_y = extent.bottom();
-			auto end_x = extent.x();
-			auto end_y = offset_y + ceil((extent.bottom() - offset_y) / dist_y) * dist_y;
+			offset_x += (-canvas.bottom()) / tan(rotation);
+			offset_y -= canvas.x() * tan(rotation);
+			auto start_x = offset_x + ceil((canvas.x() - offset_x) / dist_x) * dist_x;
+			auto start_y = canvas.bottom();
+			auto end_x = canvas.x();
+			auto end_y = offset_y + ceil((canvas.bottom() - offset_y) / dist_y) * dist_y;
 			
 			do
 			{
 				// Correct coordinates
-				if (start_x > extent.right())
+				if (start_x > canvas.right())
 				{
-					start_y += ((start_x - extent.right()) / dist_x) * dist_y;
-					start_x = extent.right();
+					start_y += ((start_x - canvas.right()) / dist_x) * dist_y;
+					start_x = canvas.right();
 				}
-				if (end_y < extent.y())
+				if (end_y < canvas.y())
 				{
-					end_x += ((end_y - extent.y()) / dist_y) * dist_x;
-					end_y = extent.y();
+					end_x += ((end_y - canvas.y()) / dist_y) * dist_x;
+					end_y = canvas.y();
 				}
 				
-				if (start_y < extent.y())
+				if (start_y < canvas.y())
 					break;
 				
 				// Create the renderable(s)
@@ -468,7 +421,7 @@ void AreaSymbol::FillPattern::createRenderables(
 }
 
 
-void AreaSymbol::FillPattern::createRenderables(const AreaRenderable& outline, float delta_rotation, const MapCoord& pattern_origin, ObjectRenderables& output) const
+void AreaSymbol::FillPattern::createRenderables(const AreaRenderable& outline, qreal delta_rotation, const MapCoord& pattern_origin, ObjectRenderables& output) const
 {
 	if (line_spacing <= 0)
 		return;
@@ -477,7 +430,7 @@ void AreaSymbol::FillPattern::createRenderables(const AreaRenderable& outline, f
 		delta_rotation = 0;
 	
 	// Make rotation unique
-	auto rotation = double(angle + delta_rotation);
+	auto rotation = angle + delta_rotation;
 	rotation = fmod(1.0 * rotation, M_PI);
 	if (rotation < 0)
 		rotation = M_PI + rotation;
@@ -501,7 +454,7 @@ void AreaSymbol::FillPattern::createRenderables(const AreaRenderable& outline, f
 			line.setLineWidth(line_width_f);
 			
 			auto margin = line_width_f / 2;
-			auto point_extent = QRectF{-margin, -margin, margin, margin};
+			auto point_extent = QRectF{-margin, -margin, line_width_f, line_width_f};
 			createRenderables<LinePattern>(outline, delta_rotation, pattern_origin, point_extent, &line, rotation, output);
 		}
 		break;
@@ -524,7 +477,7 @@ void AreaSymbol::FillPattern::createRenderables(const AreaRenderable& outline, f
 void AreaSymbol::FillPattern::createPointPatternLine(
         MapCoordF first, MapCoordF second,
         qreal delta_offset,
-        float rotation,
+        qreal rotation,
         const AreaRenderable& outline,
         ObjectRenderables& output ) const
 {
@@ -616,6 +569,19 @@ AreaSymbol::AreaSymbol() noexcept
 	// nothing else
 }
 
+AreaSymbol::AreaSymbol(const AreaSymbol& proto)
+: Symbol { proto }
+, patterns { proto.patterns }
+, color { proto.color }
+, minimum_area { proto.minimum_area }
+{
+	for (auto& new_pattern : patterns)
+	{
+		if (new_pattern.type == FillPattern::PointPattern)
+			new_pattern.point = Symbol::duplicate(*new_pattern.point).release();
+	}
+}
+
 AreaSymbol::~AreaSymbol()
 {
 	for (auto& pattern : patterns)
@@ -625,23 +591,14 @@ AreaSymbol::~AreaSymbol()
 	}
 }
 
-Symbol* AreaSymbol::duplicate(const MapColorMap* color_map) const
+
+AreaSymbol* AreaSymbol::duplicate() const
 {
-	auto new_area = new AreaSymbol();
-	new_area->duplicateImplCommon(this);
-	new_area->color = color_map ? color_map->value(color) : color;
-	new_area->minimum_area = minimum_area;
-	new_area->patterns = patterns;
-	for (auto& new_pattern : new_area->patterns)
-	{
-		if (new_pattern.type == FillPattern::PointPattern)
-			new_pattern.point = static_cast<PointSymbol*>(new_pattern.point->duplicate(color_map));
-		else if (new_pattern.type == FillPattern::LinePattern && color_map)
-			new_pattern.line_color = color_map->value(new_pattern.line_color);
-	}
-	return new_area;
+	return new AreaSymbol(*this);
 }
 
+
+
 void AreaSymbol::createRenderables(
         const Object *object,
         const VirtualCoordVector &coords,
@@ -741,7 +698,7 @@ void AreaSymbol::createHatchingRenderables(
 
 
 
-void AreaSymbol::colorDeleted(const MapColor* color)
+void AreaSymbol::colorDeletedEvent(const MapColor* color)
 {
 	if (containsColor(color))
 	{
@@ -774,6 +731,24 @@ const MapColor* AreaSymbol::guessDominantColor() const
 }
 
 
+void AreaSymbol::replaceColors(const MapColorMap& color_map)
+{
+	color = color_map.value(color);
+	for (auto& pattern : patterns)
+	{
+		switch (pattern.type)
+		{
+		case FillPattern::LinePattern:
+			pattern.line_color = color_map.value(pattern.line_color);
+			break;
+		case FillPattern::PointPattern:
+			pattern.point->replaceColors(color_map);
+			break;
+		}
+	}
+}
+
+
 
 void AreaSymbol::scale(double factor)
 {
@@ -804,27 +779,7 @@ bool AreaSymbol::hasRotatableFillPattern() const
 	});
 }
 
-#ifndef NO_NATIVE_FILE_FORMAT
 
-bool AreaSymbol::loadImpl(QIODevice* file, int version, Map* map)
-{
-	int temp;
-	file->read((char*)&temp, sizeof(int));
-	color = (temp >= 0) ? map->getColor(temp) : nullptr;
-	if (version >= 2)
-		file->read((char*)&minimum_area, sizeof(int));
-	
-	int size;
-	file->read((char*)&size, sizeof(int));
-	patterns.resize(size);
-	for (int i = 0; i < size; ++i)
-		if (!patterns[i].load(file, version, map))
-			return false;
-	
-	return true;
-}
-
-#endif
 
 void AreaSymbol::saveImpl(QXmlStreamWriter& xml, const Map& map) const
 {
@@ -836,7 +791,7 @@ void AreaSymbol::saveImpl(QXmlStreamWriter& xml, const Map& map) const
 		pattern.save(xml, map);
 }
 
-bool AreaSymbol::loadImpl(QXmlStreamReader& xml, const Map& map, SymbolDictionary& symbol_dict)
+bool AreaSymbol::loadImpl(QXmlStreamReader& xml, const Map& map, SymbolDictionary& symbol_dict, int version)
 {
 	if (xml.name() != QLatin1String("area_symbol"))
 		return false;
@@ -852,7 +807,7 @@ bool AreaSymbol::loadImpl(QXmlStreamReader& xml, const Map& map, SymbolDictionar
 		if (xml.name() == QLatin1String("pattern"))
 		{
 			patterns.push_back(FillPattern());
-			patterns.back().load(xml, map, symbol_dict);
+			patterns.back().load(xml, map, symbol_dict, version);
 		}
 		else
 		{
