@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2017 Kai Pastor
+ *    Copyright 2012-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -29,7 +29,6 @@
 #include <Qt>
 #include <QtGlobal> 
 
-class QIODevice;
 class QXmlStreamReader;
 class QXmlStreamWriter;
 
@@ -51,7 +50,7 @@ class VirtualCoordVector;
  * Symbol which can combine other line and area symbols,
  * creating renderables for each of them.
  * 
- * To use, set the number of parts with setNumParts() and set the indivdual part
+ * To use, set the number of parts with setNumParts() and set the individual part
  * pointers with setPart(). Parts can be private, i.e. the CombinedSymbol owns
  * the part symbol and it is not entered in the map as an individual symbol.
  */
@@ -63,8 +62,12 @@ friend class OCAD8FileImport;
 public:
 	CombinedSymbol();
 	~CombinedSymbol() override;
-	Symbol* duplicate(const MapColorMap* color_map = nullptr) const override;
 	
+protected:
+	explicit CombinedSymbol(const CombinedSymbol& proto);
+	CombinedSymbol* duplicate() const override;
+	
+public:
 	bool validate() const override;
 	
 	void createRenderables(
@@ -79,15 +82,16 @@ public:
 	        ObjectRenderables &output,
 	        Symbol::RenderableOptions options) const override;
 	
-	void colorDeleted(const MapColor* color) override;
+	void colorDeletedEvent(const MapColor* color) override;
 	bool containsColor(const MapColor* color) const override;
 	const MapColor* guessDominantColor() const override;
-	bool symbolChanged(const Symbol* old_symbol, const Symbol* new_symbol) override;
+	void replaceColors(const MapColorMap& color_map) override;
+	bool symbolChangedEvent(const Symbol* old_symbol, const Symbol* new_symbol) override;
 	bool containsSymbol(const Symbol* symbol) const override;
 	void scale(double factor) override;
-	Type getContainedTypes() const override;
+	TypeCombination getContainedTypes() const override;
 	
-	bool loadFinished(Map* map) override;
+	bool loadingFinishedEvent(Map* map) override;
 	
 	qreal dimensionForIcon() const override;
 	
@@ -106,11 +110,8 @@ public:
 	SymbolPropertiesWidget* createPropertiesWidget(SymbolSettingDialog* dialog) override;
 	
 protected:
-#ifndef NO_NATIVE_FILE_FORMAT
-	bool loadImpl(QIODevice* file, int version, Map* map) override;
-#endif
 	void saveImpl(QXmlStreamWriter& xml, const Map& map) const override;
-	bool loadImpl(QXmlStreamReader& xml, const Map& map, SymbolDictionary& symbol_dict) override;
+	bool loadImpl(QXmlStreamReader& xml, const Map& map, SymbolDictionary& symbol_dict, int version) override;
 	bool equalsImpl(const Symbol* other, Qt::CaseSensitivity case_sensitivity) const override;
 	
 	std::vector<bool> private_parts;
