@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2017 Kai Pastor
+ *    Copyright 2012-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -21,23 +21,32 @@
 
 #include "symbol.h"
 
+#include <algorithm>
+#include <cmath>
+#include <cstddef>
+#include <iterator>
 #include <memory>
 
 #include <QtGlobal>
-#include <QIODevice>
+#include <QBuffer>
+#include <QByteArray>
+#include <QImageReader>
+#include <QImageWriter>
 #include <QLatin1Char>
 #include <QLatin1String>
 #include <QPainter>
+#include <QPoint>
+#include <QPointF>
 #include <QRectF>
 #include <QStringRef>
-#include <QXmlStreamAttributes>
+#include <QVariant>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 
+#include "settings.h"
 #include "core/map.h"
 #include "core/map_color.h"
 #include "core/map_coord.h"
-#include "core/map_view.h"
 #include "core/objects/object.h"
 #include "core/objects/text_object.h"
 #include "core/renderables/renderable.h"
@@ -49,9 +58,8 @@
 #include "core/symbols/text_symbol.h"
 #include "fileformats/file_format.h"
 #include "fileformats/file_import_export.h"
+#include "util/xml_stream_util.h"
 #include "gui/util_gui.h"
-#include "util/util.h"
-#include "settings.h"
 
 // IWYU pragma: no_include <QObject>
 
@@ -59,11 +67,27 @@
 namespace OpenOrienteering {
 
 Symbol::Symbol(Type type) noexcept
- : type { type }
- , number { -1, -1, -1 }
- , is_helper_symbol(false)
- , is_hidden(false)
- , is_protected(false)
+: number { { -1, -1, -1 } }
+, type { type }
+, is_helper_symbol { false }
+, is_hidden { false }
+, is_protected { false }
+{
+	// nothing else
+}
+
+
+Symbol::Symbol(const Symbol& proto)
+: icon { proto.icon }
+, custom_icon { proto.custom_icon }
+, name { proto.name }
+, description { proto.description }
+, number ( proto.number )  // Cannot use {} with Android gcc 4.9
+, type { proto.type }
+, is_helper_symbol { proto.is_helper_symbol }
+, is_hidden { proto.is_hidden }
+, is_protected { proto.is_protected }
+, is_rotatable { proto.is_rotatable }
 {
 	// nothing else
 }
@@ -80,31 +104,26 @@ bool Symbol::validate() const
 
 
 
-bool Symbol::equals(const Symbol* other, Qt::CaseSensitivity case_sensitivity, bool compare_state) const
+bool Symbol::equals(const Symbol* other, Qt::CaseSensitivity case_sensitivity) const
 {
-	if (type != other->type)
-		return false;
-	for (int i = 0; i < number_components; ++i)
-	{
-		if (number[i] != other->number[i])
-			return false;
-		if (number[i] == -1 && other->number[i] == -1)
-			break;
-	}
-	if (is_helper_symbol != other->is_helper_symbol)
-		return false;
-	
-	if (compare_state && (is_hidden != other->is_hidden || is_protected != other->is_protected))
-		return false;
-	
-	if (name.compare(other->name, case_sensitivity) != 0)
-		return false;
-	if (description.compare(other->description, case_sensitivity) != 0)
-		return false;
-	
-	return equalsImpl(other, case_sensitivity);
+	return type == other->type
+	       && numberEquals(other)
+	       && is_helper_symbol == other->is_helper_symbol
+	       && is_rotatable == other->is_rotatable
+	       && name.compare(other->name, case_sensitivity) == 0
+	       && description.compare(other->description, case_sensitivity) == 0
+	       && equalsImpl(other, case_sensitivity);
 }
 
+
+bool Symbol::stateEquals(const Symbol* other) const
+{
+	return is_hidden == other->is_hidden
+	       && is_protected == other->is_protected;
+}
+
+
+
 const PointSymbol* Symbol::asPoint() const
 {
 	Q_ASSERT(type == Point);
@@ -156,149 +175,203 @@ CombinedSymbol* Symbol::asCombined()
 	return static_cast<CombinedSymbol*>(this);
 }
 
+
+
+Symbol::TypeCombination Symbol::getContainedTypes() const
+{
+	return getType();
+}
+
+
 bool Symbol::isTypeCompatibleTo(const Object* object) const
 {
-	if (type == Point && object->getType() == Object::Point)
-		return true;
-	else if ((type == Line || type == Area || type == Combined) && object->getType() == Object::Path)
-		return true;
-	else if (type == Text && object->getType() == Object::Text)
-		return true;
-
+	switch (object->getType())
+	{
+	case Object::Point:
+		return type == Point;
+	case Object::Path:
+		return type == Line || type == Area || type == Combined;
+	case Object::Text:
+		return type == Text;
+	}
 	return false;
 }
 
-bool Symbol::numberEquals(const Symbol* other, bool ignore_trailing_zeros) const
+
+
+bool Symbol::numberEquals(const Symbol* other) const
 {
-	if (ignore_trailing_zeros)
+	for (auto lhs = begin(number), rhs = begin(other->number); lhs != end(number); ++lhs, ++rhs)
 	{
-		for (int i = 0; i < number_components; ++i)
-		{
-			if (number[i] == -1 && other->number[i] == -1)
-				return true;
-			if ((number[i] == 0 || number[i] == -1) &&
-				(other->number[i] == 0 || other->number[i] == -1))
-				continue;
-			if (number[i] != other->number[i])
-				return false;
-		}
-	}
-	else
-	{
-		for (int i = 0; i < number_components; ++i)
-		{
-			if (number[i] != other->number[i])
-				return false;
-			if (number[i] == -1)
-				return true;
-		}
+		if (*lhs != *rhs)
+			return false;
+		if (*lhs == -1 && *rhs == -1)
+			break;
 	}
 	return true;
 }
 
-#ifndef NO_NATIVE_FILE_FORMAT
 
-bool Symbol::load(QIODevice* file, int version, Map* map)
+bool Symbol::numberEqualsRelaxed(const Symbol* other) const
 {
-	loadString(file, name);
-	for (int i = 0; i < number_components; ++i)
-		file->read((char*)&number[i], sizeof(int));
-	loadString(file, description);
-	file->read((char*)&is_helper_symbol, sizeof(bool));
-	if (version >= 10)
-		file->read((char*)&is_hidden, sizeof(bool));
-	if (version >= 11)
-		file->read((char*)&is_protected, sizeof(bool));
-	
-	return loadImpl(file, version, map);
+	for (auto lhs = begin(number), rhs = begin(other->number); lhs != end(number) && rhs != end(other->number); ++lhs, ++rhs)
+	{
+		// When encountering -1 on one side and 0 on the other side,
+		// move forward over all zeros on the other side.
+		if (Q_UNLIKELY(*lhs == -1 && *rhs == 0))
+		{
+			do
+			{
+				++rhs;
+				if (rhs == end(other->number))
+					return true;
+			}
+			while (*rhs == 0);
+		}
+		else if (Q_UNLIKELY(*lhs == 0 && *rhs == -1))
+		{
+			do
+			{
+				++lhs;
+				if (lhs == end(number))
+					return true;
+			}
+			while (*lhs == 0);
+		}
+		
+		if (*lhs != *rhs)
+			return false;
+		if (*lhs == -1 && *rhs == -1)
+			break;
+	}
+	return true;
 }
 
-#endif
+
 
 void Symbol::save(QXmlStreamWriter& xml, const Map& map) const
 {
-	xml.writeStartElement(QString::fromLatin1("symbol"));
-	xml.writeAttribute(QString::fromLatin1("type"), QString::number(type));
-	int id = map.findSymbolIndex(this);
+	XmlElementWriter symbol_element(xml, QLatin1String("symbol"));
+	symbol_element.writeAttribute(QLatin1String("type"), int(type));
+	auto id = map.findSymbolIndex(this);
 	if (id >= 0)
-		xml.writeAttribute(QString::fromLatin1("id"), QString::number(id)); // unique if given
-	xml.writeAttribute(QString::fromLatin1("code"), getNumberAsString()); // not always unique
+		symbol_element.writeAttribute(QLatin1String("id"), id); // unique if given
+	symbol_element.writeAttribute(QLatin1String("code"), getNumberAsString()); // not always unique
 	if (!name.isEmpty())
-		xml.writeAttribute(QString::fromLatin1("name"), name);
-	if (is_helper_symbol)
-		xml.writeAttribute(QString::fromLatin1("is_helper_symbol"), QString::fromLatin1("true"));
-	if (is_hidden)
-		xml.writeAttribute(QString::fromLatin1("is_hidden"), QString::fromLatin1("true"));
-	if (is_protected)
-		xml.writeAttribute(QString::fromLatin1("is_protected"), QString::fromLatin1("true"));
+		symbol_element.writeAttribute(QLatin1String("name"), name);
+	symbol_element.writeAttribute(QLatin1String("is_helper_symbol"), is_helper_symbol);
+	symbol_element.writeAttribute(QLatin1String("is_hidden"), is_hidden);
+	symbol_element.writeAttribute(QLatin1String("is_protected"), is_protected);
 	if (!description.isEmpty())
-		xml.writeTextElement(QString::fromLatin1("description"), description);
+		xml.writeTextElement(QLatin1String("description"), description);
 	saveImpl(xml, map);
-	xml.writeEndElement(/*symbol*/);
+	if (!custom_icon.isNull())
+	{
+		QBuffer buffer;
+		QImageWriter writer{&buffer, QByteArrayLiteral("PNG")};
+		if (writer.write(custom_icon))
+		{
+			auto data = buffer.data().toBase64();
+			// The "data" URL scheme, RFC2397 (https://tools.ietf.org/html/rfc2397)
+			data.insert(0, "data:image/png;base64,");
+			xml.writeCharacters(QLatin1String("\n"));
+			XmlElementWriter icon_element(xml, QLatin1String("icon"));
+			icon_element.writeAttribute(QLatin1String("src"), QString::fromLatin1(data));
+		}
+		else
+		{
+			qDebug("Couldn't save symbol icon '%s': %s",
+			       qPrintable(getPlainTextName()),
+			       qPrintable(writer.errorString()) );
+		}
+	}
 }
 
-Symbol* Symbol::load(QXmlStreamReader& xml, const Map& map, SymbolDictionary& symbol_dict)
+std::unique_ptr<Symbol> Symbol::load(QXmlStreamReader& xml, const Map& map, SymbolDictionary& symbol_dict, int version)
 {
 	Q_ASSERT(xml.name() == QLatin1String("symbol"));
-	
-	int symbol_type = xml.attributes().value(QLatin1String("type")).toInt();
-	Symbol* symbol = Symbol::getSymbolForType(static_cast<Symbol::Type>(symbol_type));
+	XmlElementReader symbol_element(xml);
+	auto symbol_type = symbol_element.attribute<int>(QLatin1String("type"));
+	auto symbol = Symbol::makeSymbolForType(static_cast<Symbol::Type>(symbol_type));
 	if (!symbol)
 		throw FileFormatException(::OpenOrienteering::ImportExport::tr("Error while loading a symbol of type %1 at line %2 column %3.").arg(symbol_type).arg(xml.lineNumber()).arg(xml.columnNumber()));
 	
-	QXmlStreamAttributes attributes = xml.attributes();
-	QString code = attributes.value(QLatin1String("code")).toString();
-	if (attributes.hasAttribute(QLatin1String("id")))
+	auto code = symbol_element.attribute<QString>(QLatin1String("code"));
+	if (symbol_element.hasAttribute(QLatin1String("id")))
 	{
-		QString id = attributes.value(QLatin1String("id")).toString();
+		auto id = symbol_element.attribute<QString>(QLatin1String("id"));
 		if (symbol_dict.contains(id)) 
 			throw FileFormatException(::OpenOrienteering::ImportExport::tr("Symbol ID '%1' not unique at line %2 column %3.").arg(id).arg(xml.lineNumber()).arg(xml.columnNumber()));
 		
-		symbol_dict[id] = symbol;
+		symbol_dict[id] = symbol.get();  // Will be dangling pointer when we throw an exception later
 		
 		if (code.isEmpty())
 			code = id;
 	}
 	
-	if (code.isEmpty())
-		symbol->number[0] = -1;
-	else
+	std::fill(begin(symbol->number), end(symbol->number), -1);
+	if (!code.isEmpty())
 	{
-		for (int i = 0, index = 0; i < number_components && index >= 0; ++i)
+		int pos = 0;
+		for (auto i = 0u; i < number_components; ++i)
 		{
-			if (index == -1)
-				symbol->number[i] = -1;
-			else
-			{
-				int dot = code.indexOf(QLatin1Char('.'), index+1);
-				int num = code.midRef(index, (dot == -1) ? -1 : (dot - index)).toInt();
-				symbol->number[i] = num;
-				index = dot;
-				if (index != -1)
-					index++;
-			}
+			int dot = code.indexOf(QLatin1Char('.'), pos+1);
+			symbol->number[i] = code.midRef(pos, (dot == -1) ? -1 : (dot - pos)).toInt();
+			pos = ++dot;
+			if (pos < 1)
+				break;
 		}
 	}
 	
-	symbol->name = attributes.value(QLatin1String("name")).toString();
-	symbol->is_helper_symbol = (attributes.value(QLatin1String("is_helper_symbol")) == QLatin1String("true"));
-	symbol->is_hidden = (attributes.value(QLatin1String("is_hidden")) == QLatin1String("true"));
-	symbol->is_protected = (attributes.value(QLatin1String("is_protected")) == QLatin1String("true"));
+	symbol->name = symbol_element.attribute<QString>(QLatin1String("name"));
+	symbol->is_helper_symbol = symbol_element.attribute<bool>(QLatin1String("is_helper_symbol"));
+	symbol->is_hidden = symbol_element.attribute<bool>(QLatin1String("is_hidden"));
+	symbol->is_protected = symbol_element.attribute<bool>(QLatin1String("is_protected"));
 	
 	while (xml.readNextStartElement())
 	{
 		if (xml.name() == QLatin1String("description"))
+		{
 			symbol->description = xml.readElementText();
+		}
+		else if (xml.name() == QLatin1String("icon"))
+		{
+			XmlElementReader icon_element(xml);
+			auto data = icon_element.attribute<QStringRef>(QLatin1String("src")).toLatin1();
+			
+			// The "data" URL scheme, RFC2397 (https://tools.ietf.org/html/rfc2397)
+			auto start = data.indexOf(',') + 1;
+			if (start > 0 && data.startsWith("data:image/"))
+			{
+				auto base64 = data.indexOf(";base64");
+				data = data.remove(0, start);
+				if (base64 + 8 == start)
+					data = QByteArray::fromBase64(data);
+			}
+			else
+			{
+				data.clear();  // Ignore unknown data, warning is generated later.
+			}
+			
+			QBuffer buffer{&data};
+			QImageReader reader{&buffer, QByteArrayLiteral("PNG")};
+			auto icon = reader.read();
+			if (!icon.isNull())
+				symbol->setCustomIcon(icon);
+			else
+				qDebug("Couldn't load symbol icon '%s': %s",
+				       qPrintable(symbol->getPlainTextName()),
+				       qPrintable(reader.errorString()) );
+		}
 		else
 		{
-			if (!symbol->loadImpl(xml, map, symbol_dict))
+			if (!symbol->loadImpl(xml, map, symbol_dict, version))
 				xml.skipCurrentElement();
 		}
 	}
 	
 	if (xml.error())
 	{
-		delete symbol;
 		throw FileFormatException(
 		            ::OpenOrienteering::ImportExport::tr("Error while loading a symbol of type %1 at line %2 column %3: %4")
 		            .arg(symbol_type)
@@ -310,19 +383,27 @@ Symbol* Symbol::load(QXmlStreamReader& xml, const Map& map, SymbolDictionary& sy
 	return symbol;
 }
 
-bool Symbol::loadFinished(Map* map)
+
+
+bool Symbol::loadingFinishedEvent(Map* /*map*/)
 {
-	Q_UNUSED(map);
 	return true;
 }
 
-void Symbol::createRenderables(const PathObject*, const PathPartVector&, ObjectRenderables&, Symbol::RenderableOptions) const
+
+
+void Symbol::createRenderables(
+        const PathObject* /*object*/,
+        const PathPartVector& /*path_parts*/,
+        ObjectRenderables& /*output*/,
+        Symbol::RenderableOptions /*options*/) const
 {
-	qWarning("Missing implementation of Symbol::createRenderables(const PathObject*, const PathPartVector&, ObjectRenderables&)");
+	qWarning("Missing implementation of Symbol::createRenderables(const PathObject*, const PathPartVector&, ObjectRenderables&, Symbol::RenderableOptions)");
 }
 
+
 void Symbol::createBaselineRenderables(
-        const PathObject*,
+        const PathObject* /*object*/,
         const PathPartVector& path_parts,
         ObjectRenderables& output,
         const MapColor* color) const
@@ -337,33 +418,48 @@ void Symbol::createBaselineRenderables(
 		line_symbol.setLineWidth(0);
 		for (const auto& part : path_parts)
 		{
-			auto line_renderable = new LineRenderable(&line_symbol, part, false);
-			output.insertRenderable(line_renderable);
+			output.insertRenderable(new LineRenderable(&line_symbol, part, false));
 		}
 	}
 }
 
-bool Symbol::symbolChanged(const Symbol* old_symbol, const Symbol* new_symbol)
+
+
+bool Symbol::symbolChangedEvent(const Symbol* /*old_symbol*/, const Symbol* /*new_symbol*/)
 {
-	Q_UNUSED(old_symbol);
-	Q_UNUSED(new_symbol);
 	return false;
 }
 
-bool Symbol::containsSymbol(const Symbol* symbol) const
+
+bool Symbol::containsSymbol(const Symbol* /*symbol*/) const
 {
-	Q_UNUSED(symbol);
 	return false;
 }
 
+
+
+void Symbol::setCustomIcon(const QImage& image)
+{
+	resetIcon();  // Cache must become scaled version of custom icon.
+	custom_icon = image;
+}
+
+
 QImage Symbol::getIcon(const Map* map) const
 {
-	if (icon.isNull() && map)
-		icon = createIcon(*map, Settings::getInstance().getSymbolWidgetIconSizePx());
-	
+	if (icon.isNull())
+	{
+		auto size = Settings::getInstance().getSymbolWidgetIconSizePx();
+		if (Settings::getInstance().getSetting(Settings::SymbolWidget_ShowCustomIcons).toBool()
+		    && !custom_icon.isNull())
+			icon = custom_icon.scaled(size, size, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+		else if (map)
+			icon = createIcon(*map, size);
+	}
 	return icon;
 }
 
+
 QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qreal zoom) const
 {
 	// Desktop default used to be 2x zoom at 8 mm side length, plus/minus
@@ -421,13 +517,16 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 		if (type == Line)
 		{
 			auto line = static_cast<const LineSymbol*>(this);
-			if (line->getCapStyle() == LineSymbol::RoundCap)
+			if (!line->isDashed() || line->getBreakLength() <= 0)
 			{
-				offset.setNativeX(-line->getLineWidth()/3);
-			}
-			else if (line->getCapStyle() == LineSymbol::PointedCap)
-			{
-				line_length_half = std::max(line_length_half, 0.0012 * line->getPointedCapLength());
+				if (line->getCapStyle() == LineSymbol::RoundCap)
+				{
+					offset.setNativeX(-line->getLineWidth()/3);
+				}
+				else if (line->getCapStyle() == LineSymbol::PointedCap)
+				{
+					line_length_half = std::max(line_length_half, 0.0006 * (line->startOffset() + line->endOffset()));
+				}
 			}
 			
 			if (line->getDashSymbol() && !line->getDashSymbol()->isEmpty())
@@ -445,7 +544,7 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 			    if (!line->getShowAtLeastOneSymbol())
 				{
 					if (!symbol_copy)
-						symbol_copy.reset(line->duplicate());
+						symbol_copy = duplicate(*line);
 					auto icon_line = static_cast<LineSymbol*>(symbol_copy.get());
 					icon_line->setShowAtLeastOneSymbol(true);
 				}
@@ -467,7 +566,7 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 			{
 				auto ideal_length = 2 * line->getDashesInGroup() * line->getDashLength()
 				                    + 2 * (line->getDashesInGroup() - 1) * line->getInGroupBreakLength()
-				                    + line->getBreakLength();
+				                    + 3 * line->getBreakLength() / 2;
 				if (max_ideal_length < ideal_length)
 					max_ideal_length = ideal_length;
 			}
@@ -490,21 +589,38 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 			}
 			if (max_ideal_length > 0)
 			{
+				auto cap_length = 0;
+				auto offset_factor = qreal(0);
 				auto ideal_length_half = qreal(max_ideal_length) / 2000;
+				
+				if (line->getCapStyle() == LineSymbol::PointedCap)
+				{
+					offset_factor = 1;
+					ideal_length_half += qreal(line->startOffset() + line->endOffset()) / 2000;
+				}
+				else if (line->getCapStyle() != LineSymbol::FlatCap)
+				{
+					cap_length = line->getLineWidth();
+					ideal_length_half += qreal(cap_length) / 1000;
+				}
+				
 				auto factor = qMin(qreal(0.5), line_length_half / qMax(qreal(0.001), ideal_length_half));
+				offset_factor *= factor;
 				
 				if (!symbol_copy)
-					symbol_copy.reset(line->duplicate());
+					symbol_copy = duplicate(*line);
 				
 				auto icon_line = static_cast<LineSymbol*>(symbol_copy.get());
 				icon_line->setDashLength(qRound(factor * icon_line->getDashLength()));
-				icon_line->setBreakLength(qRound(factor * icon_line->getBreakLength()));
+				icon_line->setBreakLength(cap_length + qRound(factor * (icon_line->getBreakLength() - cap_length)));
 				icon_line->setInGroupBreakLength(qRound(factor * icon_line->getInGroupBreakLength()));
 				icon_line->setShowAtLeastOneSymbol(true);
 				icon_line->getBorder().dash_length *= factor;
 				icon_line->getBorder().break_length *= factor;
 				icon_line->getRightBorder().dash_length *= factor;
 				icon_line->getRightBorder().break_length *= factor;
+				icon_line->setStartOffset(qRound(offset_factor * icon_line->startOffset()));
+				icon_line->setEndOffset(qRound(offset_factor * icon_line->endOffset()));
 			}
 		}
 		else if (type == Combined)
@@ -563,10 +679,10 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 					if (!proto)
 						continue;
 					
-					auto copy = proto->duplicate();
+					auto copy = duplicate(*proto);
 					if (copy->getType() == Line)
 					{
-						auto icon_line = static_cast<LineSymbol*>(copy);
+						auto icon_line = static_cast<LineSymbol*>(copy.get());
 						icon_line->setDashLength(qRound(factor * icon_line->getDashLength()));
 						icon_line->setBreakLength(qRound(factor * icon_line->getBreakLength()));
 						icon_line->setInGroupBreakLength(qRound(factor * icon_line->getInGroupBreakLength()));
@@ -576,7 +692,7 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 						icon_line->getRightBorder().dash_length *= factor;
 						icon_line->getRightBorder().break_length *= factor;
 					}
-					static_cast<CombinedSymbol*>(symbol_copy.get())->setPart(i, copy, true);
+					static_cast<CombinedSymbol*>(symbol_copy.get())->setPart(i, copy.release(), true);
 				}
 			}
 		}
@@ -609,6 +725,9 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 	auto w = std::max(std::abs(extent.left()), std::abs(extent.right()));
 	auto h = std::max(std::abs(extent.top()), std::abs(extent.bottom()));
 	auto real_icon_mm_half = std::max(w, h);
+	if (real_icon_mm_half <= 0)
+		return image;
+	
 	auto final_zoom = side_length * zoom * std::min(qreal(1), max_icon_mm_half / real_icon_mm_half);
 	painter.scale(final_zoom, final_zoom);
 	
@@ -627,16 +746,19 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 		painter.translate(MapCoordF(-offset));
 	}
 	
-	RenderConfig config = { map, QRectF(-10000, -10000, 20000, 20000), final_zoom, RenderConfig::HelperSymbols, 1.0 };
-	bool was_hidden = is_hidden;
 	// Ensure that an icon is created for hidden symbols.
+	if (is_hidden && !symbol_copy)
+	{
+		symbol_copy = duplicate(*this);
+		object->setSymbol(symbol_copy.get(), true);
+	}
 	if (symbol_copy)
-		symbol_copy.get()->setHidden(false);
-	else
-		is_hidden = false;
-	icon_map.draw(&painter, config);
-	is_hidden = was_hidden;
+	{
+		symbol_copy->setHidden(false);
+	}
 	
+	auto config = RenderConfig { map, QRectF(-10000, -10000, 20000, 20000), final_zoom, RenderConfig::HelperSymbols, 1.0 };
+	icon_map.draw(&painter, config);
 	painter.end();
 	
 	// Add shadow to dominant white on transparent
@@ -701,99 +823,92 @@ qreal Symbol::calculateLargestLineExtent() const
 
 
 
+QString Symbol::getPlainTextName() const
+{
+	return Util::plainText(name);
+}
+
+
+
 QString Symbol::getNumberAsString() const
 {
-	QString str;
+	QString string;
+	string.reserve(4 * number_components - 1);
 	for (auto n : number)
 	{
 		if (n < 0)
 			break;
-		if (!str.isEmpty())
-			str += QLatin1Char('.');
-		str += QString::number(n);
+		string.append(QString::number(n));
+		string.append(QLatin1Char('.'));
 	}
-	return str;
+	string.chop(1);
+	return string;
 }
 
-Symbol* Symbol::getSymbolForType(Symbol::Type type)
+
+
+void Symbol::setRotatable(bool value)
 {
-	if (type == Symbol::Point)
-		return new PointSymbol();
-	else if (type == Symbol::Line)
-		return new LineSymbol();
-	else if (type == Symbol::Area)
-		return new AreaSymbol();
-	else if (type == Symbol::Text)
-		return new TextSymbol();
-	else if (type == Symbol::Combined)
-		return new CombinedSymbol();
-	else
+	is_rotatable = value;
+}
+
+
+
+std::unique_ptr<Symbol> Symbol::makeSymbolForType(Symbol::Type type)
+{
+	switch (type)
 	{
-		Q_ASSERT(false);
-		return nullptr;
+	case Area:
+		return std::make_unique<AreaSymbol>();
+	case Combined:
+		return std::make_unique<CombinedSymbol>();
+	case Line:
+		return std::make_unique<LineSymbol>();
+	case Point:
+		return std::make_unique<PointSymbol>();
+	case Text:
+		return std::make_unique<TextSymbol>();
+	default:
+		return std::unique_ptr<Symbol>();
 	}
 }
 
-#ifndef NO_NATIVE_FILE_FORMAT
 
-bool Symbol::loadSymbol(Symbol*& symbol, QIODevice* stream, int version, Map* map)
-{
-	int save_type;
-	stream->read((char*)&save_type, sizeof(int));
-	symbol = Symbol::getSymbolForType(static_cast<Symbol::Type>(save_type));
-	if (!symbol)
-		return false;
-	if (!symbol->load(stream, version, map))
-		return false;
-	return true;
-}
-
-#endif
 
 bool Symbol::areTypesCompatible(Symbol::Type a, Symbol::Type b)
 {
 	return (getCompatibleTypes(a) & b) != 0;
 }
 
-int Symbol::getCompatibleTypes(Symbol::Type type)
-{
-	if (type == Point)
-		return Point;
-	else if (type == Line || type == Area || type == Combined)
-		return Line | Area | Combined;
-	else if (type == Text)
-		return Text;
-	
-	Q_ASSERT(false);
-	return type;
-}
 
-void Symbol::duplicateImplCommon(const Symbol* other)
+Symbol::TypeCombination Symbol::getCompatibleTypes(Symbol::Type type)
 {
-	type = other->type;
-	name = other->name;
-	for (int i = 0; i < number_components; ++i)
-		number[i] = other->number[i];
-	description = other->description;
-	is_helper_symbol = other->is_helper_symbol;
-	is_hidden = other->is_hidden;
-	icon = other->icon;
-}
-
-
-bool Symbol::compareByNumber(const Symbol* s1, const Symbol* s2)
-{
-	int n1 = s1->number_components, n2 = s2->number_components;
-	for (int i = 0; i < n1 && i < n2; i++)
+	switch (type)
 	{
-		if (s1->getNumberComponent(i) < s2->getNumberComponent(i)) return true;  // s1 < s2
-		if (s1->getNumberComponent(i) > s2->getNumberComponent(i)) return false; // s1 > s2
-		// if s1 == s2, loop to the next component
+	case Area:
+	case Combined:
+	case Line:
+		return Line | Area | Combined;
+	case Point:
+	case Text:
+	default:
+		return type;
+	}
+}
+
+
+
+bool Symbol::lessByNumber(const Symbol* s1, const Symbol* s2)
+{
+	for (auto i = 0u; i < number_components; i++)
+	{
+		if (s1->number[i] < s2->number[i]) return true;  // s1 < s2
+		if (s1->number[i] > s2->number[i]) return false; // s1 > s2
 	}
 	return false; // s1 == s2
 }
 
-bool Symbol::compareByColorPriority(const Symbol* s1, const Symbol* s2)
+bool Symbol::lessByColorPriority(const Symbol* s1, const Symbol* s2)
 {
 	const MapColor* c1 = s1->guessDominantColor();
 	const MapColor* c2 = s2->guessDominantColor();
@@ -802,40 +917,43 @@ bool Symbol::compareByColorPriority(const Symbol* s1, const Symbol* s2)
 		return c1->comparePriority(*c2);
 	else if (c2)
 		return true;
-	else if (c1)
+	else
+		return false;
+}
+
+
+
+Symbol::lessByColor::lessByColor(const Map* map)
+{
+	colors.reserve(std::size_t(map->getNumColors()));
+	for (int i = 0; i < map->getNumColors(); ++i)
+		colors.push_back(QRgb(*map->getColor(i)));
+}
+
+
+bool Symbol::lessByColor::operator() (const Symbol* s1, const Symbol* s2) const
+{
+	auto c2 = s2->guessDominantColor();
+	if (!c2)
 		return false;
 	
-	return false; // s1 == s2
-}
-
-Symbol::compareByColor::compareByColor(Map* map)
-{
-	int next_priority = map->getNumColors() - 1;
-	// Iterating in reverse order so identical colors are at the position where they appear with lowest priority.
-	for (int i = map->getNumColors() - 1; i >= 0; --i)
-	{
-		QRgb color_code = QRgb(*map->getColor(i));
-		if (!color_map.contains(color_code))
-		{
-			color_map.insert(color_code, next_priority);
-			--next_priority;
-		}
-	}
-}
-
-bool Symbol::compareByColor::operator() (const Symbol* s1, const Symbol* s2)
-{
-	const MapColor* c1 = s1->guessDominantColor();
-	const MapColor* c2 = s2->guessDominantColor();
-	
-	if (c1 && c2)
-		return color_map.value(QRgb(*c1)) < color_map.value(QRgb(*c2));
-	else if (c2)
+	auto c1 = s1->guessDominantColor();
+	if (!c1)
 		return true;
-	else if (c1)
+	
+	const auto rgb_c1 = QRgb(*c1);
+	const auto rgb_c2 = QRgb(*c2);
+	if (rgb_c1 == rgb_c2)
 		return false;
 	
-	return false; // s1 == s2
+	const auto last = colors.rend();
+	auto first = std::find_if(colors.rbegin(), last, [rgb_c2](const auto rgb) {
+		return rgb == rgb_c2;
+	});
+	auto second = std::find_if(first, last, [rgb_c1](const auto rgb) {
+		return rgb == rgb_c1;
+	});
+	return second != last;
 }
 
 
