@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Pete Curtis
- *    Copyright 2012-2015  Kai Pastor
+ *    Copyright 2012-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -21,6 +21,8 @@
 #ifndef OPENORIENTEERING_FILE_FORMAT_XML_P_H
 #define OPENORIENTEERING_FILE_FORMAT_XML_P_H
 
+#include <functional>
+
 #include <QCoreApplication>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
@@ -36,12 +38,18 @@ class XMLFileExporter : public Exporter
 	Q_DECLARE_TR_FUNCTIONS(OpenOrienteering::XMLFileExporter)
 	
 public:
-	XMLFileExporter(QIODevice* stream, Map *map, MapView *view);
-	~XMLFileExporter() override {}
+	XMLFileExporter(const QString& path, const Map* map, const MapView* view);
+	XMLFileExporter() = delete;
+	XMLFileExporter(const XMLFileExporter&) = delete;
+	XMLFileExporter(XMLFileExporter&&) = delete;
+	~XMLFileExporter() override;
 	
-	void doExport() override;
+	XMLFileExporter& operator=(const XMLFileExporter&) = delete;	
+	XMLFileExporter& operator=(XMLFileExporter&&) = delete;	
 	
 protected:
+	bool exportImplementation() override;
+	
 	void exportGeoreferencing();
 	void exportColors();
 	void exportSymbols();
@@ -63,17 +71,25 @@ class XMLFileImporter : public Importer
 	Q_DECLARE_TR_FUNCTIONS(OpenOrienteering::XMLFileImporter)
 	
 public:
-	XMLFileImporter(QIODevice* stream, Map *map, MapView *view);
-	~XMLFileImporter() override {}
-
-protected:
-	void import(bool load_symbols_only) override;
+	XMLFileImporter(const QString& path, Map *map, MapView *view);
+	XMLFileImporter() = delete;
+	XMLFileImporter(const XMLFileImporter&) = delete;
+	XMLFileImporter(XMLFileImporter&&) = delete;
+	~XMLFileImporter() override;
 	
-	void importElements(bool load_symbols_only);
+	XMLFileImporter& operator=(const XMLFileImporter&) = delete;	
+	XMLFileImporter& operator=(XMLFileImporter&&) = delete;	
+	
+protected:
+	bool importImplementation() override;
+	
+	void importElements();
+	
+	void handleBarrier(const std::function<void()>& reader);
 	
 	void addWarningUnsupportedElement();
 	void importMapNotes();
-	void importGeoreferencing(bool load_symbols_only);
+	void importGeoreferencing();
 	void importColors();
 	void importSymbols();
 	void importMapParts();
@@ -83,8 +99,10 @@ protected:
 	void importUndo();
 	void importRedo();
 	
+private:
 	QXmlStreamReader xml;
 	SymbolDictionary symbol_dict;
+	int version = -1;
 	bool georef_offset_adjusted;
 };
 
