@@ -1,5 +1,5 @@
 /*
- *    Copyright 2016-2017 Kai Pastor
+ *    Copyright 2016-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -19,9 +19,12 @@
 
 #include "gdal_manager.h"
 
+#include <cstddef>
+// IWYU pragma: no_include <type_traits>
+
 #include <cpl_conv.h>
-#include <gdal.h> // IWYU pragma: keep
-#include <ogr_api.h>
+#include <gdal.h>
+// IWYU pragma: no_include <gdal_version.h>
 
 #include <QtGlobal>
 #include <QByteArray>
@@ -33,7 +36,7 @@
 #include <QStringList>
 #include <QVariant>
 
-#include "util/backports.h"
+#include "util/backports.h"  // IWYU pragma: keep
 
 
 namespace OpenOrienteering {
@@ -43,18 +46,31 @@ class GdalManager::GdalManagerPrivate
 public:
 	const QString gdal_manager_group{ QStringLiteral("GdalManager") };
 	const QString gdal_configuration_group{ QStringLiteral("GdalConfiguration") };
-	const QString gdal_dxf_key{ QStringLiteral("dxf") };
+
+	// Enabled formats
 	const QString gdal_gpx_key{ QStringLiteral("gpx") };
-	const QString gdal_osm_key{ QStringLiteral("osm") };
+	
+	// Template display options
+	const QString gdal_hatch_key{ QStringLiteral("area_hatching") };
+	const QString gdal_baseline_key{ QStringLiteral("baseline_view") };
+	
+	// Export options
+	const QString ogr_one_layer_per_symbol_key{ QStringLiteral("per_symbol_layer") };
 	
 	GdalManagerPrivate()
 	: dirty{ true }
 	{
-		// GDAL 2.0: GDALAllRegister();
-		OGRRegisterAll();
+		GDALAllRegister();
+
+		// Prefer LIBMKL driver to the KML driver if available
+		if (GDALGetDriverByName("LIBKML") != nullptr)
+			GDALDeregisterDriver(GDALGetDriverByName("KML"));
 	}
 	
 	GdalManagerPrivate(const GdalManagerPrivate&) = delete;
+	GdalManagerPrivate(GdalManagerPrivate&&) = delete;
+	GdalManagerPrivate& operator=(const GdalManagerPrivate&) = delete;
+	GdalManagerPrivate&& operator=(GdalManagerPrivate&&) = delete;
 	
 	void configure()
 	{
@@ -62,21 +78,57 @@ public:
 			update();
 	}
 	
+	
+	QVariant settingsValue(const QString& key, const QVariant& default_value) const
+	{
+		QSettings settings;
+		settings.beginGroup(gdal_manager_group);
+		return settings.value(key, default_value);
+	}
+
+	void setSettingsValue(const QString& key, const QVariant& value)
+	{
+		QSettings settings;
+		settings.beginGroup(gdal_manager_group);
+		settings.setValue(key, value);
+	}
+	
+	
 	void setFormatEnabled(GdalManager::FileFormat format, bool enabled)
 	{
 		QString key;
 		switch (format)
 		{
-		case GdalManager::DXF:
-			key = gdal_dxf_key;
-			break;
-			
 		case GdalManager::GPX:
 			key = gdal_gpx_key;
 			break;
 			
-		case GdalManager::OSM:
-			key = gdal_osm_key;
+		}
+		setSettingsValue(key, enabled);
+		dirty = true;
+	}
+	
+	bool isFormatEnabled(FileFormat format) const
+	{
+		QString key;
+		bool default_value = true;
+		switch (format)
+		{
+		case GdalManager::GPX:
+			key = gdal_gpx_key;
+			default_value = false;
+			break;
+		}
+		return settingsValue(key, default_value).toBool();
+	}
+
+	void setExportOptionEnabled(GdalManager::ExportOption option, bool enabled)
+	{
+		QString key;
+		switch (option)
+		{
+		case GdalManager::OneLayerPerSymbol:
+			key = ogr_one_layer_per_symbol_key;
 			break;
 		}
 		QSettings settings;
@@ -84,27 +136,19 @@ public:
 		settings.setValue(key, QVariant{ enabled });
 		dirty = true;
 	}
-	
-	bool isFormatEnabled(FileFormat format) const
+
+	bool isExportOptionEnabled(GdalManager::ExportOption option) const
 	{
 		QString key;
-		switch (format)
+		switch (option)
 		{
-		case GdalManager::DXF:
-			key = gdal_dxf_key;
-			break;
-			
-		case GdalManager::GPX:
-			key = gdal_gpx_key;
-			break;
-			
-		case GdalManager::OSM:
-			key = gdal_osm_key;
+		case GdalManager::OneLayerPerSymbol:
+			key = ogr_one_layer_per_symbol_key;
 			break;
 		}
 		QSettings settings;
 		settings.beginGroup(gdal_manager_group);
-		return !settings.contains(key) || settings.value(key).toBool();
+		return settings.value(key, QVariant{ false }).toBool();
 	}
 	
 	const std::vector<QByteArray>& supportedRasterExtensions() const
@@ -114,11 +158,18 @@ public:
 		return ret; 
 	}
 	
-	const std::vector<QByteArray>& supportedVectorExtensions() const
+	const std::vector<QByteArray>& supportedVectorImportExtensions() const
 	{
 		if (dirty)
 			const_cast<GdalManagerPrivate*>(this)->update();
-		return enabled_vector_extensions;
+		return enabled_vector_import_extensions;
+	}
+
+	const std::vector<QByteArray>& supportedVectorExportExtensions() const
+	{
+		if (dirty)
+			const_cast<GdalManagerPrivate*>(this)->update();
+		return enabled_vector_export_extensions;
 	}
 	
 	QStringList parameterKeys() const
@@ -158,26 +209,40 @@ private:
 	{
 		QSettings settings;
 		
-#ifdef GDAL_DMD_EXTENSIONS
-		// GDAL >= 2.0
 		settings.beginGroup(gdal_manager_group);
 		auto count = GDALGetDriverCount();
-		enabled_vector_extensions.clear();
-		enabled_vector_extensions.reserve(std::size_t(count));
+		enabled_vector_import_extensions.clear();
+		enabled_vector_import_extensions.reserve(std::size_t(count));
+		enabled_vector_export_extensions.clear();
+		enabled_vector_export_extensions.reserve(std::size_t(count));
 		for (auto i = 0; i < count; ++i)
 		{
 			auto driver_data = GDALGetDriver(i);
 			auto type = GDALGetMetadataItem(driver_data, GDAL_DCAP_VECTOR, nullptr);
 			if (qstrcmp(type, "YES") != 0)
 				continue;
-			
-			// Skip write-only drivers.
+
+			auto extensions_raw = GDALGetMetadataItem(driver_data, GDAL_DMD_EXTENSIONS, nullptr);
+			auto extensions = QByteArray::fromRawData(extensions_raw, int(qstrlen(extensions_raw)));
+
+			auto cap_create = GDALGetMetadataItem(driver_data, GDAL_DCAP_CREATE, nullptr);
+			if (qstrcmp(cap_create, "YES") == 0)
+			{
+				for (auto pos = 0; pos >= 0; )
+				{
+					auto start = pos ? pos + 1 : 0;
+					pos = extensions.indexOf(' ', start);
+					auto extension = extensions.mid(start, pos - start);
+					if (extension.isEmpty())
+						continue;
+					enabled_vector_export_extensions.emplace_back(extension);
+				}
+			}
+
 			auto cap_open = GDALGetMetadataItem(driver_data, GDAL_DCAP_OPEN, nullptr);
 			if (qstrcmp(cap_open, "YES") != 0)
 				continue;
-			
-			auto extensions_raw = GDALGetMetadataItem(driver_data, GDAL_DMD_EXTENSIONS, nullptr);
-			auto extensions = QByteArray::fromRawData(extensions_raw, int(qstrlen(extensions_raw)));
+
 			for (auto pos = 0; pos >= 0; )
 			{
 				auto start = pos ? pos + 1 : 0;
@@ -185,36 +250,12 @@ private:
 				auto extension = extensions.mid(start, pos - start);
 				if (extension.isEmpty())
 					continue;
-				if (extension == "dxf" && !settings.value(gdal_dxf_key).toBool())
+				if (extension == "gpx" && !settings.value(gdal_gpx_key, false).toBool())
 					continue;
-				if (extension == "gpx" && !settings.value(gdal_gpx_key).toBool())
-					continue;
-				if (extension == "osm" && !settings.value(gdal_osm_key).toBool())
-					continue;
-				enabled_vector_extensions.emplace_back(extension);
+				enabled_vector_import_extensions.emplace_back(extension);
 			}
 		}
 		settings.endGroup();
-#else
-		// GDAL < 2.0 does not provide the supported extensions 
-		static const std::vector<QByteArray> default_extensions = {
-		    "shp", "dbf",
-		    /* "dxf", */
-		    /* "gpx", */
-		    /* "osm", */ "pbf",
-		};
-		enabled_vector_extensions.reserve(default_extensions.size() + 3);
-		enabled_vector_extensions = default_extensions;
-		
-		settings.beginGroup(gdal_manager_group);
-		if (settings.value(gdal_dxf_key).toBool())
-			enabled_vector_extensions.push_back("dxf");
-		if (settings.value(gdal_gpx_key).toBool())
-			enabled_vector_extensions.push_back("gpx");
-		if (settings.value(gdal_osm_key).toBool())
-			enabled_vector_extensions.push_back("osm");
-		settings.endGroup();
-#endif
 		
 		// Using osmconf.ini to detect a directory with data from gdal. The
 		// data:/gdal directory will always exist, due to mapper-osmconf.ini.
@@ -235,7 +276,7 @@ private:
 		    { "OSM_USE_CUSTOM_INDEXING", "NO"  },
 		    { "GPX_ELE_AS_25D",          "YES" },
 		};
-		for (const auto setting : defaults)
+		for (const auto* setting : defaults)
 		{
 			const auto key = QString::fromLatin1(setting[0]);
 			if (!settings.contains(key))
@@ -289,7 +330,9 @@ private:
 	
 	mutable bool dirty;
 	
-	mutable std::vector<QByteArray> enabled_vector_extensions;
+	mutable std::vector<QByteArray> enabled_vector_import_extensions;
+
+	mutable std::vector<QByteArray> enabled_vector_export_extensions;
 	
 	mutable QStringList applied_parameters;
 	
@@ -310,6 +353,28 @@ void GdalManager::configure()
 	p->configure();
 }
 
+
+bool GdalManager::isAreaHatchingEnabled() const
+{
+	return p->settingsValue(p->gdal_hatch_key, false).toBool();
+}
+
+void GdalManager::setAreaHatchingEnabled(bool enabled)
+{
+	p->setSettingsValue(p->gdal_hatch_key, enabled);
+}
+
+bool GdalManager::isBaselineViewEnabled() const
+{
+	return p->settingsValue(p->gdal_baseline_key, false).toBool();
+}
+
+void GdalManager::setBaselineViewEnabled(bool enabled)
+{
+	p->setSettingsValue(p->gdal_baseline_key, enabled);
+}
+
+
 void GdalManager::setFormatEnabled(GdalManager::FileFormat format, bool enabled)
 {
 	p->setFormatEnabled(format, enabled);
@@ -320,14 +385,29 @@ bool GdalManager::isFormatEnabled(GdalManager::FileFormat format) const
 	return p->isFormatEnabled(format);
 }
 
+void GdalManager::setExportOptionEnabled(GdalManager::ExportOption option, bool enabled)
+{
+	return p->setExportOptionEnabled(option, enabled);
+}
+
+bool GdalManager::isExportOptionEnabled(GdalManager::ExportOption option) const
+{
+	return p->isExportOptionEnabled(option);
+}
+
 const std::vector<QByteArray>&GdalManager::supportedRasterExtensions() const
 {
 	return p->supportedRasterExtensions();
 }
 
-const std::vector<QByteArray>& GdalManager::supportedVectorExtensions() const
+const std::vector<QByteArray>& GdalManager::supportedVectorImportExtensions() const
 {
-	return p->supportedVectorExtensions();
+	return p->supportedVectorImportExtensions();
+}
+
+const std::vector<QByteArray>& GdalManager::supportedVectorExportExtensions() const
+{
+	return p->supportedVectorExportExtensions();
 }
 
 QStringList GdalManager::parameterKeys() const
