@@ -54,11 +54,10 @@ protected:
 	int findDuplicateKey(const QString& key, int row) const;
 	
 private:
-	QCheckBox* import_dxf;
 	QCheckBox* import_gpx;
-	QCheckBox* import_osm;
 	QCheckBox* view_hatch;
 	QCheckBox* view_baseline;
+	QCheckBox* export_one_layer_per_symbol;
 	QTableWidget* parameters;
 };
 
