@@ -1,5 +1,5 @@
 /*
- *    Copyright 2016-2018 Kai Pastor
+ *    Copyright 2016-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -20,14 +20,15 @@
 #include "ogr_template.h"
 
 #include <algorithm>
+#include <iosfwd>
 #include <iterator>
 #include <memory>
+#include <utility>
 
 #include <Qt>
 #include <QtGlobal>
 #include <QByteArray>
 #include <QDialog>
-#include <QFile>
 #include <QLatin1String>
 #include <QPoint>
 #include <QPointF>
@@ -42,17 +43,22 @@
 #include "core/latlon.h"
 #include "core/map.h"
 #include "core/map_coord.h"
+#include "core/track.h"
 #include "core/objects/object.h"
 #include "fileformats/file_format.h"
 #include "gdal/gdal_manager.h"
 #include "gdal/ogr_file_format_p.h"
 #include "gui/georeferencing_dialog.h"
-#include "sensors/gps_track.h"
 #include "templates/template.h"
 #include "templates/template_positioning_dialog.h"
 #include "templates/template_track.h"
 
 
+#ifdef __clang_analyzer__
+#define singleShot(A, B, C) singleShot(A, B, #C) // NOLINT
+#endif
+
+
 namespace OpenOrienteering {
 
 namespace {
@@ -65,23 +71,12 @@ namespace {
 	}
 	
 	
-	std::unique_ptr<Georeferencing> getDataGeoreferencing(QFile& file, const Georeferencing& initial_georef)
-	{
-		Map tmp_map;
-		tmp_map.setGeoreferencing(initial_georef);
-		OgrFileImport importer{ &file, &tmp_map, nullptr, OgrFileImport::UnitOnGround};
-		importer.setGeoreferencingImportEnabled(true);
-		importer.doImport(true);
-		
-		return std::make_unique<Georeferencing>(tmp_map.getGeoreferencing());
-	}
-	
-	
 	bool preserveRefPoints(Georeferencing& data_georef, const Georeferencing& initial_georef)
 	{
 		// Keep a configured local reference point from initial_georef?
 		auto data_crs_spec = data_georef.getProjectedCRSSpec();
 		if ((!initial_georef.isValid() || initial_georef.isLocal())
+		    && initial_georef.getProjectedRefPoint() != QPointF{}
 		    && data_georef.isValid()
 		    && !data_georef.isLocal()
 		    && data_georef.getProjectedRefPoint() == QPointF{}
@@ -107,7 +102,21 @@ namespace {
 
 const std::vector<QByteArray>& OgrTemplate::supportedExtensions()
 {
-	return GdalManager().supportedVectorExtensions();
+	return GdalManager().supportedVectorImportExtensions();
+}
+
+
+std::unique_ptr<Georeferencing> OgrTemplate::getDataGeoreferencing(const QString& path, const Georeferencing& initial_georef)
+{
+	Map tmp_map;
+	tmp_map.setGeoreferencing(initial_georef);
+	OgrFileImport importer{ path, &tmp_map, nullptr, OgrFileImport::UnitOnGround};
+	importer.setGeoreferencingImportEnabled(true);
+	importer.setLoadSymbolsOnly(true);
+	if (!importer.doImport())
+		return {};  // failure
+	
+	return std::make_unique<Georeferencing>(tmp_map.getGeoreferencing());  // success
 }
 
 
@@ -135,16 +144,16 @@ const char* OgrTemplate::getTemplateType() const
 
 
 
-std::unique_ptr<Georeferencing> OgrTemplate::makeOrthographicGeoreferencing(QFile& file)
+std::unique_ptr<Georeferencing> OgrTemplate::makeOrthographicGeoreferencing(const QString& path)
 {
 	// Is the template's SRS orthographic, or can it be converted?
 	/// \todo Use the template's datum etc. instead of WGS84?
 	auto georef = std::make_unique<Georeferencing>();
 	georef->setScaleDenominator(int(map->getGeoreferencing().getScaleDenominator()));
 	georef->setProjectedCRS(QString{}, QStringLiteral("+proj=ortho +datum=WGS84 +ellps=WGS84 +units=m +no_defs"));
-	if (OgrFileImport::checkGeoreferencing(file, *georef))
+	if (OgrFileImport::checkGeoreferencing(path, *georef))
 	{
-		auto center = OgrFileImport::calcAverageLatLon(file);
+		auto center = OgrFileImport::calcAverageLatLon(path);
 		georef->setProjectedCRS(QString{},
 		                             QString::fromLatin1("+proj=ortho +datum=WGS84 +ellps=WGS84 +units=m +lat_0=%1 +lon_0=%2 +no_defs")
 		                             .arg(center.latitude()).arg(center.longitude()));
@@ -175,20 +184,11 @@ try
 	};
 	template_track_compatibility = ends_with_any_of(template_path, TemplateTrack::supportedExtensions());
 	
-	QFile file{ template_path };
-	
 	auto data_georef = std::unique_ptr<Georeferencing>();
 	auto& initial_georef = map->getGeoreferencing();
 	if (!initial_georef.isValid() || initial_georef.isLocal())
 	{
-		// The map doesn't have a proper georeferencing.
-		// Is there a good SRS in the data?
-		try {
-			data_georef = getDataGeoreferencing(file, initial_georef);
-		}
-		catch (FileFormatException&)
-		{}
-		
+		data_georef = getDataGeoreferencing(template_path, initial_georef);
 		if (data_georef && data_georef->isValid() && !data_georef->isLocal())
 		{
 			// If yes, does the user want to use this for the map?
@@ -211,7 +211,7 @@ try
 	{
 		// The map has got a proper georeferencing.
 		// Can the template's SRS be converted to the map's CRS?
-		if (OgrFileImport::checkGeoreferencing(file, map->getGeoreferencing()))
+		if (OgrFileImport::checkGeoreferencing(template_path, map->getGeoreferencing()))
 		{
 			is_georeferenced = true;
 			return true;
@@ -225,7 +225,7 @@ try
 	}
 	if (!data_georef)
 	{
-		data_georef = makeOrthographicGeoreferencing(file);
+		data_georef = makeOrthographicGeoreferencing(template_path);
 	}
 	if (data_georef)
 	{
@@ -259,10 +259,9 @@ catch (FileFormatException& e)
 bool OgrTemplate::loadTemplateFileImpl(bool configuring)
 try
 {
-	QFile file{ template_path };
 	auto new_template_map = std::make_unique<Map>();
 	auto unit_type = use_real_coords ? OgrFileImport::UnitOnGround : OgrFileImport::UnitOnPaper;
-	OgrFileImport importer{ &file, new_template_map.get(), nullptr, unit_type };
+	OgrFileImport importer{template_path, new_template_map.get(), nullptr, unit_type };
 	
 	// Configure generation of renderables.
 	updateView(*new_template_map);
@@ -322,14 +321,14 @@ try
 					else
 					{
 						// If the TemplateTrack approach failed, use local approach.
-						explicit_georef = makeOrthographicGeoreferencing(file);
+						explicit_georef = makeOrthographicGeoreferencing(template_path);
 						projected_crs_spec = explicit_georef->getProjectedCRSSpec();
 					}
 				}
 				
 				if (!explicit_georef)
 				{
-					explicit_georef.reset(new Georeferencing());
+					explicit_georef = std::make_unique<Georeferencing>();
 					explicit_georef->setScaleDenominator(int(map_georef.getScaleDenominator()));
 					explicit_georef->setProjectedCRS(QString{}, projected_crs_spec);
 					explicit_georef->setProjectedRefPoint({}, false);
@@ -350,7 +349,11 @@ try
 	
 	const auto pp0 = new_template_map->getGeoreferencing().getProjectedRefPoint();
 	importer.setGeoreferencingImportEnabled(false);
-	importer.doImport(false, template_path);
+	if (!importer.doImport())
+	{
+		setErrorString(importer.warnings().back());
+		return false;
+	}
 	
 	// MapCoord bounds handling may have moved the paper position of the
 	// template data during import. The template position might need to be
@@ -441,10 +444,9 @@ void OgrTemplate::reloadLater()
 {
 	if (reload_pending)
 		return;
-		
 	if (template_state == Loaded)
 		templateMap()->clear(); // no expensive operations before reloading
-	QTimer::singleShot(0, this, SLOT(reload()));
+	QTimer::singleShot(0, this, &OgrTemplate::reload);
 	reload_pending = true;
 }
 
@@ -504,7 +506,7 @@ bool OgrTemplate::loadTypeSpecificTemplateConfiguration(QXmlStreamReader& xml)
 {
 	if (xml.name() == literal::georeferencing)
 	{
-		explicit_georef.reset(new Georeferencing());
+		explicit_georef = std::make_unique<Georeferencing>();
 		explicit_georef->load(xml, false);
 	}
 	else if (xml.name() == literal::crs_spec)
