@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2017 Kai Pastor
+ *    Copyright 2012-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -25,6 +25,7 @@
 
 #include <Qt>
 #include <QtGlobal>
+#include <QAbstractButton>
 #include <QCursor>
 #include <QDate>
 #include <QDebug>
@@ -47,7 +48,6 @@
 #include <QSignalBlocker>
 #include <QSize>
 #include <QSpacerItem>
-#include <QString>
 #include <QStringRef>
 #include <QTimer>
 #include <QUrl>
@@ -71,11 +71,31 @@
 #include "gui/map/map_editor.h"
 #include "gui/widgets/crs_selector.h"
 #include "gui/util_gui.h"
+#include "util/backports.h"  // IWYU pragma: keep
 #include "util/scoped_signals_blocker.h"
 
 
+#ifdef __clang_analyzer__
+#define singleShot(A, B, C) singleShot(A, B, #C) // NOLINT 
+#endif
+
+
 namespace OpenOrienteering {
 
+Q_STATIC_ASSERT(Georeferencing::declinationPrecision() == Util::InputProperties<Util::RotationalDegrees>::decimals());
+
+
+namespace  {
+
+void setValueIfChanged(QDoubleSpinBox* field, qreal value) {
+	if (!qFuzzyCompare(field->value(), value))
+		field->setValue(value);
+}
+
+}  // namespace
+
+
+
 // ### GeoreferencingDialog ###
 
 GeoreferencingDialog::GeoreferencingDialog(MapEditorController* controller, const Georeferencing* initial, bool allow_no_georeferencing)
@@ -105,11 +125,7 @@ GeoreferencingDialog::GeoreferencingDialog(
  , tool_active(false)
  , declination_query_in_progress(false)
  , grivation_locked(!initial_georef->isValid() || initial_georef->getState() != Georeferencing::Normal)
- , original_declination(0.0)
 {
-	if (!grivation_locked)
-		original_declination = initial_georef->getDeclination();
-	
 	setWindowTitle(tr("Map Georeferencing"));
 	setWindowModality(Qt::WindowModal);
 	
@@ -157,8 +173,9 @@ GeoreferencingDialog::GeoreferencingDialog(
 	projected_ref_layout->addSpacing(qMax(ref_point_button_width, geographic_datum_label_width));
 	
 	projected_ref_label = new QLabel();
-	lat_edit = Util::SpinBox::create(8, -90.0, +90.0, trUtf8("°"));
-	lon_edit = Util::SpinBox::create(8, -180.0, +180.0, trUtf8("°"));
+	lat_edit = Util::SpinBox::create(8, -90.0, +90.0, Util::InputProperties<Util::RotationalDegrees>::unit());
+	lon_edit = Util::SpinBox::create(8, -180.0, +180.0, Util::InputProperties<Util::RotationalDegrees>::unit());
+	lon_edit->setWrapping(true);
 	auto geographic_ref_layout = new QHBoxLayout();
 	geographic_ref_layout->addWidget(lat_edit, 1);
 	geographic_ref_layout->addWidget(new QLabel(tr("N", "north")), 0);
@@ -186,7 +203,7 @@ GeoreferencingDialog::GeoreferencingDialog(
 	
 	auto map_north_label = Util::Headline::create(tr("Map north"));
 	
-	declination_edit = Util::SpinBox::create(Georeferencing::declinationPrecision(), -180.0, +180.0, trUtf8("°"));
+	declination_edit = Util::SpinBox::create<Util::RotationalDegrees>();
 	declination_button = new QPushButton(tr("Lookup..."));
 	auto declination_layout = new QHBoxLayout();
 	declination_layout->addWidget(declination_edit, 1);
@@ -234,21 +251,20 @@ GeoreferencingDialog::GeoreferencingDialog(
 	
 	connect(crs_selector, &CRSSelector::crsChanged, this, &GeoreferencingDialog::crsEdited);
 	
-	using TakingDoubleArgument = void (QDoubleSpinBox::*)(double);
-	connect(scale_factor_edit, (TakingDoubleArgument)&QDoubleSpinBox::valueChanged, this, &GeoreferencingDialog::scaleFactorEdited);
+	connect(scale_factor_edit, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &GeoreferencingDialog::scaleFactorEdited);
 	
-	connect(map_x_edit, (TakingDoubleArgument)&QDoubleSpinBox::valueChanged, this, &GeoreferencingDialog::mapRefChanged);
-	connect(map_y_edit, (TakingDoubleArgument)&QDoubleSpinBox::valueChanged, this, &GeoreferencingDialog::mapRefChanged);
+	connect(map_x_edit, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &GeoreferencingDialog::mapRefChanged);
+	connect(map_y_edit, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &GeoreferencingDialog::mapRefChanged);
 	connect(ref_point_button, &QPushButton::clicked, this, &GeoreferencingDialog::selectMapRefPoint);
 	
-	connect(easting_edit, (TakingDoubleArgument)&QDoubleSpinBox::valueChanged, this, &GeoreferencingDialog::eastingNorthingEdited);
-	connect(northing_edit, (TakingDoubleArgument)&QDoubleSpinBox::valueChanged, this, &GeoreferencingDialog::eastingNorthingEdited);
+	connect(easting_edit, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &GeoreferencingDialog::eastingNorthingEdited);
+	connect(northing_edit, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &GeoreferencingDialog::eastingNorthingEdited);
 	
-	connect(lat_edit, (TakingDoubleArgument)&QDoubleSpinBox::valueChanged, this, &GeoreferencingDialog::latLonEdited);
-	connect(lon_edit, (TakingDoubleArgument)&QDoubleSpinBox::valueChanged, this, &GeoreferencingDialog::latLonEdited);
+	connect(lat_edit, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &GeoreferencingDialog::latLonEdited);
+	connect(lon_edit, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &GeoreferencingDialog::latLonEdited);
 	connect(keep_geographic_radio, &QRadioButton::toggled, this, &GeoreferencingDialog::keepCoordsChanged);
 	
-	connect(declination_edit, (TakingDoubleArgument)&QDoubleSpinBox::valueChanged, this, &GeoreferencingDialog::declinationEdited);
+	connect(declination_edit, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &GeoreferencingDialog::declinationEdited);
 	connect(declination_button, &QPushButton::clicked, this, &GeoreferencingDialog::requestDeclination);
 	
 	connect(buttons_box, &QDialogButtonBox::accepted, this, &GeoreferencingDialog::accept);
@@ -286,7 +302,7 @@ void GeoreferencingDialog::georefStateChanged()
 		break;
 	default:
 		qDebug() << "Unhandled georeferencing state:" << georef->getState();
-		// fall through
+		Q_FALLTHROUGH();
 	case Georeferencing::Normal:
 		projectionChanged();
 		keep_geographic_radio->setEnabled(true);
@@ -304,13 +320,13 @@ void GeoreferencingDialog::transformationChanged()
 	            scale_factor_edit
 	);
 	
-	map_x_edit->setValue(georef->getMapRefPoint().x());
-	map_y_edit->setValue(-1 * georef->getMapRefPoint().y());
+	setValueIfChanged(map_x_edit, georef->getMapRefPoint().x());
+	setValueIfChanged(map_y_edit, -georef->getMapRefPoint().y());
 	
-	easting_edit->setValue(georef->getProjectedRefPoint().x());
-	northing_edit->setValue(georef->getProjectedRefPoint().y());
+	setValueIfChanged(easting_edit, georef->getProjectedRefPoint().x());
+	setValueIfChanged(northing_edit, georef->getProjectedRefPoint().y());
 	
-	scale_factor_edit->setValue(georef->getGridScaleFactor());
+	setValueIfChanged(scale_factor_edit, georef->getGridScaleFactor());
 	
 	updateGrivation();
 }
@@ -342,8 +358,8 @@ void GeoreferencingDialog::projectionChanged()
 	LatLon latlon = georef->getGeographicRefPoint();
 	double latitude  = latlon.latitude();
 	double longitude = latlon.longitude();
-	lat_edit->setValue(latitude);
-	lon_edit->setValue(longitude);
+	setValueIfChanged(lat_edit, latitude);
+	setValueIfChanged(lon_edit, longitude);
 	QString osm_link =
 	  QString::fromLatin1("http://www.openstreetmap.org/?lat=%1&lon=%2&zoom=18&layers=M").
 	  arg(latitude).arg(longitude);
@@ -366,7 +382,7 @@ void GeoreferencingDialog::projectionChanged()
 void GeoreferencingDialog::declinationChanged()
 {
 	const QSignalBlocker block(declination_edit);
-	declination_edit->setValue(georef->getDeclination());
+	setValueIfChanged(declination_edit, georef->getDeclination());
 }
 
 void GeoreferencingDialog::requestDeclination(bool no_confirm)
@@ -417,7 +433,7 @@ void GeoreferencingDialog::requestDeclination(bool no_confirm)
 #endif
 }
 
-void GeoreferencingDialog::setMapRefPoint(MapCoord coords)
+void GeoreferencingDialog::setMapRefPoint(const MapCoord& coords)
 {
 	georef->setMapRefPoint(coords);
 	reset_button->setEnabled(true);
@@ -448,18 +464,19 @@ void GeoreferencingDialog::showHelp()
 void GeoreferencingDialog::reset()
 {
 	grivation_locked = ( !initial_georef->isValid() || initial_georef->getState() != Georeferencing::Normal );
-	if (!grivation_locked)
-		original_declination = initial_georef->getDeclination();
 	*georef.data() = *initial_georef;
 	reset_button->setEnabled(false);
 }
 
 void GeoreferencingDialog::accept()
 {
-	float declination_change_degrees = georef->getDeclination() - initial_georef->getDeclination();
-	if ( !grivation_locked &&
-	     declination_change_degrees != 0 &&
-	     (map->getNumObjects() > 0 || map->getNumTemplates() > 0) )
+	auto const declination_change_degrees = georef->getDeclination() - initial_georef->getDeclination();
+	if (grivation_locked)
+	{
+		georef->updateGrivation();
+	}
+	else if (!qIsNull(declination_change_degrees)
+	         && (map->getNumObjects() > 0 || map->getNumTemplates() > 0))
 	{
 		int result = QMessageBox::question(this, tr("Declination change"), tr("The declination has been changed. Do you want to rotate the map content accordingly, too?"), QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
 		if (result == QMessageBox::Cancel)
@@ -541,7 +558,7 @@ void GeoreferencingDialog::crsEdited()
 	{
 	default:
 		qWarning("Unsupported CRS item id");
-		// fall through
+		Q_FALLTHROUGH();
 	case Georeferencing::Local:
 		// Local
 		georef_copy.setState(Georeferencing::Local);
@@ -611,8 +628,8 @@ void GeoreferencingDialog::keepCoordsChanged()
 	if (grivation_locked && keep_geographic_radio->isChecked())
 	{
 		grivation_locked = false;
-		original_declination = georef->getDeclination();
 		updateGrivation();
+		georef->updateGrivation();
 	}
 	reset_button->setEnabled(true);
 }
@@ -622,7 +639,6 @@ void GeoreferencingDialog::declinationEdited(double value)
 	if (grivation_locked)
 	{
 		grivation_locked = false;
-		original_declination = georef->getDeclination();
 		updateGrivation();
 	}
 	georef->setDeclination(value);
@@ -660,7 +676,7 @@ void GeoreferencingDialog::declinationReplyFinished(QNetworkReply* reply)
 								double declination = text.toDouble(&ok);
 								if (ok)
 								{
-									declination_edit->setValue(Georeferencing::roundDeclination(declination));
+									setValueIfChanged(declination_edit, Georeferencing::roundDeclination(declination));
 									return;
 								}
 								else 
@@ -727,7 +743,7 @@ void GeoreferencingTool::init()
 	MapEditorTool::init();
 }
 
-bool GeoreferencingTool::mousePressEvent(QMouseEvent* event, MapCoordF /*map_coord*/, MapWidget* /*widget*/)
+bool GeoreferencingTool::mousePressEvent(QMouseEvent* event, const MapCoordF& /*map_coord*/, MapWidget* /*widget*/)
 {
 	bool handled = false;
 	switch (event->button())
@@ -743,16 +759,16 @@ bool GeoreferencingTool::mousePressEvent(QMouseEvent* event, MapCoordF /*map_coo
 	return handled;
 }
 
-bool GeoreferencingTool::mouseReleaseEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget* /*widget*/)
+bool GeoreferencingTool::mouseReleaseEvent(QMouseEvent* event, const MapCoordF& map_coord, MapWidget* /*widget*/)
 {
 	bool handled = false;
 	switch (event->button())
 	{
 	case Qt::LeftButton:
 		dialog->setMapRefPoint(MapCoord(map_coord));
-		// fall through
+		Q_FALLTHROUGH();
 	case Qt::RightButton:
-		QTimer::singleShot(0, dialog, SIGNAL(exec()));  // clazy:exclude=old-style-connect
+		QTimer::singleShot(0, dialog, &QDialog::exec);
 		handled = true;
 		break;
 	default:
