@@ -37,6 +37,7 @@
 #include <QList>
 #include <QLocale>
 #include <QMouseEvent>
+#include <QObjectList>
 #include <QPainter>
 #include <QPaintEvent>
 #include <QPinchGesture>
@@ -70,6 +71,11 @@ class QGesture;
 // IWYU pragma: no_forward_declare QPinchGesture
 
 
+#ifdef __clang_analyzer__
+#define singleShot(A, B, C) singleShot(A, B, #C) // NOLINT 
+#endif
+
+
 namespace OpenOrienteering {
 
 MapWidget::MapWidget(bool show_help, bool force_antialiasing, QWidget* parent)
@@ -190,14 +196,16 @@ QRectF MapWidget::viewportToView(const QRect& input) const
 	return QRectF(input.left() - 0.5*width() - pan_offset.x(), input.top() - 0.5*height() - pan_offset.y(), input.width(), input.height());
 }
 
-QPointF MapWidget::viewportToView(QPoint input) const
+QPointF MapWidget::viewportToView(const QPoint& input) const
 {
 	return QPointF(input.x() - 0.5*width() - pan_offset.x(), input.y() - 0.5*height() - pan_offset.y());
 }
 
 QPointF MapWidget::viewportToView(QPointF input) const
 {
-	return QPointF(input.x() - 0.5*width() - pan_offset.x(), input.y() - 0.5*height() - pan_offset.y());
+	input.rx() -= 0.5*width() + pan_offset.x();
+	input.ry() -= 0.5*height() + pan_offset.y();
+	return input;
 }
 
 QRectF MapWidget::viewToViewport(const QRectF& input) const
@@ -210,38 +218,40 @@ QRectF MapWidget::viewToViewport(const QRect& input) const
 	return QRectF(input.left() + 0.5*width() + pan_offset.x(), input.top() + 0.5*height() + pan_offset.y(), input.width(), input.height());
 }
 
-QPointF MapWidget::viewToViewport(QPoint input) const
+QPointF MapWidget::viewToViewport(const QPoint& input) const
 {
 	return QPointF(input.x() + 0.5*width() + pan_offset.x(), input.y() + 0.5*height() + pan_offset.y());
 }
 
 QPointF MapWidget::viewToViewport(QPointF input) const
 {
-	return QPointF(input.x() + 0.5*width() + pan_offset.x(), input.y() + 0.5*height() + pan_offset.y());
+	input.rx() += 0.5*width() + pan_offset.x();
+	input.ry() += 0.5*height() + pan_offset.y();
+	return input;
 }
 
 
-MapCoord MapWidget::viewportToMap(QPoint input) const
+MapCoord MapWidget::viewportToMap(const QPoint& input) const
 {
 	return view->viewToMap(viewportToView(input));
 }
 
-MapCoordF MapWidget::viewportToMapF(QPoint input) const
+MapCoordF MapWidget::viewportToMapF(const QPoint& input) const
 {
 	return view->viewToMapF(viewportToView(input));
 }
 
-MapCoordF MapWidget::viewportToMapF(QPointF input) const
+MapCoordF MapWidget::viewportToMapF(const QPointF& input) const
 {
 	return view->viewToMapF(viewportToView(input));
 }
 
-QPointF MapWidget::mapToViewport(MapCoord input) const
+QPointF MapWidget::mapToViewport(const MapCoord& input) const
 {
 	return viewToViewport(view->mapToView(input));
 }
 
-QPointF MapWidget::mapToViewport(MapCoordF input) const
+QPointF MapWidget::mapToViewport(const QPointF& input) const
 {
 	return viewToViewport(view->mapToView(input));
 }
@@ -268,13 +278,13 @@ void MapWidget::viewChanged(MapView::ChangeFlags changes)
 		updateZoomDisplay();
 }
 
-void MapWidget::setPanOffset(QPoint offset)
+void MapWidget::setPanOffset(const QPoint& offset)
 {
 	pan_offset = offset;
 	update();
 }
 
-void MapWidget::startDragging(QPoint cursor_pos)
+void MapWidget::startDragging(const QPoint& cursor_pos)
 {
 	Q_ASSERT(!dragging);
 	Q_ASSERT(!pinching);
@@ -284,13 +294,13 @@ void MapWidget::startDragging(QPoint cursor_pos)
 	setCursor(Qt::ClosedHandCursor);
 }
 
-void MapWidget::updateDragging(QPoint cursor_pos)
+void MapWidget::updateDragging(const QPoint& cursor_pos)
 {
 	Q_ASSERT(dragging);
 	view->setPanOffset(cursor_pos - drag_start_pos);
 }
 
-void MapWidget::finishDragging(QPoint cursor_pos)
+void MapWidget::finishDragging(const QPoint& cursor_pos)
 {
 	Q_ASSERT(dragging);
 	dragging = false;
@@ -305,7 +315,7 @@ void MapWidget::cancelDragging()
 	setCursor(normal_cursor);
 }
 
-qreal MapWidget::startPinching(QPoint center)
+qreal MapWidget::startPinching(const QPoint& center)
 {
 	Q_ASSERT(!dragging);
 	Q_ASSERT(!pinching);
@@ -316,7 +326,7 @@ qreal MapWidget::startPinching(QPoint center)
 	return pinching_factor;
 }
 
-void MapWidget::updatePinching(QPoint center, qreal factor)
+void MapWidget::updatePinching(const QPoint& center, qreal factor)
 {
 	Q_ASSERT(pinching);
 	pinching_center = center;
@@ -325,7 +335,7 @@ void MapWidget::updatePinching(QPoint center, qreal factor)
 	update();
 }
 
-void MapWidget::finishPinching(QPoint center, qreal factor)
+void MapWidget::finishPinching(const QPoint& center, qreal factor)
 {
 	pinching = false;
 	view->finishPanning(center - drag_start_pos);
@@ -522,7 +532,7 @@ void MapWidget::updateDrawingLater(const QRectF& map_rect, int pixel_border)
 		if (!cached_update_rect.isValid())
 		{
 			// Start the update timer
-			QTimer::singleShot(15, this, SLOT(updateDrawingLaterSlot()));  // clazy:exclude=old-style-connect
+			QTimer::singleShot(15, this, &MapWidget::updateDrawingLaterSlot);
 		}
 		
 		// NOTE: this may require a mutex for concurrent access with updateDrawingLaterSlot()?
@@ -587,7 +597,7 @@ void MapWidget::setCoordsDisplay(CoordsType type)
 	updateCursorposLabel(last_cursor_pos);
 }
 
-void MapWidget::updateCursorposLabel(const MapCoordF pos)
+void MapWidget::updateCursorposLabel(const MapCoordF& pos)
 {
 	last_cursor_pos = pos;
 	
@@ -791,7 +801,7 @@ void MapWidget::paintEvent(QPaintEvent* event)
 		return;
 	}
 	
-	// No colors, symbols, or objects? Provide a litte help message ...
+	// No colors, symbols, or objects? Provide a little help message ...
 	bool no_contents = view->getMap()->getNumObjects() == 0 && view->getMap()->getNumTemplates() == 0 && !view->isGridVisible();
 	
 	QTransform transform = painter.worldTransform();
