@@ -20,27 +20,38 @@
 
 #include "settings_dialog.h"
 
+#include <algorithm>
+
 #include <Qt>
 #include <QtGlobal>
 #include <QAbstractButton> // IWYU pragma: keep
+#include <QAbstractItemModel>
+#include <QAbstractItemView>
 #include <QAction>
 #include <QDialogButtonBox>
 #include <QFormLayout>
 #include <QFlags>
 #include <QFrame>
+#include <QHeaderView>
 #include <QKeyEvent>
 #include <QLabel>
 #include <QLatin1String>
 #include <QLayout>
+#include <QList>
+#include <QObjectList>
+#include <QRect>
 #include <QScrollArea>
+#include <QScrollBar>
 #include <QScroller>
+#include <QSize>
 #include <QStackedWidget>
 #include <QTabWidget>
+#include <QTableWidget>
 #include <QToolBar>
 #include <QVBoxLayout>
 #include <QWidget>
 
-#include "gui/main_window.h"
+#include "settings.h"
 #include "gui/util_gui.h"
 #include "gui/widgets/editor_settings_page.h"
 #include "gui/widgets/general_settings_page.h"
@@ -51,6 +62,10 @@
 #  include "gdal/gdal_settings_page.h"
 #endif
 
+#ifdef MAPPER_USE_SENSORS
+#  include "sensors/sensors_settings_page.h"
+#endif
+
 
 namespace OpenOrienteering {
 
@@ -58,13 +73,14 @@ SettingsDialog::SettingsDialog(QWidget* parent)
  : QDialog        { parent }
  , tab_widget     { nullptr }
  , stack_widget   { nullptr }
+ , scrollbar_extent { QScrollBar(Qt::Vertical).sizeHint().width() }
 {
 	setWindowTitle(tr("Settings"));
 	
 	auto layout = new QVBoxLayout(this);
 	
 	auto buttons = QDialogButtonBox::StandardButtons{ QDialogButtonBox::Ok };
-	if (MainWindow::mobileMode())
+	if (Settings::mobileModeEnforced())
 	{
 		if (parent)
 			setGeometry(parent->geometry());
@@ -90,7 +106,7 @@ SettingsDialog::SettingsDialog(QWidget* parent)
 	
 	button_box = new QDialogButtonBox(buttons, Qt::Horizontal);
 	connect(button_box, &QDialogButtonBox::clicked, this, &SettingsDialog::buttonPressed);
-	if (MainWindow::mobileMode())
+	if (stack_widget)
 	{
 		int left, top, right, bottom;
 		layout->getContentsMargins(&left, &top, &right, &bottom);
@@ -116,7 +132,7 @@ SettingsDialog::~SettingsDialog() = default;
 
 void SettingsDialog::closeEvent(QCloseEvent* event)
 {
-	if (MainWindow::mobileMode())
+	if (stack_widget)
 		callOnAllPages(&SettingsPage::apply);
 	QDialog::closeEvent(event);
 }
@@ -127,7 +143,7 @@ void SettingsDialog::keyPressEvent(QKeyEvent* event)
 	{
 	case Qt::Key_Back:
 	case Qt::Key_Escape:
-		if (MainWindow::mobileMode() && stack_widget->currentIndex() > 0)
+		if (stack_widget && stack_widget->currentIndex() > 0)
 		{
 			stack_widget->setCurrentIndex(0);
 			auto buttons = button_box->standardButtons();
@@ -141,6 +157,60 @@ void SettingsDialog::keyPressEvent(QKeyEvent* event)
 	QDialog::keyPressEvent(event);
 }
 
+
+void SettingsDialog::resizeEvent(QResizeEvent* event)
+{
+	if (stack_widget)
+	{
+		for (auto* widget : stack_widget->children())
+		{
+			if (auto* scrollarea = qobject_cast<QScrollArea*>(widget))
+				resizeToFit(*scrollarea);
+		}
+	}
+	QDialog::resizeEvent(event);
+}
+
+void SettingsDialog::resizeToFit(QScrollArea& scrollarea)
+{
+	auto* page = scrollarea.widget();
+	auto size_hint = page->sizeHint();
+	if (!size_hint.isValid())
+		return;
+	
+	// Use QWidget::heightForWidth() when it is available,
+	// otherwise grow, but not shrink, to QWidget::sizeHint()'s height.
+	auto heightForWidth = [&size_hint](const QWidget* widget, int height) {
+		return widget->hasHeightForWidth() ? widget->heightForWidth(height)
+		                                   : std::max(widget->sizeHint().height(), size_hint.height());
+	};
+	auto const table_widgets = page->findChildren<QTableWidget*>(QString{}, Qt::FindDirectChildrenOnly);
+	for (auto const* table_widget : table_widgets)
+	{
+		// Expand to full table widget height
+		auto* model = table_widget->model();
+		auto top = table_widget->visualRect(model->index(0, 0)).top();
+		auto bottom = table_widget->visualRect(model->index(model->rowCount() - 1, 0)).bottom();
+		size_hint.setHeight(size_hint.height() - table_widget->sizeHint().height() + table_widget->horizontalHeader()->height() + bottom - top);
+		size_hint = size_hint.expandedTo(scrollarea.size());
+	}
+	auto size = scrollarea.size();
+	if (size_hint.width() > size.width())
+	{
+		// Fit to width
+		size_hint.setWidth(size.width());
+		size_hint.setHeight(heightForWidth(page, size.width()));
+	}
+	if (size_hint.height() > size.height())
+	{
+		// Fit to width minus scrollbar
+		size_hint.setWidth(std::min(size_hint.width(), size.width() - scrollbar_extent));
+		size_hint.setHeight(heightForWidth(page, size_hint.width()));
+	}
+	page->resize(size_hint);
+}
+
+
 void SettingsDialog::addPages()
 {
 	addPage(new GeneralSettingsPage(this));
@@ -148,20 +218,22 @@ void SettingsDialog::addPages()
 #ifdef MAPPER_USE_GDAL
 	addPage(new GdalSettingsPage(this));
 #endif
+#ifdef MAPPER_USE_SENSORS
+	addPage(new SensorsSettingsPage(this));
+#endif
 }
 
 void SettingsDialog::addPage(SettingsPage* page)
 {
-	if (MainWindow::mobileMode())
+	if (stack_widget)
 	{
 		if (auto form_layout = qobject_cast<QFormLayout*>(page->layout()))
 		{
 			form_layout->setRowWrapPolicy(QFormLayout::WrapAllRows);
-			auto labels = page->findChildren<QLabel*>();
-			for (auto label : qAsConst(labels))
+			auto const labels = page->findChildren<QLabel*>();
+			for (auto* label : labels)
 				label->setWordWrap(true);
 		}
-		page->setMaximumWidth(width());
 		
 		auto scrollarea = new QScrollArea();
 		scrollarea->setFrameShape(QFrame::NoFrame);
@@ -174,9 +246,18 @@ void SettingsDialog::addPage(SettingsPage* page)
 		connect(action, &QAction::triggered, this, [this, scrollarea]()
 		{
 			stack_widget->setCurrentWidget(scrollarea);
+			resizeToFit(*scrollarea);
 			scrollarea->ensureVisible(0, 0);
 			button_box->setStandardButtons(button_box->standardButtons() | QDialogButtonBox::Reset);
 		} );
+		
+		auto const table_widgets = page->findChildren<QTableWidget*>(QString{}, Qt::FindDirectChildrenOnly);
+		for (auto* table_widget : table_widgets)
+		{
+			table_widget->setEditTriggers(QAbstractItemView::SelectedClicked);
+			table_widget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+			connect(table_widget->model(), &QAbstractItemModel::rowsInserted, this, [this, scrollarea]() { resizeToFit(*scrollarea); });
+		}
 	}
 	else
 	{
@@ -186,9 +267,9 @@ void SettingsDialog::addPage(SettingsPage* page)
 
 void SettingsDialog::callOnAllPages(void (SettingsPage::*member)())
 {
-	auto pages = MainWindow::mobileMode() ? stack_widget->findChildren<SettingsPage*>()
-	                                      : tab_widget->findChildren<SettingsPage*>();
-	for (auto page : qAsConst(pages))
+	auto const pages = stack_widget ? stack_widget->findChildren<SettingsPage*>()
+	                                : tab_widget->findChildren<SettingsPage*>();
+	for (auto* page : pages)
 		(page->*member)();
 }
 
@@ -199,7 +280,7 @@ void SettingsDialog::buttonPressed(QAbstractButton* button)
 	{
 	case QDialogButtonBox::Ok:
 		callOnAllPages(&SettingsPage::apply);
-		if (MainWindow::mobileMode() && stack_widget->currentIndex() > 0)
+		if (stack_widget && stack_widget->currentIndex() > 0)
 		{
 			stack_widget->setCurrentIndex(0);
 			button_box->setStandardButtons(button_box->standardButtons() & ~QDialogButtonBox::Reset);
