@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2014 Kai Pastor
+ *    Copyright 2014-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -34,169 +34,40 @@
 #include <QResizeEvent>
 #include <QScopedValueRollback>
 
-#include "core/map_color.h"
+#include "settings.h"
 #include "core/map.h"
 #include "core/objects/object.h"
-#include "../../settings.h"
-#include "core/symbols/symbol.h"
 #include "core/symbols/area_symbol.h"
 #include "core/symbols/combined_symbol.h"
 #include "core/symbols/line_symbol.h"
 #include "core/symbols/point_symbol.h"
-#include "gui/symbols/symbol_setting_dialog.h"
+#include "core/symbols/symbol.h"
+#include "core/symbols/symbol_icon_decorator.h"
 #include "core/symbols/text_symbol.h"
-#include "../../util/overriding_shortcut.h"
-#include "symbol_tooltip.h"
+#include "gui/symbols/symbol_setting_dialog.h"
+#include "gui/widgets/symbol_tooltip.h"
+#include "util/backports.h"  // IWYU pragma: keep
+#include "util/overriding_shortcut.h"
 
 
+namespace OpenOrienteering {
+
 namespace MimeType {
 
 /// The index of a symbol during drag-and-drop
-static const QString oo_symbol_index { QStringLiteral("openorienteering/symbol_index") };
+const QString OpenOrienteeringSymbolIndex()
+{
+	return QStringLiteral("openorienteering/symbol_index");
+}
 
 /// Symbol definitions
-static const QString oo_symbols      { QStringLiteral("openorienteering/symbols") };
-
-}
-
-//### SymbolIconDecorator ###
-
-/**
- * @brief Abstract interface for classes which draws icon decorations.
- * 
- * The icon is expected to be at (0, 0) in the painter's coordinates.
- */
-class SymbolIconDecorator
+const QString OpenOrienteeringSymbols()
 {
-public:
-	virtual ~SymbolIconDecorator();
-	virtual void draw(QPainter& p) const = 0;
-};
-
-SymbolIconDecorator::~SymbolIconDecorator()
-{
-	; // nothing
+	return QStringLiteral("openorienteering/symbols");
 }
 
 
-
-//### ProtectedSymbolDecorator ###
-
-/**
- * @brief Draws the decoration for a protected symbol.
- * 
- * A small gray lock is drawn in the top-right corner of the icon.
- */
-class ProtectedSymbolDecorator : public SymbolIconDecorator
-{
-public:
-	explicit ProtectedSymbolDecorator(int icon_size);
-	virtual ~ProtectedSymbolDecorator();
-	virtual void draw(QPainter& p) const;
-	
-private:
-	int arc_size;
-	int pen_width;
-	int box_width;
-	int box_height;
-	QPoint offset;
-};
-
-ProtectedSymbolDecorator::ProtectedSymbolDecorator(int icon_size)
-: arc_size(qFloor(qMax(3.0, 0.15*icon_size)))
-, pen_width(qMax(1, qCeil(0.4*arc_size)))
-, box_width(arc_size + pen_width + qMax(1, qFloor(0.1*icon_size)))
-, box_height(qMax(arc_size, qCeil(0.6*box_width)))
-, offset(icon_size - 3 - box_width, 1 + pen_width)
-{
-	; // nothing else
-}
-
-ProtectedSymbolDecorator::~ProtectedSymbolDecorator()
-{
-	; // nothing
-}
-
-void ProtectedSymbolDecorator::draw(QPainter& painter) const
-{
-	// Draw a lock symbol
-	painter.save();
-	painter.setRenderHint(QPainter::Antialiasing, true);
-	painter.translate(offset);
-	
-	painter.setOpacity(0.5);
-	QPen arc_background_pen(Qt::white);
-	arc_background_pen.setWidth(pen_width+2);
-	painter.setPen(arc_background_pen);
-	painter.drawRoundedRect(0.5*(box_width-arc_size), 0.0, qreal(arc_size), qreal(arc_size+pen_width), qreal(pen_width), qreal(pen_width));
-	painter.fillRect(-1, arc_size-1, box_width+2, box_height+2, QBrush(Qt::white));
-	
-	painter.setOpacity(1.0);
-	QPen arc_pen(Qt::darkGray);
-	arc_pen.setWidth(pen_width);
-	painter.setPen(arc_pen);
-	painter.drawRoundedRect(0.5*(box_width-arc_size), 0.0, qreal(arc_size), qreal(arc_size+pen_width), qreal(pen_width), qreal(pen_width));
-	painter.fillRect(0, arc_size, box_width, box_height, QBrush(Qt::darkGray));
-	
-	painter.restore();
-}
-
-
-
-//### HiddenSymbolDecorator ###
-
-/**
- * @brief Draws the decoration for a hidden symbol.
- * 
- * A small red x is drawn in the top-left corner of the icon.
- */
-class HiddenSymbolDecorator : public SymbolIconDecorator
-{
-public:
-	explicit HiddenSymbolDecorator(int icon_size);
-	virtual ~HiddenSymbolDecorator();
-	virtual void draw(QPainter& p) const;
-	
-private:
-	int icon_size;
-	int pen_width;
-	int x_width;
-	QPoint offset;
-};
-
-HiddenSymbolDecorator::HiddenSymbolDecorator(int icon_size)
-: icon_size(icon_size)
-, pen_width(qMax(1, qCeil(0.06*icon_size)))
-, x_width(icon_size/3)
-, offset(1 + pen_width, 1 + pen_width)
-{
-	; // nothing else
-}
-
-HiddenSymbolDecorator::~HiddenSymbolDecorator()
-{
-	; // nothing
-}
-
-void HiddenSymbolDecorator::draw(QPainter& painter) const
-{
-	// Draw a lock symbol
-	painter.save();
-	painter.setRenderHint(QPainter::Antialiasing, true);
-	
-	painter.setOpacity(0.6);
-	painter.fillRect(0, 0, icon_size, icon_size, QBrush(Qt::white));
-	
-	painter.translate(offset);
-	painter.setOpacity(1.0);
-	QPen pen(Qt::red);
-	pen.setWidth(pen_width);
-	painter.setPen(pen);
-	painter.drawLine(QPoint(0, 0), QPoint(x_width, x_width));
-	painter.drawLine(QPoint(x_width, 0), QPoint(0, x_width));
-	
-	painter.restore();
-}
+}  // namespace MimeType
 
 
 
@@ -251,8 +122,8 @@ SymbolRenderWidget::SymbolRenderWidget(Map* map, bool mobile_mode, QWidget* pare
 	copy_action = context_menu->addAction(QIcon(QStringLiteral(":/images/copy.png")), tr("Copy"), this, SLOT(copySymbols()));
 	paste_action = context_menu->addAction(QIcon(QStringLiteral(":/images/paste.png")), tr("Paste"), this, SLOT(pasteSymbols()));
 	context_menu->addSeparator();
-	switch_symbol_action = context_menu->addAction(QIcon(QStringLiteral(":/images/tool-switch-symbol.png")), tr("Switch symbol of selected object(s)"), this, SIGNAL(switchSymbolClicked()));
-	fill_border_action = context_menu->addAction(QIcon(QStringLiteral(":/images/tool-fill-border.png")), tr("Fill / Create border for selected object(s)"), this, SIGNAL(fillBorderClicked()));
+	switch_symbol_action = context_menu->addAction(QIcon(QStringLiteral(":/images/tool-switch-symbol.png")), tr("Switch symbol of selected objects"), this, SIGNAL(switchSymbolClicked()));
+	fill_border_action = context_menu->addAction(QIcon(QStringLiteral(":/images/tool-fill-border.png")), tr("Fill / Create border for selected objects"), this, SIGNAL(fillBorderClicked()));
 	// text will be filled in by updateContextMenuState()
 	select_objects_action = context_menu->addAction(QIcon(QStringLiteral(":/images/tool-edit.png")), {}, this, SLOT(selectObjectsExclusively()));
 	select_objects_additionally_action = context_menu->addAction(QIcon(QStringLiteral(":/images/tool-edit.png")), {}, this, SLOT(selectObjectsAdditionally()));
@@ -264,6 +135,9 @@ SymbolRenderWidget::SymbolRenderWidget(Map* map, bool mobile_mode, QWidget* pare
 	protect_action->setCheckable(true);
 	context_menu->addSeparator();
 	
+	show_custom_icons = context_menu->addAction(tr("Show custom icons"), this, SLOT(setCustomIconsVisible(bool)));
+	show_custom_icons->setCheckable(true);
+	
 	QMenu* select_menu = new QMenu(tr("Select symbols"), context_menu);
 	select_menu->addAction(tr("Select all"), this, SLOT(selectAll()));
 	select_menu->addAction(tr("Select unused"), this, SLOT(selectUnused()));
@@ -279,11 +153,13 @@ SymbolRenderWidget::SymbolRenderWidget(Map* map, bool mobile_mode, QWidget* pare
 	sort_manual_action->setCheckable(true);
 	context_menu->addMenu(sort_menu);
 	
-	connect(map, SIGNAL(colorDeleted(int, const MapColor*)), this, SLOT(update()));
-	connect(map, SIGNAL(symbolAdded(int, const Symbol*)), this, SLOT(updateAll()));
-	connect(map, SIGNAL(symbolDeleted(int, const Symbol*)), this, SLOT(symbolDeleted(int, const Symbol*)));
-	connect(map, SIGNAL(symbolChanged(int, const Symbol*, const Symbol*)), this, SLOT(symbolChanged(int, const Symbol*, const Symbol*)));
-	connect(map, SIGNAL(symbolIconChanged(int)), this, SLOT(updateSingleIcon(int)));
+	connect(map, &Map::colorDeleted, this, QOverload<>::of(&QWidget::update));
+	connect(map, &Map::symbolAdded, this, &SymbolRenderWidget::updateAll);
+	connect(map, &Map::symbolDeleted, this, &SymbolRenderWidget::symbolDeleted);
+	connect(map, &Map::symbolChanged, this, &SymbolRenderWidget::symbolChanged);
+	connect(map, &Map::symbolIconChanged, this, &SymbolRenderWidget::updateSingleIcon);
+	connect(map, &Map::symbolIconZoomChanged, this, &SymbolRenderWidget::updateAll);
+	connect(&Settings::getInstance(), &Settings::settingsChanged, this, &SymbolRenderWidget::settingsChanged);
 }
 
 SymbolRenderWidget::~SymbolRenderWidget()
@@ -341,6 +217,23 @@ void SymbolRenderWidget::updateSingleIcon(int i)
 	}
 }
 
+void SymbolRenderWidget::settingsChanged()
+{
+	const auto new_size = Settings::getInstance().getSymbolWidgetIconSizePx();
+	if (icon_size != new_size)
+	{
+		for (int i = 0; i < map->getNumSymbols(); ++i)
+		{
+			auto symbol = map->getSymbol(i);
+			if (symbol->getIcon(map).width() != new_size)
+				symbol->resetIcon();
+		}
+		updateAll();
+	}
+}
+
+
+
 QSize SymbolRenderWidget::sizeHint() const
 {
 	return preferred_size;
@@ -348,15 +241,19 @@ QSize SymbolRenderWidget::sizeHint() const
 
 void SymbolRenderWidget::adjustLayout()
 {
-	icon_size = Settings::getInstance().getSymbolWidgetIconSizePx();
+	auto old_icon_size = icon_size;
+	icon_size = Settings::getInstance().getSymbolWidgetIconSizePx() + 1;
 	// Allow symbol widget to be that much wider than the viewport
 	int overflow = icon_size / 3;
 	icons_per_row = qMax(1, (width() + overflow) / icon_size);
 	num_rows = qMax(1, (map->getNumSymbols() + icons_per_row -1) / icons_per_row);
 	setFixedHeight(num_rows * icon_size);
 	
-	hidden_symbol_decoration.reset(new HiddenSymbolDecorator(icon_size));
-	protected_symbol_decoration.reset(new ProtectedSymbolDecorator(icon_size));
+	if (old_icon_size != icon_size)
+	{
+		hidden_symbol_decoration.reset(new HiddenSymbolDecorator(icon_size));
+		protected_symbol_decoration.reset(new ProtectedSymbolDecorator(icon_size));
+	}
 }
 
 void SymbolRenderWidget::emitGuarded_selectedSymbolsChanged()
@@ -369,14 +266,14 @@ void SymbolRenderWidget::emitGuarded_selectedSymbolsChanged()
 const Symbol* SymbolRenderWidget::singleSelectedSymbol() const
 {
 	if (selected_symbols.size() != 1)
-		return NULL;
+		return nullptr;
 	return static_cast<const Map*>(map)->getSymbol(*(selected_symbols.begin()));
 }
 
 Symbol* SymbolRenderWidget::singleSelectedSymbol()
 {
 	if (selected_symbols.size() != 1)
-		return NULL;
+		return nullptr;
 	return map->getSymbol(*(selected_symbols.begin()));
 }
 
@@ -417,7 +314,7 @@ void SymbolRenderWidget::selectSingleSymbol(int i)
 	emitGuarded_selectedSymbolsChanged();
 }
 
-void SymbolRenderWidget::hover(QPoint pos)
+void SymbolRenderWidget::hover(const QPoint& pos)
 {
 	int i = symbolIndexAt(pos);
 	
@@ -433,7 +330,7 @@ void SymbolRenderWidget::hover(QPoint pos)
 			if (tooltip->getSymbol() != symbol)
 			{
 				const QRect icon_rect(mapToGlobal(iconPosition(hover_symbol_index)), QSize(icon_size, icon_size));
-				tooltip->scheduleShow(symbol, icon_rect);
+				tooltip->scheduleShow(symbol, map, icon_rect, mobile_mode);
 			}
 		}
 		else
@@ -448,7 +345,7 @@ QPoint SymbolRenderWidget::iconPosition(int i) const
 	return QPoint((i % icons_per_row) * icon_size, (i / icons_per_row) * icon_size);
 }
 
-int SymbolRenderWidget::symbolIndexAt(QPoint pos) const
+int SymbolRenderWidget::symbolIndexAt(const QPoint& pos) const
 {
 	int i = -1;
 	
@@ -465,11 +362,11 @@ int SymbolRenderWidget::symbolIndexAt(QPoint pos) const
 
 void SymbolRenderWidget::updateSelectedIcons()
 {
-	for (std::set<int>::const_iterator it = selected_symbols.begin(); it != selected_symbols.end(); ++it)
-		updateSingleIcon(*it);
+	for (auto symbol_index : selected_symbols)
+		updateSingleIcon(symbol_index);
 }
 
-bool SymbolRenderWidget::dropPosition(QPoint pos, int& row, int& pos_in_row)
+bool SymbolRenderWidget::dropPosition(const QPoint& pos, int& row, int& pos_in_row)
 {
 	row = pos.y() / icon_size;
 	if (row >= num_rows)
@@ -621,6 +518,8 @@ void SymbolRenderWidget::mouseMoveEvent(QMouseEvent* event)
 	{
 		if (event->buttons() & Qt::LeftButton)
 		{
+			hover(event->pos());
+			
 			if ((event->pos() - last_click_pos).manhattanLength() < Settings::getInstance().getStartDragDistancePx())
 				return;
 			dragging = sort_manual_action->isChecked();
@@ -642,7 +541,7 @@ void SymbolRenderWidget::mouseMoveEvent(QMouseEvent* event)
 			
 			QByteArray data;
 			data.append((const char*)&current_symbol_index, sizeof(int));
-			mime_data->setData(MimeType::oo_symbol_index, data);
+			mime_data->setData(MimeType::OpenOrienteeringSymbolIndex(), data);
 			drag->setMimeData(mime_data);
 			
 			drag->exec(Qt::MoveAction);
@@ -661,7 +560,10 @@ void SymbolRenderWidget::mousePressEvent(QMouseEvent* event)
 	
 	if (mobile_mode)
 	{
+		tooltip->hide();
+		
 		last_click_pos = event->pos();
+		hover(event->pos());
 		return;
 	}
 	
@@ -734,6 +636,10 @@ void SymbolRenderWidget::mouseReleaseEvent(QMouseEvent* event)
 			return;
 		}
 		
+		hover(event->pos());
+		if (tooltip->isVisible())
+			return;
+		
 		updateSingleIcon(current_symbol_index);
 		current_symbol_index = symbolIndexAt(event->pos());
 		updateSingleIcon(current_symbol_index);
@@ -771,13 +677,13 @@ void SymbolRenderWidget::leaveEvent(QEvent* event)
 
 void SymbolRenderWidget::dragEnterEvent(QDragEnterEvent* event)
 {
-	if (event->mimeData()->hasFormat(MimeType::oo_symbol_index))
+	if (event->mimeData()->hasFormat(MimeType::OpenOrienteeringSymbolIndex()))
 		event->acceptProposedAction();
 }
 
 void SymbolRenderWidget::dragMoveEvent(QDragMoveEvent* event)
 {
-	if (event->mimeData()->hasFormat(MimeType::oo_symbol_index))
+	if (event->mimeData()->hasFormat(MimeType::OpenOrienteeringSymbolIndex()))
 	{
 		int row, pos_in_row;
 		if (!dropPosition(event->pos(), row, pos_in_row))
@@ -829,8 +735,8 @@ void SymbolRenderWidget::dropEvent(QDropEvent* event)
 		
         // save selection
         std::set<Symbol *> sel;
-        for (std::set<int>::const_iterator it = selected_symbols.begin(); it != selected_symbols.end(); ++it) {
-            sel.insert(map->getSymbol(*it));
+        for (auto symbol_index : selected_symbols) {
+            sel.insert(map->getSymbol(symbol_index));
         }
 
 		map->moveSymbol(current_symbol_index, pos);
@@ -893,8 +799,7 @@ void SymbolRenderWidget::editSymbol()
 	dialog.setWindowModality(Qt::WindowModal);
 	if (dialog.exec() == QDialog::Accepted)
 	{
-		symbol = dialog.getNewSymbol();
-		map->setSymbol(symbol, current_symbol_index);
+		map->setSymbol(dialog.getNewSymbol().release(), current_symbol_index);
 	}
 }
 
@@ -903,14 +808,13 @@ void SymbolRenderWidget::scaleSymbol()
 	Q_ASSERT(!selected_symbols.empty());
 	
 	bool ok;
-	double percent = QInputDialog::getDouble(this, tr("Scale symbol(s)"), tr("Scale to percentage:"), 100, 0, 999999, 6, &ok);
+	double percent = QInputDialog::getDouble(this, tr("Scale symbols"), tr("Scale to percentage:"), 100, 0, 999999, 6, &ok);
 	if (!ok || percent == 100)
 		return;
 	
-	for (std::set<int>::const_iterator it = selected_symbols.begin(); it != selected_symbols.end(); ++it)
+	for (auto symbol_index : selected_symbols)
 	{
-		Symbol* symbol = map->getSymbol(*it);
-		
+		auto symbol = map->getSymbol(symbol_index);
 		symbol->scale(percent / 100.0);
 		updateSingleIcon(current_symbol_index);
 		map->changeSymbolForAllObjects(symbol, symbol);	// update the objects
@@ -921,34 +825,59 @@ void SymbolRenderWidget::scaleSymbol()
 
 void SymbolRenderWidget::deleteSymbols()
 {
-	// save selected symbols
+	// Collect selected symbols
 	std::vector<const Symbol*> saved_selection;
 	saved_selection.reserve(selected_symbols.size());
-	for (std::set<int>::const_iterator it = selected_symbols.begin(); it != selected_symbols.end(); ++it)
-	{
-		saved_selection.push_back(map->getSymbol(*it));
-	}
 	
-	// delete symbols in order
-	for (std::vector<const Symbol*>::iterator it = saved_selection.begin(); it != saved_selection.end(); ++it)
+	bool need_confirmation = true;
+	for (auto symbol_index : selected_symbols)
 	{
-		if (map->existsObjectWithSymbol(*it))
+		auto* symbol = map->getSymbol(symbol_index);
+		if (need_confirmation && map->existsObjectWithSymbol(symbol))
 		{
-			if (QMessageBox::warning(this, tr("Confirmation"), tr("The map contains objects with the symbol \"%1\". Deleting it will delete those objects and clear the undo history! Do you really want to do that?").arg((*it)->getName()), QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
+			auto answer = QMessageBox::warning(
+			                  this,
+			                  tr("Confirmation"),
+			                  tr("The map contains objects with the symbol \"%1\". "
+			                     "Deleting it will delete those objects and clear the undo history! "
+			                     "Do you really want to do that?")
+			                  .arg(symbol->getName()),
+			                  QMessageBox::Yes | QMessageBox::YesToAll | QMessageBox::No | QMessageBox::Cancel);
+			switch (answer)
+			{
+			case QMessageBox::Cancel:
+				return;
+			case QMessageBox::YesToAll:
+				need_confirmation = false;
+				break;
+			case QMessageBox::Yes:
+				break;
+			case QMessageBox::No:
 				continue;
+			default:
+				Q_UNREACHABLE();
+			}
 		}
-		map->deleteSymbol(map->findSymbolIndex(*it));
+		saved_selection.push_back(symbol);
 	}
 	
-	if (selected_symbols.empty() && map->getFirstSelectedObject())
-		selectSingleSymbol(map->getFirstSelectedObject()->getSymbol());
+	// Delete collected symbols
+	for (auto* symbol : saved_selection)
+		map->deleteSymbol(map->findSymbolIndex(symbol));
+	
+	if (selected_symbols.empty()) {
+		if (map->getFirstSelectedObject())
+			selectSingleSymbol(map->getFirstSelectedObject()->getSymbol());
+		else
+			selectSingleSymbol(-1);
+	}
 }
 
 void SymbolRenderWidget::duplicateSymbol()
 {
 	Q_ASSERT(current_symbol_index >= 0);
 	
-	map->addSymbol(map->getSymbol(current_symbol_index)->duplicate(), current_symbol_index + 1);
+	map->addSymbol(duplicate(*map->getSymbol(current_symbol_index)).release(), current_symbol_index + 1);
 	selectSingleSymbol(current_symbol_index + 1);
 }
 
@@ -956,60 +885,74 @@ void SymbolRenderWidget::copySymbols()
 {
 	// Create map containing all colors and the selected symbols.
 	// Copying all colors improves preservation of relative order during paste.
-	Map* copy_map = new Map();
-	copy_map->setScaleDenominator(map->getScaleDenominator());
-
-	copy_map->importMap(map, Map::ColorImport, this);
+	Map copy_map;
+	copy_map.setScaleDenominator(map->getScaleDenominator());
+	copy_map.importMap(*map, Map::ColorImport);
 	
 	std::vector<bool> selection(map->getNumSymbols(), false);
-	for (std::set<int>::const_iterator it = selected_symbols.begin(); it != selected_symbols.end(); ++it)
-		selection[*it] = true;
+	for (auto symbol_index : selected_symbols)
+		selection[symbol_index] = true;
 	
-	copy_map->importMap(map, Map::MinimalSymbolImport, this, &selection);
+	copy_map.importMap(*map, Map::MinimalSymbolImport, &selection);
 	
 	// Save map to memory
 	QBuffer buffer;
-	if (!copy_map->exportToIODevice(&buffer))
+	if (!copy_map.exportToIODevice(buffer))
 	{
-		delete copy_map;
-		QMessageBox::warning(NULL, tr("Error"), tr("An internal error occurred, sorry!"));
+		QMessageBox::warning(nullptr, tr("Error"), tr("An internal error occurred, sorry!"));
 		return;
 	}
-	delete copy_map;
 	
 	// Put buffer into clipboard
 	QMimeData* mime_data = new QMimeData();
-	mime_data->setData(MimeType::oo_symbols, buffer.data());
+	mime_data->setData(MimeType::OpenOrienteeringSymbols(), buffer.data());
 	QApplication::clipboard()->setMimeData(mime_data);
 }
 
 void SymbolRenderWidget::pasteSymbols()
 {
-	if (!QApplication::clipboard()->mimeData()->hasFormat(MimeType::oo_symbols))
+	if (!QApplication::clipboard()->mimeData()->hasFormat(MimeType::OpenOrienteeringSymbols()))
 	{
-		QMessageBox::warning(NULL, tr("Error"), tr("There are no symbols in clipboard which could be pasted!"));
+		QMessageBox::warning(nullptr, tr("Error"), tr("There are no symbols in clipboard which could be pasted!"));
 		return;
 	}
 	
 	// Get buffer from clipboard
-	QByteArray byte_array = QApplication::clipboard()->mimeData()->data(MimeType::oo_symbols);
+	QByteArray byte_array = QApplication::clipboard()->mimeData()->data(MimeType::OpenOrienteeringSymbols());
 	QBuffer buffer(&byte_array);
-	buffer.open(QIODevice::ReadOnly);
 	
 	// Create map from buffer
-	Map* paste_map = new Map();
-	if (!paste_map->importFromIODevice(&buffer))
+	Map paste_map;
+	if (!paste_map.importFromIODevice(buffer))
 	{
-		QMessageBox::warning(NULL, tr("Error"), tr("An internal error occurred, sorry!"));
+		QMessageBox::warning(nullptr, tr("Error"), tr("An internal error occurred, sorry!"));
 		return;
 	}
 	
+	if (paste_map.getScaleDenominator() != map->getScaleDenominator())
+	{
+		/// \todo Adjust message to this particular action, and remove Map context.
+		///       See also MapEditor::importMap.
+		int answer = QMessageBox::question(
+		                 window(),
+		                 Map::tr("Question"),
+		                 Map::tr("The scale of the imported data is 1:%1 "
+		                         "which is different from this map's scale of 1:%2.\n\n"
+		                         "Rescale the imported data?")
+		                 .arg(QLocale().toString(paste_map.getScaleDenominator()),
+		                      QLocale().toString(map->getScaleDenominator())),
+		                 QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel, QMessageBox::Yes);
+		if (answer == QMessageBox::Cancel)
+			return;
+		
+		paste_map.changeScale(map->getScaleDenominator(), {0, 0}, answer == QMessageBox::Yes, false, false, false);
+	}
+	
 	// Ensure that a change in selection is detected
 	selectSingleSymbol(-1);
-	
+		
 	// Import pasted map
-	map->importMap(paste_map, Map::MinimalSymbolImport, this, NULL, current_symbol_index, false);
-	delete paste_map;
+	map->importMap(paste_map, Map::MinimalSymbolImport, nullptr, current_symbol_index, false);
 	
 	selectSingleSymbol(current_symbol_index);
 }
@@ -1017,13 +960,13 @@ void SymbolRenderWidget::pasteSymbols()
 void SymbolRenderWidget::setSelectedSymbolVisibility(bool checked)
 {
 	bool selection_changed = false;
-	for (std::set<int>::const_iterator it = selected_symbols.begin(); it != selected_symbols.end(); ++it)
+	for (auto symbol_index : selected_symbols)
 	{
-		Symbol* symbol = map->getSymbol(*it);
+		auto symbol = map->getSymbol(symbol_index);
 		if (symbol->isHidden() != checked)
 		{
 			symbol->setHidden(checked);
-			updateSingleIcon(*it);
+			updateSingleIcon(symbol_index);
 			if (checked)
 				selection_changed |= map->removeSymbolFromSelection(symbol, false);
 		}
@@ -1034,16 +977,17 @@ void SymbolRenderWidget::setSelectedSymbolVisibility(bool checked)
 	map->setSymbolsDirty();
 	emitGuarded_selectedSymbolsChanged();
 }
+
 void SymbolRenderWidget::setSelectedSymbolProtection(bool checked)
 {
 	bool selection_changed = false;
-	for (std::set<int>::const_iterator it = selected_symbols.begin(); it != selected_symbols.end(); ++it)
+	for (auto symbol_index : selected_symbols)
 	{
-		Symbol* symbol = map->getSymbol(*it);
+		auto symbol = map->getSymbol(symbol_index);
 		if (symbol->isProtected() != checked)
 		{
 			symbol->setProtected(checked);
-			updateSingleIcon(*it);
+			updateSingleIcon(symbol_index);
 			if (checked)
 				selection_changed |= map->removeSymbolFromSelection(symbol, false);
 		}
@@ -1095,20 +1039,31 @@ void SymbolRenderWidget::invertSelection()
 
 void SymbolRenderWidget::sortByNumber()
 {
-    sort(Symbol::compareByNumber);
+    sort(Symbol::lessByNumber);
 }
 
 void SymbolRenderWidget::sortByColor()
 {
-	sort(Symbol::compareByColor(map));
+	sort(Symbol::lessByColor(map));
 }
 
 void SymbolRenderWidget::sortByColorPriority()
 {
-	sort(Symbol::compareByColorPriority);
+	sort(Symbol::lessByColorPriority);
 }
 
-void SymbolRenderWidget::showContextMenu(QPoint global_pos)
+void SymbolRenderWidget::setCustomIconsVisible(bool checked)
+{
+	for (int i = 0; i < map->getNumSymbols(); ++i)
+	{
+		auto symbol = map->getSymbol(i);
+		if (!symbol->getCustomIcon().isNull())
+			symbol->resetIcon();
+	}
+	Settings::getInstance().setSetting(Settings::SymbolWidget_ShowCustomIcons, checked);
+}
+
+void SymbolRenderWidget::showContextMenu(const QPoint& global_pos)
 {
 	updateContextMenuState();
 	context_menu->popup(global_pos);
@@ -1127,11 +1082,11 @@ void SymbolRenderWidget::updateContextMenuState()
 	const Symbol* single_symbol = singleSelectedSymbol();
 	bool all_symbols_hidden = have_selection;
 	bool all_symbols_protected = have_selection;
-	for (std::set<int>::const_iterator it = selected_symbols.begin(); it != selected_symbols.end(); ++it)
+	for (auto symbol_index : selected_symbols)
 	{
-		if (!map->getSymbol(*it)->isHidden())
+		if (!map->getSymbol(symbol_index)->isHidden())
 			all_symbols_hidden = false;
-		if (!map->getSymbol(*it)->isProtected())
+		if (!map->getSymbol(symbol_index)->isProtected())
 			all_symbols_protected = false;
 		if (!all_symbols_hidden && !all_symbols_protected)
 			break;
@@ -1144,7 +1099,7 @@ void SymbolRenderWidget::updateContextMenuState()
 	edit_action->setEnabled(single_selection);
 	scale_action->setEnabled(have_selection);
 	copy_action->setEnabled(have_selection);
-	paste_action->setEnabled(QApplication::clipboard()->mimeData()->hasFormat(MimeType::oo_symbols));
+	paste_action->setEnabled(QApplication::clipboard()->mimeData()->hasFormat(MimeType::OpenOrienteeringSymbols()));
 	switch_symbol_action->setEnabled(single_symbol_compatible && single_symbol_different);
 	fill_border_action->setEnabled(single_symbol_compatible && single_symbol_different);
 	hide_action->setEnabled(have_selection);
@@ -1173,6 +1128,8 @@ void SymbolRenderWidget::updateContextMenuState()
 	select_objects_action->setEnabled(have_selection);
 	select_objects_additionally_action->setEnabled(have_selection);
 	deselect_objects_action->setEnabled(have_selection);
+	
+	show_custom_icons->setChecked(Settings::getInstance().getSetting(Settings::SymbolWidget_ShowCustomIcons).toBool());
 }
 
 bool SymbolRenderWidget::newSymbol(Symbol* prototype)
@@ -1183,9 +1140,8 @@ bool SymbolRenderWidget::newSymbol(Symbol* prototype)
 	if (dialog.exec() == QDialog::Rejected)
 		return false;
 	
-	Symbol* new_symbol = dialog.getNewSymbol();
 	int pos = (current_symbol_index >= 0) ? current_symbol_index : map->getNumSymbols();
-	map->addSymbol(new_symbol, pos);
+	map->addSymbol(dialog.getNewSymbol().release(), pos);
 	// Ensure that a change in selection is detected
 	selectSingleSymbol(-1);
 	selectSingleSymbol(pos);
@@ -1197,9 +1153,9 @@ void SymbolRenderWidget::sort(T compare)
 {
 	// save selection
 	std::set<const Symbol*> saved_selection;
-	for (std::set<int>::const_iterator it = selected_symbols.begin(); it != selected_symbols.end(); ++it)
+	for (auto symbol_index : selected_symbols)
 	{
-		saved_selection.insert(map->getSymbol(*it));
+		saved_selection.insert(map->getSymbol(symbol_index));
 	}
 	
 	map->sortSymbols(compare);
@@ -1214,3 +1170,6 @@ void SymbolRenderWidget::sort(T compare)
 	
 	update();
 }
+
+
+}  // namespace OpenOrienteering
