@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2017 Kai Pastor
+ *    Copyright 2012-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -21,36 +21,38 @@
 
 #include <clocale>
 #include <memory>
+#include <utility>
+// IWYU pragma: no_include <type_traits>
 
 #include <Qt>
 #include <QtGlobal>
+#include <QtPlugin>  // IWYU pragma: keep
 #include <QApplication>
+#include <QCoreApplication>
+#include <QGuiApplication>
 #include <QLatin1String>
+#include <QList>
 #include <QLocale>
 #include <QObject>
+#include <QPalette>
 #include <QPointer>
-#include <QSettings>  // IWYU pragma: keep
+#include <QSettings>
 #include <QString>
 #include <QStringList>
-#include <QStyle>
-#include <QStyleFactory>
 #include <QTranslator>
+#include <QWidget>
 
-#include <mapper_config.h>
-
-#if defined(QT_NETWORK_LIB)
-#  define MAPPER_USE_QTSINGLEAPPLICATION 1
-#  include <QtSingleApplication>  // IWYU pragma: keep
-#else
-#  define MAPPER_USE_QTSINGLEAPPLICATION 0
+#ifdef MAPPER_USE_QTSINGLEAPPLICATION
+#include <QtSingleApplication>
+#include <QFileInfo>
 #endif
 
 #include "global.h"
+#include "mapper_config.h"
 #include "mapper_resource.h"
 #include "gui/home_screen_controller.h"
 #include "gui/main_window.h"
 #include "gui/widgets/mapper_proxystyle.h"
-#include "util/backports.h"
 #include "util/recording_translator.h"  // IWYU pragma: keep
 #include "util/translation_util.h"
 
@@ -59,6 +61,11 @@
 using namespace OpenOrienteering;
 
 
+#if defined(Q_OS_WIN32) && defined(MAPPER_USE_POWERSHELL_POSITION_PLUGIN)
+Q_IMPORT_PLUGIN(PowershellPositionPlugin)
+#endif
+
+
 // From map.h
 
 namespace OpenOrienteering {
@@ -68,26 +75,32 @@ extern QPointer<QTranslator> map_symbol_translator;
 }  // namespace OpenOrienteering
 
 
-
-#if MAPPER_USE_QTSINGLEAPPLICATION
-
-void resetActivationWindow()
+QStringList firstRemoved(QStringList&& input)
 {
-	auto app = qobject_cast<QtSingleApplication*>(qApp);
-	app->setActivationWindow(nullptr);
+	if (!input.empty())
+		input.removeFirst();
+	return std::move(input);
+}
+
+
+#ifdef MAPPER_USE_QTSINGLEAPPLICATION
+
+void resetActivationWindow(QtSingleApplication& app)
+{
+	const auto* const old_window = app.activationWindow();
+	app.setActivationWindow(nullptr);
 	
-	if (!app->closingDown())
+	if (!QCoreApplication::closingDown())
 	{
-		const auto old_window = app->activationWindow();
-		const auto top_level_widgets = app->topLevelWidgets();
-		for (const auto widget : top_level_widgets)
+		const auto top_level_widgets = QApplication::topLevelWidgets();
+		for (auto* widget : top_level_widgets)
 		{	
-			auto new_window = qobject_cast<MainWindow*>(widget);
+			auto* const new_window = qobject_cast<MainWindow*>(widget);
 			if (new_window && new_window != old_window)
 			{
-				app->setActivationWindow(new_window);
-				QObject::connect(new_window, &QObject::destroyed, &resetActivationWindow);
-				QObject::connect(app, &QtSingleApplication::messageReceived, new_window, &MainWindow::openPath);
+				app.setActivationWindow(new_window);
+				QObject::connect(new_window, &QObject::destroyed, &app, [&app]() { resetActivationWindow(app); });
+				QObject::connect(&app, &QtSingleApplication::messageReceived, new_window, &MainWindow::openPathLater);
 				break;
 			}
 		}
@@ -99,30 +112,36 @@ void resetActivationWindow()
 
 int main(int argc, char** argv)
 {
-#if MAPPER_USE_QTSINGLEAPPLICATION
+#ifdef MAPPER_USE_QTSINGLEAPPLICATION
 	// Create single-instance application.
 	// Use "oo-mapper" instead of the executable as identifier, in case we launch from different paths.
 	QtSingleApplication qapp(QString::fromLatin1("oo-mapper"), argc, argv);
 	if (qapp.isRunning()) {
 		// Send a message to activate the running app, and optionally open a file
-		qapp.sendMessage(QString::fromUtf8((argc > 1) ? argv[1] : ""));
+		auto const arguments = firstRemoved(QCoreApplication::arguments());
+		for (auto const& arg : arguments)
+			qapp.sendMessage(QFileInfo(arg).absoluteFilePath());
 		return 0;
 	}
 #else
 	QApplication qapp(argc, argv);
 #endif
 	
+#ifdef Q_OS_ANDROID
+	qputenv("QT_USE_ANDROID_NATIVE_STYLE", "1");
+#endif
+	
 	// Load resources
 	Q_INIT_RESOURCE(resources);
 	
 	// QSettings on OS X benefits from using an internet domain here.
-	QApplication::setOrganizationName(QString::fromLatin1("OpenOrienteering.org"));
-	QApplication::setApplicationName(QString::fromLatin1("Mapper"));
-	qapp.setApplicationDisplayName(APP_NAME + QString::fromUtf8(" " APP_VERSION));
+	QCoreApplication::setOrganizationName(QString::fromLatin1("OpenOrienteering.org"));
+	QCoreApplication::setApplicationName(QString::fromLatin1("Mapper"));
+	QGuiApplication::setApplicationDisplayName(APP_NAME + QString::fromUtf8(" " APP_VERSION));
 	
 #ifdef WIN32
 	// Load plugins on Windows
-	qapp.addLibraryPath(QCoreApplication::applicationDirPath() + QLatin1String("/plugins"));
+	QCoreApplication::addLibraryPath(QCoreApplication::applicationDirPath() + QLatin1String("/plugins"));
 #endif
 	
 	MapperResource::setSeachPaths();
@@ -134,7 +153,7 @@ int main(int argc, char** argv)
 	QLocale::setDefault(QLocale(translation.code()));
 #if defined(Q_OS_MACOS)
 	// Normally this is done in Settings::apply() because it is too late here.
-	// But Mapper 0.6.2/0.6.3 accidently wrote a string instead of a list. This
+	// But Mapper 0.6.2/0.6.3 accidentally wrote a string instead of a list. This
 	// error caused crashes when opening native dialogs (i.e. the open-file dialog!).
 	settings.setValue(QString::fromLatin1("AppleLanguages"), QStringList{ translation.code() });
 #endif
@@ -142,11 +161,11 @@ int main(int argc, char** argv)
 	if (!translation.getAppTranslator().isEmpty())
 	{
 		// Debug translation only if there is a Mapper translation, i.e. not for English.
-		qapp.installTranslator(new RecordingTranslator());
+		QCoreApplication::installTranslator(new RecordingTranslator());
 	}
 #endif
-	qapp.installTranslator(&translation.getQtTranslator());
-	qapp.installTranslator(&translation.getAppTranslator());
+	QCoreApplication::installTranslator(&translation.getQtTranslator());
+	QCoreApplication::installTranslator(&translation.getAppTranslator());
 	map_symbol_translator = translation.load(QString::fromLatin1("map_symbols")).release();
 	if (map_symbol_translator)
 		map_symbol_translator->setParent(&qapp);
@@ -157,19 +176,9 @@ int main(int argc, char** argv)
 	// Initialize static things like the file format registry.
 	doStaticInitializations();
 	
-	QStyle* base_style = nullptr;
-#if !defined(Q_OS_WIN) && !defined(Q_OS_MACOS)
-	if (QApplication::platformName() == QLatin1String("xcb"))
-	{
-		// Use the modern 'fusion' style instead of the 
-		// default "windows" style on X11.
-		base_style = QStyleFactory::create(QString::fromLatin1("fusion"));
-	}
-#endif
-	QApplication::setStyle(new MapperProxyStyle(base_style));
-#if !defined(Q_OS_MACOS)
-	QApplication::setPalette(QApplication::style()->standardPalette());
-#endif
+	auto const palette = QApplication::palette();
+	QApplication::setStyle(new MapperProxyStyle());
+	QApplication::setPalette(palette);
 	
 	// Create first main window
 	auto first_window = new MainWindow();
@@ -181,21 +190,18 @@ int main(int argc, char** argv)
 	// top of a regular main window (home screen or other file).
 	
 	// Treat all program parameters as files to be opened
-	QStringList args(qapp.arguments());
-	args.removeFirst(); // the program name
-	for (const auto& arg : qAsConst(args))
-	{
+	auto const arguments = firstRemoved(QCoreApplication::arguments());
+	for (auto const& arg : arguments)
 		first_window->openPathLater(arg);
-	}
 	
 	first_window->applicationStateChanged();
 	
-#if MAPPER_USE_QTSINGLEAPPLICATION
-	resetActivationWindow();
+#ifdef MAPPER_USE_QTSINGLEAPPLICATION
+	resetActivationWindow(qapp);
 #endif
 	
 	// Let application run
 	first_window->setVisible(true);
 	first_window->raise();
-	return qapp.exec();
+	return QApplication::exec();
 }
