@@ -24,8 +24,8 @@
  */
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ****************************************************************************/
 
@@ -58,6 +58,7 @@
 #include "advanced_pdf_p.h"
 #include <private/qpaintengine_p.h>
 #include "qprintengine.h"
+#include <private/qprint_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -67,16 +68,6 @@ class QPen;
 class QPointF;
 class QRegion;
 class QFile;
-class AdvancedPdfPrintEngine;
-
-namespace AdvancedPdf {
-
-    struct PaperSize {
-        int width, height; // in postscript points
-    };
-    PaperSize paperSize(QPrinter::PaperSize paperSize);
-    const char *paperSizeToString(QPrinter::PaperSize paperSize);
-}
 
 class AdvancedPdfPrintEnginePrivate;
 
@@ -85,21 +76,21 @@ class AdvancedPdfPrintEngine : public AdvancedPdfEngine, public QPrintEngine
     Q_DECLARE_PRIVATE(AdvancedPdfPrintEngine)
 public:
     AdvancedPdfPrintEngine(QPrinter::PrinterMode m);
-    virtual ~AdvancedPdfPrintEngine();
+    ~AdvancedPdfPrintEngine() override;
 
     // reimplementations QPaintEngine
-    bool begin(QPaintDevice *pdev);
-    bool end();
+    bool begin(QPaintDevice *pdev) Q_DECL_OVERRIDE;
+    bool end() Q_DECL_OVERRIDE;
     // end reimplementations QPaintEngine
 
     // reimplementations QPrintEngine
-    bool abort() {return false;}
-    QPrinter::PrinterState printerState() const {return state;}
+    bool abort() Q_DECL_OVERRIDE {return false;}
+    QPrinter::PrinterState printerState() const Q_DECL_OVERRIDE {return state;}
 
-    bool newPage();
-    int metric(QPaintDevice::PaintDeviceMetric) const;
-    virtual void setProperty(PrintEnginePropertyKey key, const QVariant &value);
-    virtual QVariant property(PrintEnginePropertyKey key) const;
+    bool newPage() Q_DECL_OVERRIDE;
+    int metric(QPaintDevice::PaintDeviceMetric) const Q_DECL_OVERRIDE;
+    void setProperty(PrintEnginePropertyKey key, const QVariant &value) Q_DECL_OVERRIDE;
+    QVariant property(PrintEnginePropertyKey key) const Q_DECL_OVERRIDE;
     // end reimplementations QPrintEngine
 
     QPrinter::PrinterState state;
@@ -116,13 +107,11 @@ class AdvancedPdfPrintEnginePrivate : public AdvancedPdfEnginePrivate
     Q_DECLARE_PUBLIC(AdvancedPdfPrintEngine)
 public:
     AdvancedPdfPrintEnginePrivate(QPrinter::PrinterMode m);
-    ~AdvancedPdfPrintEnginePrivate();
+    ~AdvancedPdfPrintEnginePrivate() override;
 
     virtual bool openPrintDevice();
     virtual void closePrintDevice();
 
-    virtual void updatePaperSize();
-
 private:
     Q_DISABLE_COPY(AdvancedPdfPrintEnginePrivate)
 
@@ -133,15 +122,12 @@ private:
     QString printProgram;
     QString selectionOption;
 
-    QPrinter::DuplexMode duplex;
+    QPrint::DuplexMode duplex;
     bool collate;
     int copies;
     QPrinter::PageOrder pageOrder;
     QPrinter::PaperSource paperSource;
 
-    QPrinter::PaperSize printerPaperSize;
-    QSizeF customPaperSize; // in postscript points
-    bool pageMarginsSet;
     int fd;
 };
 
