@@ -24,15 +24,12 @@
 
 #include <QtGlobal>
 #include <QObject>
-#if defined(QT_POSITIONING_LIB)
-#  include <QGeoPositionInfo>  // IWYU pragma: keep
-#  include <QGeoPositionInfoSource>  // IWYU pragma: keep
-#else
-class QGeoPositionInfoSource;  // IWYU pragma: keep
-#endif
+#include <QString>
 
 #include "core/map_coord.h"
 
+class QGeoPositionInfo;
+class QGeoPositionInfoSource;
 class QPainter;
 class QTimerEvent;
 
@@ -109,7 +106,7 @@ protected:
 signals:
 	/// Is emitted whenever a new position update happens.
 	/// If the accuracy is unknown, -1 will be given.
-	void mapPositionUpdated(MapCoordF coord, float accuracy);
+	void mapPositionUpdated(const OpenOrienteering::MapCoordF& coord, float accuracy);
 	
 	/// Like mapPositionUpdated(), but gives the values as
 	/// latitude / longitude in degrees and also gives altitude
@@ -122,11 +119,9 @@ signals:
 	void positionUpdatesInterrupted();
 	
 private slots:
-#if defined(QT_POSITIONING_LIB)
     void positionUpdated(const QGeoPositionInfo& info);
-	void error(QGeoPositionInfoSource::Error positioningError);
+	void error();
 	void updateTimeout();
-#endif
 	void debugPositionUpdate();
 	
 private:
@@ -166,9 +161,6 @@ private:
 	MapWidget* widget;
 	const Georeferencing& georeferencing;
 	QGeoPositionInfoSource* source = nullptr;
-#if defined(QT_POSITIONING_LIB)
-	QGeoPositionInfo latest_pos_info;
-#endif
 	MapCoordF latest_gps_coord;
 	float latest_gps_coord_accuracy = 0;
 	PulsatingOpacity pulsating_opacity;
