@@ -21,9 +21,14 @@
 
 #include "settings.h"
 
+#include <initializer_list>
+
 #include <QtGlobal>
+#if defined(QT_WIDGETS_LIB)
 #include <QApplication>
+#endif
 #include <QByteArray>
+#include <QGuiApplication>
 #include <QLatin1String>
 #include <QLocale>
 #include <QScreen>
@@ -58,13 +63,13 @@ namespace Util
 	
 	qreal mmToPixelLogical(qreal millimeters)
 	{
-		auto ppi = QApplication::primaryScreen()->logicalDotsPerInch();
+		auto ppi = QGuiApplication::primaryScreen()->logicalDotsPerInch();
 		return millimeters * ppi / qreal(25.4);
 	}
 	
 	qreal pixelToMMLogical(qreal pixels)
 	{
-		auto ppi = QApplication::primaryScreen()->logicalDotsPerInch();
+		auto ppi = QGuiApplication::primaryScreen()->logicalDotsPerInch();
 		return pixels * qreal(25.4) / ppi;
 	}
 	
@@ -83,32 +88,40 @@ namespace Util
 
 
 
+Settings& Settings::getInstance()
+{
+	static Settings instance;
+	return instance;
+}
+
+
+
 Settings::Settings()
  : QObject()
 {
-	const float touch_button_minimum_size_default = 11;
+	const float touch_button_minimum_size_default = 6.5;
 	float symbol_widget_icon_size_mm_default;
 	float map_editor_click_tolerance_default;
 	float map_editor_snap_distance_default;
 	int start_drag_distance_default;
 	
 	// Platform-specific settings defaults
-	#if defined(ANDROID)
-		symbol_widget_icon_size_mm_default = touch_button_minimum_size_default;
-		map_editor_click_tolerance_default = 4.0f;
-		map_editor_snap_distance_default = 15.0f;
-		start_drag_distance_default = Util::mmToPixelLogical(3.0f);
-	#else
-		symbol_widget_icon_size_mm_default = 8;
-		map_editor_click_tolerance_default = 3.0f;
-		map_editor_snap_distance_default = 10.0f;
-		start_drag_distance_default = QApplication::startDragDistance();
-	#endif
+#if defined(ANDROID) || !defined(QT_WIDGETS_LIB)
+	symbol_widget_icon_size_mm_default = touch_button_minimum_size_default;
+	map_editor_click_tolerance_default = 4.0f;
+	map_editor_snap_distance_default = 15.0f;
+	start_drag_distance_default = Util::mmToPixelLogical(3.0f);
+#else
+	symbol_widget_icon_size_mm_default = 8;
+	map_editor_click_tolerance_default = 3.0f;
+	map_editor_snap_distance_default = 10.0f;
+	start_drag_distance_default = QApplication::startDragDistance();
+#endif
 	
-	qreal ppi = QApplication::primaryScreen()->physicalDotsPerInch();
+	qreal ppi = QGuiApplication::primaryScreen()->physicalDotsPerInch();
 	// Beware of https://bugreports.qt-project.org/browse/QTBUG-35701
 	if (ppi > 2048.0 || ppi < 16.0)
-		ppi = QApplication::primaryScreen()->logicalDotsPerInch();
+		ppi = QGuiApplication::primaryScreen()->logicalDotsPerInch();
 	
 	registerSetting(MapDisplay_TextAntialiasing, "MapDisplay/text_antialiasing", false);
 	registerSetting(MapEditor_ClickToleranceMM, "MapEditor/click_tolerance_mm", map_editor_click_tolerance_default);
@@ -128,6 +141,7 @@ Settings::Settings()
 	
 	registerSetting(ActionGridBar_ButtonSizeMM, "ActionGridBar/button_size_mm", touch_button_minimum_size_default);
 	registerSetting(SymbolWidget_IconSizeMM, "SymbolWidget/icon_size_mm", symbol_widget_icon_size_mm_default);
+	registerSetting(SymbolWidget_ShowCustomIcons, "SymbolWidget/show_custom_icons", true);
 	
 	registerSetting(General_RetainCompatiblity, "retainCompatiblity", false);
 	registerSetting(General_SaveUndoRedo, "saveUndoRedo", true);
@@ -138,7 +152,6 @@ Settings::Settings()
 	registerSetting(General_RecentFilesList, "recentFileList", QVariant(QStringList()));
 	registerSetting(General_OpenMRUFile, "openMRUFile", false);
 	registerSetting(General_Local8BitEncoding, "local_8bit_encoding", QLatin1String("Default"));
-	registerSetting(General_NewOcd8Implementation, "new_ocd8_implementation", true);
 	registerSetting(General_StartDragDistance, "startDragDistance", start_drag_distance_default);
 	
 	registerSetting(HomeScreen_TipsVisible, "HomeScreen/tipsVisible", true);
@@ -147,12 +160,21 @@ Settings::Settings()
 	// Set antialiasing default depending on screen pixels per inch
 	registerSetting(MapDisplay_Antialiasing, "MapDisplay/antialiasing", Util::isAntialiasingRequired(getSetting(General_PixelsPerInch).toReal()));
 	
+	QSettings settings;
+	
+#ifndef Q_OS_ANDROID
+	// Overwrite default value with actual setting
+	touch_mode_enabled = mobileModeEnforced() || settings.value(QLatin1String("General/touch_mode_enabled"), touch_mode_enabled).toBool();
+#endif
+	
+	sensors.position_source = settings.value(QLatin1String("Sensors/position_source"), sensors.position_source).toString();
+	sensors.nmea_serialport = settings.value(QLatin1String("Sensors/nmea_serialport"), sensors.nmea_serialport).toString();
+	
 	// Migrate old settings
 	static bool migration_checked = false;
 	if (!migration_checked)
 	{
 		QVariant current_version { QLatin1String("0.8") };
-		QSettings settings;
 		auto settings_version = settings.value(QLatin1String("version")).toString();
 		if (!settings_version.isEmpty() && settings_version != current_version)
 		{
@@ -181,7 +203,7 @@ void Settings::migrateSettings(QSettings& settings, const QVariant& version)
 	
 	if (!settings.value(QLatin1String("MapEditor/units_rectified"), false).toBool())
 	{
-		const auto factor = QApplication::primaryScreen()->logicalDotsPerInch()
+		const auto factor = QGuiApplication::primaryScreen()->logicalDotsPerInch()
 		                    / getSetting(Settings::General_PixelsPerInch).toReal();
 		for (auto setting : { SymbolWidget_IconSizeMM, MapEditor_ClickToleranceMM, MapEditor_SnapDistanceMM, RectangleTool_HelperCrossRadiusMM })
 		{
@@ -326,4 +348,49 @@ int Settings::getStartDragDistancePx()
 }
 
 
+#ifndef Q_OS_ANDROID
+
+void Settings::setTouchModeEnabled(bool enabled)
+{
+	if (!mobileModeEnforced() && touch_mode_enabled != enabled)
+	{
+		touch_mode_enabled = enabled;
+		QSettings().setValue(QLatin1String("General/touch_mode_enabled"), enabled);
+		emit settingsChanged();
+	}
+}
+
+// static
+bool Settings::mobileModeEnforced() noexcept
+{
+	static bool const mobile_mode_enforced = qEnvironmentVariableIsSet("MAPPER_MOBILE_GUI")
+	                                         ? (qgetenv("MAPPER_MOBILE_GUI") != "0")
+	                                         : false;
+	return mobile_mode_enforced;
+}
+
+#endif
+
+
+void Settings::setPositionSource(const QString& name)
+{
+	if (name != sensors.position_source)
+	{
+		sensors.position_source = name;
+		QSettings().setValue(QLatin1String("Sensors/position_source"), name);
+		emit settingsChanged();
+	}
+}
+
+void Settings::setNmeaSerialPort(const QString& name)
+{
+	if (name != sensors.nmea_serialport)
+	{
+		sensors.nmea_serialport = name;
+		QSettings().setValue(QLatin1String("Sensors/nmea_serialport"), name);
+		emit settingsChanged();
+	}
+}
+
+
 }  // namespace OpenOrienteering
