@@ -22,6 +22,7 @@
 #include "template_tool_paint.h"
 
 #include <cstddef>
+#include <memory>
 
 #include <Qt>
 #include <QtMath>
@@ -30,7 +31,6 @@
 #include <QCursor>
 #include <QDir>
 #include <QFileInfo>
-#include <QFlags>
 #include <QFontMetrics>
 #include <QHBoxLayout>
 #include <QIcon>
@@ -140,7 +140,7 @@ int PaintOnTemplateTool::erase_width = 4;
 
 
 PaintOnTemplateTool::PaintOnTemplateTool(MapEditorController* editor, QAction* tool_action)
-: MapEditorTool(editor, Other, tool_action)
+: MapEditorTool(editor, Scribble, tool_action)
 {
 	connect(map(), &Map::templateDeleted, this, &PaintOnTemplateTool::templateDeleted);
 }
@@ -205,7 +205,7 @@ void PaintOnTemplateTool::redoSelected()
 }
 
 
-bool PaintOnTemplateTool::mousePressEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget* /*widget*/)
+bool PaintOnTemplateTool::mousePressEvent(QMouseEvent* event, const MapCoordF& map_coord, MapWidget* /*widget*/)
 {
 	if (event->button() == Qt::LeftButton || event->button() == Qt::RightButton)
 	{
@@ -219,7 +219,7 @@ bool PaintOnTemplateTool::mousePressEvent(QMouseEvent* event, MapCoordF map_coor
 	return false;
 }
 
-bool PaintOnTemplateTool::mouseMoveEvent(QMouseEvent* /*event*/, MapCoordF map_coord, MapWidget* widget)
+bool PaintOnTemplateTool::mouseMoveEvent(QMouseEvent* /*event*/, const MapCoordF& map_coord, MapWidget* widget)
 {
 	if (dragging && temp)
 	{
@@ -237,7 +237,7 @@ bool PaintOnTemplateTool::mouseMoveEvent(QMouseEvent* /*event*/, MapCoordF map_c
 	return false;
 }
 
-bool PaintOnTemplateTool::mouseReleaseEvent(QMouseEvent* /*event*/, MapCoordF map_coord, MapWidget* /*widget*/)
+bool PaintOnTemplateTool::mouseReleaseEvent(QMouseEvent* /*event*/, const MapCoordF& map_coord, MapWidget* /*widget*/)
 {
 	if (dragging && temp)
 	{
@@ -577,6 +577,7 @@ Template* PaintOnTemplateSelectDialog::addNewTemplate() const
 	temp->setTemplatePosition(MapCoord{top_left + MapCoordF{size_mm/2, size_mm/2}});
 	temp->setTemplateScaleX(1.0/pixel_per_mm);
 	temp->setTemplateScaleY(1.0/pixel_per_mm);
+	temp->setTemplateShear(0);
 	temp->setTemplateRotation(0);
 	temp->loadTemplateFile(false);
 	
