@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2013-2015 Kai Pastor
+ *    Copyright 2013-2018 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -33,7 +33,6 @@
 #include <QString>
 
 #include "core/map.h"
-#include "core/map_coord.h"
 #include "core/objects/boolean_tool.h"
 #include "core/objects/object.h"
 #include "core/symbols/symbol.h"
@@ -68,12 +67,20 @@ const QCursor& CutHoleTool::getCursor() const
 	return cursor;
 }
 
+void CutHoleTool::finishEditing()
+{
+	Q_ASSERT(editingInProgress());
+	if (path_tool)
+		path_tool->finishEditing();
+	MapEditorTool::finishEditing();
+}
+
 CutHoleTool::~CutHoleTool()
 {
 	delete path_tool;
 }
 
-bool CutHoleTool::mousePressEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget* widget)
+bool CutHoleTool::mousePressEvent(QMouseEvent* event, const MapCoordF& map_coord, MapWidget* widget)
 {
 	if (path_tool)
 		return path_tool->mousePressEvent(event, map_coord, widget);
@@ -104,11 +111,13 @@ bool CutHoleTool::mousePressEvent(QMouseEvent* event, MapCoordF map_coord, MapWi
 	
 	path_tool->init();
 	path_tool->mousePressEvent(event, map_coord, widget);
+	Q_ASSERT(path_tool->editingInProgress());
+	setEditingInProgress(true);
 	
 	return true;
 }
 
-bool CutHoleTool::mouseMoveEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget* widget)
+bool CutHoleTool::mouseMoveEvent(QMouseEvent* event, const MapCoordF& map_coord, MapWidget* widget)
 {
 	if (path_tool)
 		return path_tool->mouseMoveEvent(event, map_coord, widget);
@@ -116,7 +125,7 @@ bool CutHoleTool::mouseMoveEvent(QMouseEvent* event, MapCoordF map_coord, MapWid
 	return false;
 }
 
-bool CutHoleTool::mouseReleaseEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget* widget)
+bool CutHoleTool::mouseReleaseEvent(QMouseEvent* event, const MapCoordF& map_coord, MapWidget* widget)
 {
 	if (path_tool)
 		return path_tool->mouseReleaseEvent(event, map_coord, widget);
@@ -124,7 +133,7 @@ bool CutHoleTool::mouseReleaseEvent(QMouseEvent* event, MapCoordF map_coord, Map
 	return false;
 }
 
-bool CutHoleTool::mouseDoubleClickEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget* widget)
+bool CutHoleTool::mouseDoubleClickEvent(QMouseEvent* event, const MapCoordF& map_coord, MapWidget* widget)
 {
 	if (path_tool)
 		return path_tool->mouseDoubleClickEvent(event, map_coord, widget);
@@ -196,6 +205,7 @@ void CutHoleTool::pathAborted()
 {
 	path_tool->deleteLater();
 	path_tool = nullptr;
+	setEditingInProgress(false);
 	updateDirtyRect();
 	updateStatusText();
 }
