@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2017 Kai Pastor
+ *    Copyright 2012-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -114,7 +114,7 @@ void CutTool::updateStatusText()
 
 void CutTool::objectSelectionChangedImpl()
 {
-	for (const auto object : map()->selectedObjects())
+	for (const auto* object : map()->selectedObjects())
 	{
 		if (object->getSymbol()->getContainedTypes() & (Symbol::Line | Symbol::Area))
 		{
@@ -128,7 +128,7 @@ void CutTool::objectSelectionChangedImpl()
 
 
 
-bool CutTool::mousePressEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget* widget)
+bool CutTool::mousePressEvent(QMouseEvent* event, const MapCoordF& map_coord, MapWidget* widget)
 {
 	if (path_tool)
 		return path_tool->mousePressEvent(event, map_coord, widget);
@@ -144,7 +144,7 @@ bool CutTool::mousePressEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget
 }
 
 
-bool CutTool::mouseMoveEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget* widget)
+bool CutTool::mouseMoveEvent(QMouseEvent* event, const MapCoordF& map_coord, MapWidget* widget)
 {
 	if (path_tool)
 		return path_tool->mouseMoveEvent(event, map_coord, widget);
@@ -160,7 +160,7 @@ bool CutTool::mouseMoveEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget*
 }
 
 
-bool CutTool::mouseReleaseEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget* widget)
+bool CutTool::mouseReleaseEvent(QMouseEvent* event, const MapCoordF& map_coord, MapWidget* widget)
 {
 	if (path_tool)
 		return path_tool->mouseReleaseEvent(event, map_coord, widget);
@@ -175,7 +175,7 @@ bool CutTool::mouseReleaseEvent(QMouseEvent* event, MapCoordF map_coord, MapWidg
 }
 
 
-bool CutTool::mouseDoubleClickEvent(QMouseEvent* event, MapCoordF map_coord, MapWidget* widget)
+bool CutTool::mouseDoubleClickEvent(QMouseEvent* event, const MapCoordF& map_coord, MapWidget* widget)
 {
 	if (path_tool)
 		return path_tool->mouseDoubleClickEvent(event, map_coord, widget);
@@ -226,7 +226,10 @@ void CutTool::clickRelease()
 		// Line split in single point
 		auto split_objects = line_point.object->splitLineAt(line_point);
 		if (!split_objects.empty())
+		{
 			replaceObject(line_point.object, split_objects);
+			updateHoverState(click_pos_map);
+		}
 	}
 }
 
@@ -293,7 +296,7 @@ void CutTool::startCuttingLine(const ObjectPathCoord& point)
 }
 
 
-void CutTool::updateCuttingLine(MapCoordF cursor_pos)
+void CutTool::updateCuttingLine(const MapCoordF& cursor_pos)
 {
 	Q_ASSERT(editingInProgress());
 	
@@ -351,10 +354,12 @@ void CutTool::finishCuttingLine()
 	if (reverse_drag)
 		qSwap(drag_start_len, drag_end_len);
 	
-	auto split_objects = edit_object->removeFromLine(drag_part_index, qreal(drag_start_len), qreal(drag_end_len));
+	auto split_objects = edit_object->removeFromLine(drag_part_index, drag_start_len, drag_end_len);
 	replaceObject(edit_object, split_objects);
 	
-	finishEditing();
+	if (editingInProgress())
+		finishEditing();
+	updateHoverState(cur_pos_map);
 	updatePreviewObjects();
 }
 
@@ -444,7 +449,7 @@ void CutTool::finishCuttingArea(PathObject* split_path)
 		holes->deletePart(0);
 	}
 	
-	bool ok; Q_UNUSED(ok) // "ok" is only used in Q_ASSERT.
+	bool ok = true;
 	std::vector<PathObject*> out_paths = { new PathObject { edit_object->parts().front() }, nullptr };
 	const PathPart& drag_part = edit_object->parts()[drag_part_index];
 	if (drag_part.isClosed())
@@ -452,11 +457,11 @@ void CutTool::finishCuttingArea(PathObject* split_path)
 		out_paths[1] = out_paths[0]->duplicate();
 		
 		out_paths[0]->changePathBounds(drag_part_index, drag_start_len, end_path_coord.clen);
-		ok = out_paths[0]->connectIfClose(split_path, split_threshold);
+		ok &= out_paths[0]->connectIfClose(split_path, split_threshold);
 		Q_ASSERT(ok);
 
 		out_paths[1]->changePathBounds(drag_part_index, end_path_coord.clen, drag_start_len);
-		ok = out_paths[1]->connectIfClose(split_path, split_threshold);
+		ok &= out_paths[1]->connectIfClose(split_path, split_threshold);
 		Q_ASSERT(ok);
 	}
 	else
@@ -466,7 +471,7 @@ void CutTool::finishCuttingArea(PathObject* split_path)
 		auto path_len = drag_part.path_coords.back().clen;
 		if (min_cut_pos <= 0 && max_cut_pos >= path_len)
 		{
-			ok = out_paths[0]->connectIfClose(split_path, split_threshold);
+			ok &= out_paths[0]->connectIfClose(split_path, split_threshold);
 			Q_ASSERT(ok);
 			
 			out_paths[1] = split_path->duplicate();
@@ -478,11 +483,11 @@ void CutTool::finishCuttingArea(PathObject* split_path)
 			out_paths[1] = out_paths[0]->duplicate();
 			
 			out_paths[0]->changePathBounds(drag_part_index, 0, cut_pos);
-			ok = out_paths[0]->connectIfClose(split_path, split_threshold);
+			ok &= out_paths[0]->connectIfClose(split_path, split_threshold);
 			Q_ASSERT(ok);
 			
 			out_paths[1]->changePathBounds(drag_part_index, cut_pos, path_len);
-			ok = out_paths[1]->connectIfClose(split_path, split_threshold);
+			ok &= out_paths[1]->connectIfClose(split_path, split_threshold);
 			Q_ASSERT(ok);
 		}
 		else
@@ -491,20 +496,21 @@ void CutTool::finishCuttingArea(PathObject* split_path)
 			PathObject* temp_path = out_paths[0]->duplicate();
 			
 			out_paths[0]->changePathBounds(drag_part_index, min_cut_pos, max_cut_pos);
-			ok = out_paths[0]->connectIfClose(split_path, split_threshold);
+			ok &= out_paths[0]->connectIfClose(split_path, split_threshold);
 			Q_ASSERT(ok);
 			
 			out_paths[1]->changePathBounds(drag_part_index, 0, min_cut_pos);
-			ok = out_paths[1]->connectIfClose(split_path, split_threshold);
+			ok &= out_paths[1]->connectIfClose(split_path, split_threshold);
 			Q_ASSERT(ok);
 			
 			temp_path->changePathBounds(drag_part_index, max_cut_pos, path_len);
-			ok = out_paths[1]->connectIfClose(temp_path, split_threshold);
+			ok &= out_paths[1]->connectIfClose(temp_path, split_threshold);
 			Q_ASSERT(ok);
 			
 			delete temp_path;
 		}
 	}
+	Q_UNUSED(ok) // "ok" is only used in Q_ASSERT.
 	
 	for (auto& object : out_paths)
 	{
@@ -590,7 +596,7 @@ int CutTool::updateDirtyRectImpl(QRectF& rect)
 	}
 	else if (map->selectedObjects().size() <= max_objects_for_handle_display)
 	{
-		for (const auto object : map->selectedObjects())
+		for (const auto* object : map->selectedObjects())
 			object->includeControlPointsRect(rect);
 	}
 	
@@ -612,7 +618,7 @@ void CutTool::drawImpl(QPainter* painter, MapWidget* widget)
 	}
 	else if (map->selectedObjects().size() <= max_objects_for_handle_display)
 	{
-		for (const auto object: map->selectedObjects())
+		for (const auto* object: map->selectedObjects())
 		{
 			auto hover_point = MapCoordVector::size_type { hover_object == object ? this->hover_point : no_point };
 			pointHandles().draw(painter, widget, object, hover_point, false);
@@ -639,7 +645,7 @@ void CutTool::drawImpl(QPainter* painter, MapWidget* widget)
 
 
 
-void CutTool::updateHoverState(MapCoordF cursor_pos)
+void CutTool::updateHoverState(const MapCoordF& cursor_pos)
 {
 	auto new_hover_state = HoverState { HoverFlag::OverNothing };
 	PathObject* new_hover_object = nullptr;
@@ -658,7 +664,7 @@ void CutTool::updateHoverState(MapCoordF cursor_pos)
 	else if (map()->selectedObjects().size() <= max_objects_for_handle_display)
 	{
 		auto best_distance_sq = std::numeric_limits<double>::max();
-		for (const auto object : map()->selectedObjects())
+		for (auto* object : map()->selectedObjects())
 		{
 			if (object->getType() != Object::Path)
 				continue;
@@ -691,7 +697,7 @@ void CutTool::updateHoverState(MapCoordF cursor_pos)
 }
 
 
-ObjectPathCoord CutTool::findEditPoint(MapCoordF cursor_pos_map, int with_type, int without_type) const
+ObjectPathCoord CutTool::findEditPoint(const MapCoordF& cursor_pos_map, int with_type, int without_type) const
 {
 	ObjectPathCoord result;
 	if (hover_state == HoverFlag::OverObjectNode
@@ -705,7 +711,7 @@ ObjectPathCoord CutTool::findEditPoint(MapCoordF cursor_pos_map, int with_type,
 		// Check if a line segment was clicked
 		const auto click_tolerance_map = 0.001 * cur_map_widget->getMapView()->pixelToLength(clickTolerance());
 		auto min_distance_sq = float(qMin(999999.9, click_tolerance_map * click_tolerance_map));
-		for (const auto object : map()->selectedObjects())
+		for (auto* object : map()->selectedObjects())
 		{
 			if (object->getType() != Object::Path
 			    || !(object->getSymbol()->getContainedTypes() & with_type)
