@@ -1,5 +1,5 @@
 /*
- *    Copyright 2013-2017 Kai Pastor
+ *    Copyright 2013-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -20,20 +20,25 @@
 #ifndef OPENORIENTEERING_XML_STREAM_UTIL_H
 #define OPENORIENTEERING_XML_STREAM_UTIL_H
 
-#include <vector>
-
+#include <QtGlobal>
 #include <QHash>
+#include <QLatin1String>
 #include <QRectF>
+#include <QSizeF>
 #include <QString>
+#include <QStringRef>
+#include <QXmlStreamAttributes>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 
-#include "fileformats/file_format.h"
+#include "core/map_coord.h"
 
+class QRectF;
+class QSizeF;
+class QXmlStreamReader;
+class QXmlStreamWriter;
 
-// Originally defined in map_coord.h, but we want to avoid the depedency.
-class MapCoord;
-typedef std::vector<MapCoord> MapCoordVector;
+namespace OpenOrienteering {
 
 
 /**
@@ -42,6 +47,11 @@ typedef std::vector<MapCoord> MapCoordVector;
 void writeLineBreak(QXmlStreamWriter& xml);
 
 
+/**
+ * Returns the number of characters which are significant for input/output.
+ */
+QString numberToString(double value, int precision);
+
 
 /**
  * This class provides recovery from invalid characters in an XML stream.
@@ -54,7 +64,7 @@ void writeLineBreak(QXmlStreamWriter& xml);
  * was initialized.
  * 
  * In a single recovery attempt, the utility tries to handle all offending
- * characters from the element for wich the tool was constructed. For each
+ * characters from the element for which the tool was constructed. For each
  * offending character, the whole XML data is parsed again from the start.
  * That's why multiple corrections may take a long time to run.
  * 
@@ -129,11 +139,19 @@ public:
 	 */
 	XmlElementWriter(QXmlStreamWriter& xml, const QLatin1String& element_name);
 	
+	XmlElementWriter(const XmlElementWriter&) = delete;
+	XmlElementWriter(XmlElementWriter&&) = delete;
+	
 	/**
 	 * Writes the end tag of the element.
 	 */
 	~XmlElementWriter();
 	
+	
+	XmlElementWriter& operator=(const XmlElementWriter&) = delete;
+	XmlElementWriter& operator=(XmlElementWriter&&) = delete;
+	
+	
 	/**
 	 * Writes an attribute with the given name and value.
 	 */
@@ -272,6 +290,9 @@ public:
 	 */
 	XmlElementReader(QXmlStreamReader& xml);
 	
+	XmlElementReader(const XmlElementReader&) = delete;
+	XmlElementReader(XmlElementReader&&) = delete;
+	
 	/**
 	 * Destructor.
 	 * 
@@ -279,6 +300,11 @@ public:
 	 */
 	~XmlElementReader();
 	
+	
+	XmlElementReader& operator=(const XmlElementReader&) = delete;
+	XmlElementReader& operator=(XmlElementReader&&) = delete;
+	
+	
 	/**
 	 * Tests whether the element has an attribute with the given name.
 	 */
@@ -318,6 +344,19 @@ public:
 	 */
 	void read(MapCoordVector& coords);
 	
+	/**
+	 * Reads the coordinates vector for a text object.
+	 * 
+	 * This is either a single anchor, or an anchor and a size, packed as a
+	 * coordinates vector. Regular coordinate bounds checking is not applied
+	 * to the size.
+	 * 
+	 * \todo Make box size explicit data.
+	 * 
+	 * \see read(MapCoordVector&)
+	 */
+	void readForText(MapCoordVector& coords);
+	
 	/**
 	 * Read tags.
 	 */
@@ -434,19 +473,19 @@ void XmlElementWriter::writeAttribute(const QLatin1String& qualifiedName, const
 inline
 void XmlElementWriter::writeAttribute(const QLatin1String& qualifiedName, const double value, int precision)
 {
-	xml.writeAttribute(qualifiedName, QString::number(value, 'f', precision));
+	xml.writeAttribute(qualifiedName, numberToString(value, precision));
 }
 
 inline
 void XmlElementWriter::writeAttribute(const QLatin1String& qualifiedName, const float value)
 {
-	xml.writeAttribute(qualifiedName, QString::number(value));
+	writeAttribute(qualifiedName, double(value));
 }
 
 inline
 void XmlElementWriter::writeAttribute(const QLatin1String& qualifiedName, const float value, int precision)
 {
-	xml.writeAttribute(qualifiedName, QString::number(value, 'f', precision));
+	writeAttribute(qualifiedName, double(value), precision);
 }
 
 inline
@@ -617,7 +656,7 @@ long unsigned int XmlElementReader::attribute(const QLatin1String& qualifiedName
 	unsigned int value = 0;
 	const QStringRef ref = attributes.value(qualifiedName);
 	if (ref.size())
-		value = QString::fromRawData(ref.data(), ref.size()).toULong();
+		value = QString::fromRawData(ref.data(), ref.size()).toUInt();
 	return value;
 }
 
@@ -729,4 +768,7 @@ void XmlElementReader::read(QHash<QString, QString> &tags)
 	}
 }
 
+
+}  // namespace OpenOrienteering
+
 #endif
