@@ -17,17 +17,18 @@
  *    along with OpenOrienteering.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _OPENORIENTEERING_XML_BENCHMARK_T_H
-#define _OPENORIENTEERING_XML_BENCHMARK_T_H
-
-#include <QtTest/QtTest>
+#ifndef OPENORIENTEERING_XML_BENCHMARK_T_H
+#define OPENORIENTEERING_XML_BENCHMARK_T_H
 
 #include <QBuffer>
-#include <QXmlStreamReader>
-#include <QXmlStreamWriter>
+#include <QObject>
 
 #include "core/map_coord.h"
 
+class QXmlStreamWriter;
+
+namespace OpenOrienteering {
+
 
 /**
  * @test Benchmarks different implementations of saving and loading
@@ -107,4 +108,7 @@ private:
 	QBuffer buffer;
 };
 
+
+}  // namespace OpenOrienteering
+
 #endif
