@@ -1,5 +1,5 @@
 /*
- *    Copyright 2012-2015 Kai Pastor
+ *    Copyright 2012-2015, 2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -19,17 +19,32 @@
 
 #include "georeferencing_t.h"
 
-#include <QtTest>
+#include <cmath>
+#include <cstddef>
 
-#include <proj_api.h>
+#include <QtMath>
+#include <QtTest>
+#include <QLatin1String>
+#include <QLineF>
+#include <QPoint>
+#include <QPointF>
+
+#ifndef ACCEPT_USE_OF_DEPRECATED_PROJ_API_H
+#  include <proj.h>
+#endif
 
 #include "core/crs_template.h"
+#include "core/latlon.h"
+#include "core/map_coord.h"
 #include "fileformats/xml_file_format.h"
 
 using namespace OpenOrienteering;
 
 
+#ifndef MAPPER_COMMON_LIB
+// mock stuff to satisfy link-time dependencies
 int XMLFileFormat::active_version = 6;
+#endif
 
 
 namespace
@@ -53,7 +68,7 @@ namespace
 
 void GeoreferencingTest::initTestCase()
 {
-	// nothing
+	XMLFileFormat::active_version = 6;
 }
 
 
@@ -93,13 +108,13 @@ void GeoreferencingTest::testGridScaleFactor()
 	
 	// Standard Mercator scale factor is as simple as:
 	//
-	//   auto scale_factor = 1.0 / cos(Georeferencing::degToRad(50.0));
+	//   scale_factor = 1.0 / cos(degToRad(50.0));
 	//
 	// EPSG:3857 aka Web Mercator scale factor is more complicated, and
 	// different E/W vs. N/S. For our test, we need the east/west scale factor:
 	auto e = 0.081819191; // WGS84 eccentricity
-	auto phi = Georeferencing::degToRad(50.0); // Latitude as used for UTM above
-	auto scale_factor = pow(1.0 - e * e * sin(phi) * sin(phi), 0.5) / cos(phi);
+	auto phi = qDegreesToRadians(50.0); // Latitude as used for UTM above
+	auto scale_factor = std::pow(1.0 - e * e * std::sin(phi) * std::sin(phi), 0.5) / std::cos(phi);
 	QVERIFY(scale_factor != 1.0);
 	mercator_georef.setGridScaleFactor(scale_factor);
 	QCOMPARE(mercator_georef.getGridScaleFactor(), scale_factor);
@@ -107,7 +122,7 @@ void GeoreferencingTest::testGridScaleFactor()
 	// With the scale factor applied, we should get the same ground distance.
 	auto ground_distance_utm  = QLineF(point_a_utm, point_b_utm).length();
 	auto ground_distance_mercator  = QLineF(point_a_mercator, point_b_mercator).length();
-	auto equal_ground_distance = (qAbs(ground_distance_mercator / scale_factor - ground_distance_utm) < 1.0); // meter
+	auto equal_ground_distance = (std::fabs(ground_distance_mercator / scale_factor - ground_distance_utm) < 1.0); // meter
 	if (!equal_ground_distance)
 	{
 		// Fail with clear output
@@ -117,7 +132,7 @@ void GeoreferencingTest::testGridScaleFactor()
 	// With the scale factor applied, we should get the same paper distance.
 	auto map_distance_utm = QLineF(utm_georef.toMapCoordF(point_a_utm), utm_georef.toMapCoordF(point_b_utm)).length();
 	auto map_distance_mercator  = QLineF(mercator_georef.toMapCoordF(point_a_mercator), mercator_georef.toMapCoordF(point_b_mercator)).length();
-	equal_ground_distance = (qAbs(map_distance_mercator - map_distance_utm) < 1000.0); // millimeters, scale is 1:1
+	equal_ground_distance = (std::fabs(map_distance_mercator - map_distance_utm) < 1000.0); // millimeters, scale is 1:1
 	if (!equal_ground_distance)
 	{
 		// Fail with clear output
@@ -177,18 +192,20 @@ void GeoreferencingTest::testProjection_data()
 	// Selected from http://geoportal.cuzk.cz/geoprohlizec/?lng=EN, source Bodová pole, layer Bod ZPBP určený v ETRS
 	// http://dataz.cuzk.cz/gu.php?1=25&2=08&3=024&4=a&stamp=7oexTQLNPE8ri5SXY04ARS7vIMnJu3N2
 	QTest::newRow("EPSG 5514 ČÚZK Dolní Temenice") << epsg5514_spec   << -563714.79 << -1076943.54 << degFromDMS(49, 58, 37.5577) << degFromDMS(16, 57, 35.5493);
+	
+	// Swiss CH1903+/LV95
+	auto epsg2056_spec = QStringLiteral("+init=epsg:2056");
+	// Projection centre, according to https://www.epsg-registry.org/
+	QTest::newRow("EPSG 2056 Bern") << epsg2056_spec << 2600000.0 << 1200000.0 << degFromDMS(46, 57, 03.898) << degFromDMS(7, 26, 19.077);
+	// Issue GH-1325
+	QTest::newRow("EPSG 2056 GH-1325") << epsg2056_spec << 2643092.73 << 1150008.01 << 46.5 << 8.0;
 }
 
 
 void GeoreferencingTest::testProjection()
 {
-#if PJ_VERSION >= 480
 	const double max_dist_error = 2.2; // meter
-	const double max_angl_error = 0.00004; // degrees
-#else
-	const double max_dist_error = 5.5; // meter
-	const double max_angl_error = 0.00007; // degrees
-#endif
+	const double max_angl_error = 0.00005; // degrees
 	
 	QFETCH(QString, proj);
 	QVERIFY2(georef.setProjectedCRS(proj, proj), proj.toLatin1());
@@ -205,21 +222,53 @@ void GeoreferencingTest::testProjection()
 	QPointF proj_coord = georef.toProjectedCoords(lat_lon, &ok);
 	QVERIFY(ok);
 	
-	if (fabs(proj_coord.x() - easting) > max_dist_error)
+	if (std::fabs(proj_coord.x() - easting) > max_dist_error)
 		QCOMPARE(QString::number(proj_coord.x(), 'f'), QString::number(easting, 'f'));
-	if (fabs(proj_coord.y() - northing) > max_dist_error)
+	if (std::fabs(proj_coord.y() - northing) > max_dist_error)
 		QCOMPARE(QString::number(proj_coord.y(), 'f'), QString::number(northing, 'f'));
 	
 	// projected to geographic
 	proj_coord = QPointF(easting, northing);
 	lat_lon = georef.toGeographicCoords(proj_coord, &ok);
 	QVERIFY(ok);
-	if (fabs(lat_lon.latitude() - latitude) > max_angl_error)
+	if (std::fabs(lat_lon.latitude() - latitude) > max_angl_error)
 		QCOMPARE(QString::number(lat_lon.latitude(), 'f'), QString::number(latitude, 'f'));
-	if (fabs(lat_lon.longitude() - longitude) > (max_angl_error / cos(latitude)))
+	if (std::fabs(lat_lon.longitude() - longitude) > (max_angl_error * std::cos(qDegreesToRadians(latitude))))
 		QCOMPARE(QString::number(lat_lon.longitude(), 'f'), QString::number(longitude, 'f'));
 }
 
 
 
+#ifndef ACCEPT_USE_OF_DEPRECATED_PROJ_API_H
+
+namespace {
+	static bool finder_called;
+
+	extern "C"
+	const char* projFinderTestFakeCRS(PJ_CONTEXT* /*ctx*/, const char* name, void* /*user_data*/)
+	{
+		finder_called = true;
+		[name]() {
+			QCOMPARE(name, "fake_crs");
+		}();
+		return nullptr;
+	}
+}
+
+void GeoreferencingTest::testProjContextSetFileFinder()
+{
+	finder_called = false;
+	
+	proj_context_set_file_finder(nullptr, &projFinderTestFakeCRS, nullptr);
+	QVERIFY(!finder_called);
+	
+	Georeferencing fake_georef;
+	fake_georef.setProjectedCRS(QStringLiteral("Fake CRS"), QString::fromLatin1("+init=fake_crs:123"));
+	QVERIFY(finder_called);
+}
+
+#endif
+
+
+
 QTEST_GUILESS_MAIN(GeoreferencingTest)
