@@ -1,5 +1,5 @@
 /*
- *    Copyright 2014-2017 Kai Pastor
+ *    Copyright 2014-2019 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -43,6 +43,7 @@
 #include <QRectF>
 #include <QStringList>
 #include <QStringRef>
+#include <QTextStream>
 #include <QXmlStreamAttributes>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
@@ -56,12 +57,13 @@
 #include "core/map_coord.h"
 #include "core/map_printer.h"
 #include "core/map_view.h"
+#include "core/objects/symbol_rule_set.h"
 #include "core/symbols/area_symbol.h"
 #include "core/symbols/symbol.h"
 #include "fileformats/xml_file_format_p.h"
 #include "templates/template.h"
 #include "undo/undo_manager.h"
-#include "util/backports.h"
+#include "util/backports.h"  // IWYU pragma: keep
 
 
 using namespace OpenOrienteering;
@@ -69,10 +71,15 @@ using namespace OpenOrienteering;
 namespace
 {
 
+const auto legacy_symbol_sets =                              // clazy:exclude=non-pod-global-static
+  QString::fromLatin1("ISOM2000;ISOM2017;ISSkiOM")
+  .split(QLatin1Char(';'));
+
+const auto Vanished = QString::fromLatin1("vanished");       // clazy:exclude=non-pod-global-static
 const auto Obsolete = QString::fromLatin1("obsolete");       // clazy:exclude=non-pod-global-static
 const auto NeedsReview = QString::fromLatin1("unfinished");  // clazy:exclude=non-pod-global-static
 
-const auto translation_suffix = { "template", "cs", "de", "eo", "fi", "fr", "nl", "ru", "sv", "uk" };
+const auto map_symbol_translations = QByteArray(MAP_SYMBOL_TRANSLATIONS);  // clazy:exclude=non-pod-global-static
 
 using TranslationEntries = std::vector<SymbolSetTool::TranslationEntry>;
 
@@ -84,10 +91,42 @@ void addSource(TranslationEntries& entries, const QString& context, const QStrin
 		return entry.context == context && entry.comment == comment;
 	});
 	QVERIFY(found == end(entries));
-	entries.push_back({context, source, comment, {} });
-	entries.back().translations.reserve(std::extent<decltype(translation_suffix)>::value);
+	entries.push_back({context, source, comment});
 }
 
+void processTranslation(const Map& map, TranslationEntries& translation_entries)
+{
+	auto const id = map.symbolSetId();
+	
+	auto num_colors = map.getNumColors();
+	for (int i = 0; i < num_colors; ++i)
+	{
+		auto* color = map.getColor(i);
+		auto const& source = color->getName();
+		auto comment = QString{QLatin1String("Color ") + QString::number(color->getPriority())};
+		addSource(translation_entries, id, source, comment);
+	}
+	
+	auto num_symbols = map.getNumSymbols();
+	for (int i = 0; i < num_symbols; ++i)
+	{
+		auto symbol = map.getSymbol(i);
+		auto source = symbol->getName();
+		auto comment = QString{QLatin1String("Name of symbol ") + symbol->getNumberAsString()};
+		addSource(translation_entries, id, source, comment);
+		
+		source = symbol->getDescription();
+		comment = QString{QLatin1String("Description of symbol ") + symbol->getNumberAsString()};
+		if (source.isEmpty())
+			qInfo("%s: empty", qPrintable(comment));
+		else
+			addSource(translation_entries, id, source, comment);
+	}
+}
+
+
+/// \todo Review unused function
+#ifdef SYMBOL_SET_T_UNUSED
 
 void addTranslation(TranslationEntries& entries, const QString& context, const QString& comment, const QString& language, const QString& translation)
 {
@@ -122,8 +161,10 @@ QString findSuggestion(TranslationEntries& translation_entries, const QString& s
 	return {};
 }
 
+#endif  // SYMBOL_SET_T_UNUSED
 
-TranslationEntries readTsFile(QIODevice& device, const QString& language)
+
+TranslationEntries readTsFile(QIODevice& device)
 {
 	auto result = TranslationEntries{};
 	
@@ -162,18 +203,18 @@ TranslationEntries readTsFile(QIODevice& device, const QString& language)
 							}
 							else if (xml.name() == QLatin1String("translation"))
 							{
-								auto type = xml.attributes().value(QLatin1String("type")).toString();
-								auto translation = xml.readElementText();
-								if (!translation.isEmpty())
-									entry.translations.resize(1, { language, translation, type });
+								entry.type = xml.attributes().value(QLatin1String("type")).toString();
+								entry.translation = xml.readElementText();
 							}
 							else
 							{
 								xml.skipCurrentElement();
 							}
 						}
-						if (!entry.translations.empty())
+						if (!entry.translation.isEmpty())
+						{
 							result.push_back(std::move(entry));
+						}
 					}
 					else
 					{
@@ -192,10 +233,383 @@ TranslationEntries readTsFile(QIODevice& device, const QString& language)
 	return result;
 }
 
+void writeObsoleteEntries(QXmlStreamWriter& xml, TranslationEntries& entries, const QString& context)
+{
+	for (auto& entry: entries)
+	{
+		if (entry.type.isEmpty() && entry.translation.isEmpty() && legacy_symbol_sets.contains(context))
+			continue;
+		
+		if (entry.obsolete && entry.context == context)
+		{
+			entry.type = Obsolete;
+			entry.write(xml);
+		}
+	}
+}
+
+
+bool isMarked(const Symbol& symbol) noexcept
+{
+	return symbol.getName().startsWith(QStringLiteral("[MARK]"));
+}
+
+void markSymbol(Symbol& symbol)
+{
+	if (!isMarked(symbol))
+		symbol.setName(QStringLiteral("[MARK]") + symbol.getName());
+}
+
+void markSymbolsByColor(Map& map, const MapColor* color)
+{
+	auto num_symbols = map.getNumSymbols();
+	for (auto i = 0; i < num_symbols; ++i)
+	{
+		auto* symbol = map.getSymbol(i);
+		if (symbol->containsColor(color))
+			markSymbol(*symbol);
+	}
+}
+
+void deleteMarkedSymbols(Map& map)
+{
+	auto num_symbols = map.getNumSymbols();
+	for (auto i = num_symbols; i > 0; --i)
+	{
+		auto const index = i - 1;
+		auto* symbol = map.getSymbol(index);
+		if (isMarked(*symbol))
+			map.deleteSymbol(index);
+	}
+}
+
+
+namespace ISOM_2017_2
+{
+
+void scale(Map& map, unsigned int source_scale, unsigned int target_scale)
+{
+	map.setScaleDenominator(target_scale);
+	
+	auto const factor = double(source_scale) / double(target_scale);
+	map.scaleAllObjects(factor, MapCoord{});
+	
+	int symbols_changed = 0;
+	for (int i = 0; i < map.getNumSymbols(); ++i)
+	{
+		auto* symbol = map.getSymbol(i);
+		auto const code = symbol->getNumberComponent(0);
+		switch (code)
+		{
+		case 602:  // Registration mark
+		case 999:  // OpenOrienteering logo
+			break;
+		default:
+			symbol->scale(factor);
+			++symbols_changed;
+		}
+	}
+	QCOMPARE(symbols_changed, 189);
+}
+
+}  // namespace ISOM_2017_2
+
+
+namespace ISSOM
+{
+
+void scale(Map& map, unsigned int /*source_scale*/, unsigned int target_scale)
+{
+	map.setScaleDenominator(target_scale);
+	
+	int north_lines_changed = 0;
+	for (int i = 0; i < map.getNumSymbols(); ++i)
+	{
+		auto* symbol = map.getSymbol(i);
+		auto const code = symbol->getNumberComponent(0);
+		if (code == 601 && symbol->getType() == Symbol::Area)
+		{
+			AreaSymbol::FillPattern& pattern0 = symbol->asArea()->getFillPattern(0);
+			if (pattern0.type == AreaSymbol::FillPattern::LinePattern)
+			{
+				switch (target_scale)
+				{
+				case 4000u:
+					pattern0.line_spacing = 37500;
+					break;
+				default:
+					QFAIL("Undefined north line spacing for this scale");
+				}
+				++north_lines_changed;
+			}
+		}
+	}
+	QCOMPARE(north_lines_changed, 2);
+}
+
+}  // namespace ISSOM
+
+
+namespace ISMTBOM
+{
+
+void scale(Map& map, unsigned int /*source_scale*/, unsigned int target_scale)
+{
+	map.setScaleDenominator(target_scale);
+	
+	auto const factor = (target_scale >= 15000u) ? 1.0 : 1.5;
+	map.scaleAllObjects(factor, MapCoord());
+	
+	int symbols_changed = 0;
+	for (int i = 0; i < map.getNumSymbols(); ++i)
+	{
+		auto* symbol = map.getSymbol(i);
+		auto const code = symbol->getNumberComponent(0);
+		switch (code)
+		{
+		case 602:  // Registration mark
+		case 999:  // OpenOrienteering logo
+			break;
+		default:
+			symbol->scale(factor);
+			++symbols_changed;
+		}
+	}
+	QCOMPARE(symbols_changed, 168);
+}
+
+}  // namespace ISMTBOM
+
+
+namespace ISSkiOM_2019 {
+
+auto const included_ISOM_codes = {
+    101, 102, 103, 104, 105, 107, 109, 111,  // Land forms
+    201, 202, 204, 205, 206, 207, 208, 209,  // Rock and boulders
+    304, 305,  // Water and marsh
+    401, 402, 403, 404, 405, 406, 413, 414, 415, 416, 419,  // Open land and vegetation
+    501, 502, 503, 504, 508, 509, 510, 511, 512, 513, 515,
+    516, 518, 519, 520, 521, 524, 525, 529, 530, 531,  // Man-made features
+    601,  // Technical symbols
+};
+
+bool symbolOrder(const Symbol* s1, const Symbol* s2)
+{
+	if (s1->getNumberComponent(0) == 301
+	    && s2->getNumberComponent(0) == 301)
+	{
+		// ISSkiOM 301.1 goes after 301.x
+		if (s1->getNumberComponent(1) == 1)
+			return false;
+		if (s2->getNumberComponent(1) == 1)
+			return true;
+	}
+	return Symbol::lessByNumber(s1, s2);
+}
+
+void mergeISOM(Map& target, const QDir& symbol_set_dir)
+{
+	// Load to-be-merged symbol set
+	auto const target_scale = target.getScaleDenominator();
+	auto const source_filename = QString::fromLatin1("src/ISOM 2017-2_%1.xmap").arg(target_scale);
+	QVERIFY(symbol_set_dir.exists(source_filename));
+	
+	Map map;
+	MapView view{ &map };
+	map.loadFrom(symbol_set_dir.absoluteFilePath(source_filename), &view);
+	QCOMPARE(map.getScaleDenominator(), target_scale);
+	
+	// Delete some colors, and mark the symbols which use these colors
+	auto num_colors_deleted = 0;
+	auto const num_colors = map.getNumColors();
+	for (auto i = num_colors; i > 0; --i)
+	{
+		auto const index = i - 1;
+		auto* const color = map.getColor(index);
+		if (color->getSpotColorName() == QStringLiteral("GREEN 100, BLACK 50")
+		    || color->getSpotColorName() == QStringLiteral("GREEN 60") )
+		{
+			markSymbolsByColor(map, color);
+			map.deleteColor(index);
+			++num_colors_deleted;
+		}
+	}
+	QCOMPARE(num_colors_deleted, 3);
+	
+	// Mark all but explicitly included symbols
+	auto const first_code = begin(included_ISOM_codes);
+	auto const last_code = end(included_ISOM_codes);
+	auto num_symbols = map.getNumSymbols();
+	for (auto i = num_symbols; i > 0; --i)
+	{
+		auto const index = i - 1;
+		auto* const symbol = map.getSymbol(index);
+		auto const code = symbol->getNumberComponent(0);
+		if (std::find(first_code, last_code, code) == last_code)
+			markSymbol(*symbol);
+	}
+	
+	// Actual merge
+	auto first_imported_symbol = target.getNumSymbols();
+	target.importMap(map, Map::CompleteImport, nullptr);
+	
+	auto const crt_filename = QString::fromLatin1("%1-%2.crt").arg(map.symbolSetId(), target.symbolSetId());
+	QVERIFY(symbol_set_dir.exists(crt_filename));
+	
+	QFile crt_file {symbol_set_dir.absoluteFilePath(crt_filename)};
+	QVERIFY(crt_file.open(QIODevice::ReadOnly));
+	QTextStream stream {&crt_file};
+	
+	auto rules = SymbolRuleSet::loadCrt(stream, target);
+	QCOMPARE(stream.status(), QTextStream::Ok);
+	
+	// Postprocess CRT: Find symbols by code number
+	for (auto& item : rules)
+	{
+		QVERIFY(item.type != SymbolRule::NoAssignment);
+		QVERIFY(item.symbol);
+		QCOMPARE(item.query.getOperator(), ObjectQuery::OperatorSearch);
+		
+		auto operands = item.query.tagOperands();
+		QVERIFY(operands);
+		QVERIFY(!operands->value.isEmpty());
+		
+		// Find original symbol number matching the pattern
+		for (int i = first_imported_symbol; i < target.getNumSymbols(); ++i)
+		{
+			auto* symbol = target.getSymbol(i);
+			if (symbol->getNumberAsString() == operands->value
+			    && Symbol::areTypesCompatible(symbol->getType(), item.symbol->getType()))
+			{
+				item.query = { symbol };
+				break;
+			}
+		}
+		if (item.query.getOperator() == ObjectQuery::OperatorSearch)
+		{
+			QFAIL(qPrintable(QString::fromLatin1("Invalid replacement: ISOM %1 -> ISSkiOM 2019 %2")
+			                 .arg(operands->value, item.symbol->getNumberAsString())));
+		}
+		
+		// Find inconsistencies
+		if (item.query.getOperator() == ObjectQuery::OperatorSymbol)
+		{
+			auto has_conflict = [&item](const auto& other)->bool {
+				return &other != &item
+				       && other.type != SymbolRule::NoAssignment
+				       && item.query == other.query
+				       && item.symbol != other.symbol;
+			};
+			if (std::any_of(begin(rules), end(rules), has_conflict))
+			{
+				QFAIL(qPrintable(QString::fromLatin1("There are multiple replacements for symbol %1.")
+				                 .arg(item.query.symbolOperand()->getNumberAsString())));
+			}
+		}
+	}
+	
+	target.applyOnAllObjects(rules);
+		
+	deleteMarkedSymbols(target);
+	
+	target.sortSymbols(ISSkiOM_2019::symbolOrder);
+}
+
+
+struct ScaleFactors
+{
+	unsigned int scale;
+	double track_symbol_factor;
+	double general_factor;
+};
+
+auto const scale_factors = {
+    ScaleFactors { 15000u, 1.0,  1.0 },
+    ScaleFactors { 12500u, 1.0,  1.2 },
+    ScaleFactors { 10000u, 1.0,  1.5 },
+    ScaleFactors {  7500u, 1.33, 1.5 },
+    ScaleFactors {  5000u, 1.33, 1.5 },
+};
+
+void scale(Map& map, unsigned int /*source_scale*/, unsigned int target_scale)
+{
+	map.setScaleDenominator(target_scale);
+	
+	auto scaling = std::find_if(begin(scale_factors), end(scale_factors), [target_scale](auto const& factors) {
+		return factors.scale == target_scale;
+	});
+	QVERIFY(scaling != end(scale_factors));
+	
+	map.scaleAllObjects(scaling->general_factor, MapCoord());
+	
+	int track_symbols_changed = 0;
+	int general_symbols_changed = 0;
+	auto const num_symbols = map.getNumSymbols();
+	for (int i = 0; i < num_symbols; ++i)
+	{
+		Symbol* symbol = map.getSymbol(i);
+		const int code = symbol->getNumberComponent(0);
+		if (code >= 800 && code < 900)
+		{
+			symbol->scale(scaling->track_symbol_factor);
+			++track_symbols_changed;
+		}
+		else
+		{
+			symbol->scale(scaling->general_factor);
+			++general_symbols_changed;
+		}
+	}
+	QCOMPARE(track_symbols_changed, 15);
+	QCOMPARE(general_symbols_changed, 116);
+}
+
+}  // namespace ISSkiOM_2019
+
+
+namespace Course_Design
+{
+
+MapView* makeView(Map& map, unsigned int target_scale)
+{
+	[&map]() { QCOMPARE(map.getNumTemplates(), 1); } ();
+	auto* view = new MapView { &map };
+	view->setGridVisible(true);
+	if (target_scale == 10000)
+		view->setTemplateVisibility(map.getTemplate(0), { 1, true });
+	else
+		map.deleteTemplate(0);
+	return view;
+}
+
+void resetPrinterConfig(Map& map)
+{
+	auto printer_config = map.printerConfig();
+	printer_config.options.show_templates = true;
+	printer_config.single_page_print_area = true;
+	printer_config.center_print_area = true;
+	printer_config.page_format = { { 200.0, 287.0 }, 5.0 };
+	printer_config.page_format.page_size = QPageSize::A4; 
+	printer_config.print_area = printer_config.page_format.page_rect;
+	map.setPrinterConfig(printer_config);
+}
+
+void scale(Map& map, unsigned int source_scale, unsigned int target_scale)
+{
+	map.setScaleDenominator(target_scale);
+	
+	auto const factor = double(source_scale) / double(target_scale);
+	map.scaleAllObjects(factor, MapCoord());
+}
+
+}  // namespace Course_Design
+
+
 }  // namespace
 
 
-void SymbolSetTool::TranslationEntry::write(QXmlStreamWriter& xml, const QString& language)
+
+void SymbolSetTool::TranslationEntry::write(QXmlStreamWriter& xml) const
 {
 	if (source.isEmpty())
 	{
@@ -207,18 +621,9 @@ void SymbolSetTool::TranslationEntry::write(QXmlStreamWriter& xml, const QString
 	xml.writeTextElement(QLatin1String("source"), source);
 	xml.writeTextElement(QLatin1String("comment"), comment);
 	xml.writeStartElement(QLatin1String("translation"));
-	for (const auto& translation : translations)
-	{
-		if (translation.language == language)
-		{
-			if (!translation.type.isEmpty())
-			{
-				xml.writeAttribute(QLatin1String("type"), translation.type);
-			}
-			xml.writeCharacters(translation.translation);		
-			break;
-		}
-	}
+	if (!type.isEmpty())
+		xml.writeAttribute(QLatin1String("type"), type);
+	xml.writeCharacters(translation);		
 	xml.writeEndElement();  //  translation
 	xml.writeEndElement();  // message
 }
@@ -244,17 +649,18 @@ void saveIfDifferent(const QString& path, Map* map, MapView* view = nullptr)
 		new_data.reserve(existing_data.size()*2);
 	}
 	
-	QBuffer buffer(&new_data);
-	buffer.open(QFile::WriteOnly);
-	XMLFileExporter exporter(&buffer, map, view);
+	XMLFileExporter exporter({}, map, view);
 	auto is_src_format = path.contains(QLatin1String(".xmap"));
 	exporter.setOption(QString::fromLatin1("autoFormatting"), is_src_format);
 	auto retain_compatibility = is_src_format && map->getNumParts() == 1
 	                            && !path.contains(QLatin1String("ISOM2017"));
 	Settings::getInstance().setSetting(Settings::General_RetainCompatiblity, retain_compatibility);
+	
+	QBuffer buffer(&new_data);
+	exporter.setDevice(&buffer);
 	exporter.doExport();
-	QVERIFY(exporter.warnings().empty());
 	buffer.close();
+	QVERIFY(exporter.warnings().empty());
 	
 	if (new_data != existing_data)
 	{
@@ -268,6 +674,44 @@ void saveIfDifferent(const QString& path, Map* map, MapView* view = nullptr)
 	QVERIFY(file.exists());
 }
 
+void saveIfDifferent(QFile& file, QByteArray& new_data, const QByteArray& existing_data)
+{
+	auto find_eol = [](auto first, auto last) {
+		return std::find(first, last, '\n');
+	};
+	auto find_printing = [](auto first, auto last) {
+		return std::find_if(first, last, [](auto c) {
+			return c != ' ' && c != '\t' && c != '\n';
+		});
+	};
+	using std::begin; using std::end;
+	auto last1 = end(new_data);
+	auto first1 = find_printing(begin(new_data), last1);
+	auto last2 = end(existing_data);
+	auto first2 = find_printing(begin(existing_data), last2);
+	auto equal = true;
+	while (equal && first1 != last1)
+	{
+		auto eol1 = find_eol(first1, last1);
+		auto eol2 = find_eol(first2, last2);
+		equal = std::equal(first1, eol1, first2, eol2);
+		first1 = find_printing(eol1, last1);
+		first2 = find_printing(eol2, last2);
+	}
+	
+	if (!equal)
+	{
+		new_data.replace("'", "&apos;");
+		QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Truncate));
+		file.write(new_data);
+		QVERIFY(file.flush());
+		file.close();
+		QCOMPARE(file.error(), QFileDevice::NoError);
+	}
+}
+
+
+
 void SymbolSetTool::initTestCase()
 {
 	QCoreApplication::setOrganizationName(QString::fromLatin1("OpenOrienteering.org"));
@@ -287,7 +731,7 @@ void SymbolSetTool::initTestCase()
 	translations_dir.cd(QDir(QString::fromUtf8(MAPPER_TEST_SOURCE_DIR)).absoluteFilePath(QStringLiteral("../translations")));
 	QVERIFY(translations_dir.exists());
 	
-	Template::pathForSaving = &Template::getTemplateRelativePath;
+	Template::suppressAbsolutePaths = true;
 	
 	translations_complete = false;
 }
@@ -301,11 +745,13 @@ void SymbolSetTool::processSymbolSet_data()
 	QTest::addColumn<unsigned int>("source_scale");
 	QTest::addColumn<unsigned int>("target_scale");
 
-	QTest::newRow("ISOM2017 1:15000") << QString::fromLatin1("ISOM2017")  << 15000u << 15000u;
-	QTest::newRow("ISOM2017 1:10000") << QString::fromLatin1("ISOM2017")  << 15000u << 10000u;
+	QTest::newRow("ISOM 2017-2 1:15000") << QString::fromLatin1("ISOM 2017-2")  << 15000u << 15000u;
+	QTest::newRow("ISOM 2017-2 1:10000") << QString::fromLatin1("ISOM 2017-2")  << 15000u << 10000u;
+	
+	QTest::newRow("ISOM2017 translation-only") << QString::fromLatin1("ISOM2017") << 15000u << 15000u;
+	
+	QTest::newRow("ISOM2000 translation-only") << QString::fromLatin1("ISOM2000") << 15000u << 15000u;
 	
-	QTest::newRow("ISOM2000 1:15000") << QString::fromLatin1("ISOM2000")  << 15000u << 15000u;
-	QTest::newRow("ISOM2000 1:10000") << QString::fromLatin1("ISOM2000")  << 15000u << 10000u;
 	QTest::newRow("ISSOM 1:5000") << QString::fromLatin1("ISSOM") <<  5000u <<  5000u;
 	QTest::newRow("ISSOM 1:4000") << QString::fromLatin1("ISSOM") <<  5000u <<  4000u;
 	
@@ -315,9 +761,13 @@ void SymbolSetTool::processSymbolSet_data()
 	QTest::newRow("ISMTBOM 1:7500")  << QString::fromLatin1("ISMTBOM") << 15000u <<  7500u;
 	QTest::newRow("ISMTBOM 1:5000")  << QString::fromLatin1("ISMTBOM") << 15000u <<  5000u;
 	
-	QTest::newRow("ISSkiOM 1:15000") << QString::fromLatin1("ISSkiOM") << 15000u << 15000u;
-	QTest::newRow("ISSkiOM 1:10000") << QString::fromLatin1("ISSkiOM") << 15000u << 10000u;
-	QTest::newRow("ISSkiOM 1:5000")  << QString::fromLatin1("ISSkiOM") << 15000u <<  5000u;
+	QTest::newRow("ISSkiOM translation-only") << QString::fromLatin1("ISSkiOM") << 15000u << 15000u;
+	
+	QTest::newRow("ISSkiOM 2019 1:15000") << QString::fromLatin1("ISSkiOM 2019") << 15000u << 15000u;
+	QTest::newRow("ISSkiOM 2019 1:12500") << QString::fromLatin1("ISSkiOM 2019") << 15000u << 12500u;
+	QTest::newRow("ISSkiOM 2019 1:10000") << QString::fromLatin1("ISSkiOM 2019") << 15000u << 10000u;
+	QTest::newRow("ISSkiOM 2019 1:7500")  << QString::fromLatin1("ISSkiOM 2019") << 15000u <<  7500u;
+	QTest::newRow("ISSkiOM 2019 1:5000")  << QString::fromLatin1("ISSkiOM 2019") << 15000u <<  5000u;
 	
 	QTest::newRow("Course Design 1:15000") << QString::fromLatin1("Course_Design") << 10000u << 15000u;
 	QTest::newRow("Course Design 1:10000") << QString::fromLatin1("Course_Design") << 10000u << 10000u;
@@ -331,23 +781,10 @@ void SymbolSetTool::processSymbolSet()
 	auto completeness  = translations_complete;
 	translations_complete = false;
 	
-	auto raw_tag = QTest::currentDataTag();
-	auto tag = QByteArray::fromRawData(raw_tag, int(qstrlen(raw_tag)));
-	
 	QFETCH(QString, name);
 	QFETCH(unsigned int, source_scale);
 	QFETCH(unsigned int, target_scale);
 	
-	auto id = name;
-	auto language = QString{};
-	if (!tag.endsWith('0'))
-	{
-		auto suffix_index = name.lastIndexOf(QLatin1Char('_'));
-		id = name.left(suffix_index);
-		language = name.mid(suffix_index + 1);
-		Q_ASSERT(language.length() == 2);
-	}
-	
 	QString source_filename = QString::fromLatin1("src/%1_%2.xmap").arg(name, QString::number(source_scale));
 	QVERIFY(symbol_set_dir.exists(source_filename));
 	
@@ -355,11 +792,13 @@ void SymbolSetTool::processSymbolSet()
 	
 	Map map;
 	MapView view{ &map };
-	map.loadFrom(source_path, nullptr, &view, false, false);
+	map.loadFrom(source_path, &view);
 	QCOMPARE(map.getScaleDenominator(), source_scale);
 	QCOMPARE(map.getNumClosedTemplates(), 0);
 	
-	map.setSymbolSetId(id);
+	auto id = map.symbolSetId();
+	QCOMPARE(id, name);
+	
 	map.resetPrinterConfig();
 	map.undoManager().clear();
 	for (int i = 0; i < map.getNumColors(); ++i)
@@ -377,6 +816,11 @@ void SymbolSetTool::processSymbolSet()
 	}
 	saveIfDifferent(source_path, &map, &view);
 	
+	if (name == QLatin1String("ISSkiOM 2019"))
+	{
+		ISSkiOM_2019::mergeISOM(map, symbol_set_dir);
+	}
+	
 	const int num_symbols = map.getNumSymbols();
 	QStringList previous_numbers;
 	for (int i = 0; i < num_symbols; ++i)
@@ -390,215 +834,49 @@ void SymbolSetTool::processSymbolSet()
 		QVERIFY2(symbol->validate(), qPrintable(number_and_name + QLatin1String(": Symbol validation failed")));
 	}
 	
-	auto purple = QColor::fromCmykF(0, 1, 0, 0).hueF();
+	if (std::none_of(begin(translation_entries), end(translation_entries),
+	                 [&id](auto const& entry) { return entry.context == id; }) )
+	{
+		processTranslation(map, translation_entries);
+	}
+	
 	if (source_scale != target_scale)
 	{
-		map.setScaleDenominator(target_scale);
-		
-		if (name.startsWith(QLatin1String("ISOM2000")))
+		if (name == QStringLiteral("ISOM 2017-2"))
 		{
-			const double factor = double(source_scale) / double(target_scale);
-			map.scaleAllObjects(factor, MapCoord());
-			
-			int symbols_changed = 0;
-			int north_lines_changed = 0;
-			for (int i = 0; i < num_symbols; ++i)
-			{
-				Symbol* symbol = map.getSymbol(i);
-				const int code = symbol->getNumberComponent(0);
-				const QColor& color = *symbol->guessDominantColor();
-				if (qAbs(purple - color.hueF()) > 0.1
-				    && code != 602
-				    && code != 999)
-				{
-					symbol->scale(factor);
-					++symbols_changed;
-				}
-				
-				if (code == 601 && symbol->getType() == Symbol::Area)
-				{
-					AreaSymbol::FillPattern& pattern0 = symbol->asArea()->getFillPattern(0);
-					if (pattern0.type == AreaSymbol::FillPattern::LinePattern)
-					{
-						switch (target_scale)
-						{
-						case 10000u:
-							pattern0.line_spacing = 40000;
-							break;
-						default:
-							QFAIL("Undefined north line spacing for this scale");
-						}
-						++north_lines_changed;
-					}
-				}
-			}
-			QCOMPARE(symbols_changed, 139);
-			QCOMPARE(north_lines_changed, 2);
-		}
-		else if (name.startsWith(QLatin1String("ISOM2017")))
-		{
-			const auto factor = double(source_scale) / double(target_scale);
-			map.scaleAllObjects(factor, MapCoord{});
-			
-			int symbols_changed = 0;
-			for (int i = 0; i < num_symbols; ++i)
-			{
-				Symbol* symbol = map.getSymbol(i);
-				const int code = symbol->getNumberComponent(0);
-				if (code != 602
-				    && code != 999)
-				{
-					symbol->scale(factor);
-					++symbols_changed;
-				}
-			}
-			QCOMPARE(symbols_changed, 184);
+			ISOM_2017_2::scale(map, source_scale, target_scale);
 		}
 		else if (name.startsWith(QLatin1String("ISSOM")))
 		{
-			int north_lines_changed = 0;
-			for (int i = 0; i < num_symbols; ++i)
-			{
-				Symbol* symbol = map.getSymbol(i);
-				const int code = symbol->getNumberComponent(0);
-				if (code == 601 && symbol->getType() == Symbol::Area)
-				{
-					AreaSymbol::FillPattern& pattern0 = symbol->asArea()->getFillPattern(0);
-					if (pattern0.type == AreaSymbol::FillPattern::LinePattern)
-					{
-						switch (target_scale)
-						{
-						case 4000u:
-							pattern0.line_spacing = 37500;
-							break;
-						default:
-							QFAIL("Undefined north line spacing for this scale");
-						}
-						++north_lines_changed;
-					}
-				}
-			}
-			QCOMPARE(north_lines_changed, 2);
+			ISSOM::scale(map, source_scale, target_scale);
 		}
 		else if (name.startsWith(QLatin1String("ISMTBOM")))
 		{
-			QCOMPARE(source_scale, 15000u);
-			const double factor = (target_scale >= 15000u) ? 1.0 : 1.5;
-			map.scaleAllObjects(factor, MapCoord());
-			
-			int symbols_changed = 0;
-			for (int i = 0; i < num_symbols; ++i)
-			{
-				Symbol* symbol = map.getSymbol(i);
-				const int code = symbol->getNumberComponent(0);
-				if (code != 602
-				    && code != 999)
-				{
-					symbol->scale(factor);
-					++symbols_changed;
-				}
-			}
-			QCOMPARE(symbols_changed, 168);
+			ISMTBOM::scale(map, source_scale, target_scale);
 		}
-		else if (name.startsWith(QLatin1String("ISSkiOM")))
+		else if (name == QLatin1String("ISSkiOM 2019"))
 		{
-			QCOMPARE(source_scale, 15000u);
-			const double factor = (target_scale >= 15000u) ? 1.0 : 1.5;
-			map.scaleAllObjects(factor, MapCoord());
-			
-			int symbols_changed = 0;
-			int north_lines_changed = 0;
-			for (int i = 0; i < num_symbols; ++i)
-			{
-				Symbol* symbol = map.getSymbol(i);
-				const int code = symbol->getNumberComponent(0);
-				const QColor& color = *symbol->guessDominantColor();
-				if (qAbs(purple - color.hueF()) > 0.1
-				    && code != 602
-				    && code != 999)
-				{
-					symbol->scale(factor);
-					++symbols_changed;
-				}
-				
-				if (code == 601 && symbol->getType() == Symbol::Area)
-				{
-					AreaSymbol::FillPattern& pattern0 = symbol->asArea()->getFillPattern(0);
-					if (pattern0.type == AreaSymbol::FillPattern::LinePattern)
-					{
-						switch (target_scale)
-						{
-						case 5000u:
-						case 10000u:
-							pattern0.line_spacing = 40000;
-							break;
-						default:
-							QFAIL("Undefined north line spacing for this scale");
-						}
-						++north_lines_changed;
-					}
-				}
-			}
-			QCOMPARE(symbols_changed, 152);
-			QCOMPARE(north_lines_changed, 2);
+			ISSkiOM_2019::scale(map, source_scale, target_scale);
 		}
 		else if (name.startsWith(QLatin1String("Course_Design")))
 		{
-			const double factor = double(source_scale) / double(target_scale);
-			map.scaleAllObjects(factor, MapCoord());
+			Course_Design::scale(map, source_scale, target_scale);
 		}
 		else
 		{
 			QFAIL("Symbol set not recognized");
 		}
 	}
-	else
-	{
-		// Not scaled: Collect translation source strings.
-		auto num_colors = map.getNumColors();
-		for (int i = 0; i < num_colors; ++i)
-		{
-			auto color = map.getColor(i);
-			auto source = color->getName();
-			auto comment = QString{QLatin1String("Color ") + QString::number(color->getPriority())};
-			addSource(translation_entries, id, source, comment);
-		}
-		
-		for (int i = 0; i < num_symbols; ++i)
-		{
-			auto symbol = map.getSymbol(i);
-			auto source = symbol->getName();
-			auto comment = QString{QLatin1String("Name of symbol ") + symbol->getNumberAsString()};
-			addSource(translation_entries, id, source, comment);
-			
-			source = symbol->getDescription();
-			comment = QString{QLatin1String("Description of symbol ") + symbol->getNumberAsString()};
-			if (source.isEmpty())
-				qInfo("%s: empty", qPrintable(comment));
-			else
-				addSource(translation_entries, id, source, comment);
-		}
-	}
+	QCOMPARE(map.getScaleDenominator(), target_scale);
+	
+	if (legacy_symbol_sets.contains(id))
+		return;
 	
 	MapView* new_view = nullptr;
 	if (name.startsWith(QLatin1String("Course_Design")))
 	{
-		QCOMPARE(map.getNumTemplates(), 1);
-		new_view = new MapView { &map };
-		new_view->setGridVisible(true);
-		if (target_scale == 10000)
-			new_view->setTemplateVisibility(map.getTemplate(0), { 1, true });
-		else
-			map.deleteTemplate(0);
-		
-		auto printer_config = map.printerConfig();
-		printer_config.options.show_templates = true;
-		printer_config.single_page_print_area = true;
-		printer_config.center_print_area = true;
-		printer_config.page_format = { { 200.0, 287.0 }, 5.0 };
-		printer_config.page_format.paper_size = QPrinter::A4; 
-		printer_config.print_area = printer_config.page_format.page_rect;
-		map.setPrinterConfig(printer_config);
+		new_view = Course_Design::makeView(map, target_scale);
+		Course_Design::resetPrinterConfig(map);
 	}
 	else
 	{
@@ -615,19 +893,22 @@ void SymbolSetTool::processSymbolSet()
 void SymbolSetTool::processSymbolSetTranslations_data()
 {
 	QTest::addColumn<int>("unused");
-	for (auto suffix : translation_suffix)
-		QTest::newRow(suffix) << 0;
+	QTest::newRow("map_symbols_template.ts") << 0;
+#ifdef MAP_SYMBOL_TRANSLATIONS
+	const auto files = map_symbol_translations.split(';');
+	for (const auto& file : files)
+		QTest::newRow(file) << 0;
+#endif
 }
 	
-void SymbolSetTool::processSymbolSetTranslations()
+void SymbolSetTool::processSymbolSetTranslations() const
 {
-	auto suffix = QTest::currentDataTag();
-	auto language = QString::fromLatin1(suffix);
+	auto translation_filename = QString::fromLatin1(QTest::currentDataTag());
+	auto language = translation_filename.mid(int(qstrlen("map_symbols_")));
+	language.chop(int(qstrlen(".ts")));
 	if (language.length() > 2)
 		language.clear();
 	
-	auto translation_filename = QString::fromLatin1("map_symbols_%1.ts").arg(QLatin1String(suffix));
-	
 	QByteArray new_data;
 	QByteArray existing_data;
 	QFile file(translations_dir.absoluteFilePath(translation_filename));
@@ -647,7 +928,7 @@ void SymbolSetTool::processSymbolSetTranslations()
 	}
 	
 	QBuffer buffer(&existing_data);
-	auto translations_from_ts = readTsFile(buffer, language);
+	auto translations_from_ts = readTsFile(buffer);
 	QVERIFY(!buffer.isOpen());
 	
 	buffer.setBuffer(&new_data);
@@ -662,75 +943,84 @@ void SymbolSetTool::processSymbolSetTranslations()
 		xml.writeAttribute(QLatin1String("language"), language);
 	
 	auto context = QString{};
-	for (auto& entry : translation_entries)
+	for (auto it = begin(translation_entries), last = end(translation_entries); it != last; ++it)
 	{
+		auto entry = TranslationEntry{*it};  // copy, don't touch original entry.
+		entry.type = NeedsReview;
+		entry.obsolete = false;
+		
 		if (context != entry.context)
 		{
 			if (!context.isEmpty())
+			{
+				writeObsoleteEntries(xml, translations_from_ts, context);
 				xml.writeEndElement(); // context
+			}
 			xml.writeStartElement(QLatin1String("context"));
 			xml.writeTextElement(QLatin1String("name"), entry.context);
 			context = entry.context;
 		}
 		
-		auto item = std::find_if(begin(entry.translations), end(entry.translations), [&language](auto& translation) {
-			return translation.language == language;
-		});
-		if (item == end(entry.translations))
-		{
-			entry.translations.push_back({language, {}, NeedsReview});
-			item = --entry.translations.end();
-		}
-		auto& translation = item->translation;
-		auto& type = item->type;
-		
-		
 		// First attempt: exact context + source + comment (symbol number!) match.
 		auto found = std::find_if(begin(translations_from_ts), end(translations_from_ts), [&entry](auto& current) {
-			return entry.context == current.context && entry.source == current.source && entry.comment == current.comment;
+			return !current.translation.isEmpty()
+			       && entry.context == current.context
+			       && entry.source == current.source
+			       && entry.comment == current.comment;
 		});
-		if (found == end(translations_from_ts))
+		if (found != end(translations_from_ts))
+		{
+			found->obsolete = false;
+		}
+		else
 		{
 			// Second attempt: exact context + source match.
 			found = std::find_if(begin(translations_from_ts), end(translations_from_ts), [&entry](auto& current) {
-				return entry.context == current.context && entry.source == current.source;
+				return !current.translation.isEmpty()
+				       && entry.context == current.context
+				       && entry.source == current.source;
 			});
 		}
 		if (found == end(translations_from_ts))
 		{
 			// Third attempt: exact source match.
 			found = std::find_if(begin(translations_from_ts), end(translations_from_ts), [&entry](auto& current) {
-				return entry.source == current.source;
+				return !current.translation.isEmpty()
+				       && entry.source == current.source;
 			});
 		}
 		if (found != end(translations_from_ts))
 		{
 			// If any of the previous attempts to find a translation succeeded,
 			// the translation is chosen and marked as not being obsolete.
-			auto match = found->translations.front();
-			translation = match.translation;
-			if (match.type != Obsolete)
-				type = match.type;
+			entry.translation = found->translation;
+			if (found->type != Obsolete && found->type != Vanished)
+				entry.type = found->type;
 		}
 		else
 		{
 			// Find an existing translation even with changed source,
 			// using the comment (symbol number!) instead of source.
 			auto found = std::find_if(begin(translations_from_ts), end(translations_from_ts), [&entry](auto& current) {
-				return entry.context == current.context && entry.comment == current.comment;
+				return !current.translation.isEmpty()
+				       && entry.context == current.context
+				       && entry.comment == current.comment;
 			});
 			if (found != end(translations_from_ts))
 			{
-				auto match = found->translations.front();
-				translation = match.translation;
+				entry.translation = found->translation;
 			}
-			// Anyway, this translation needs review.
-			type = NeedsReview;
 		}
-		entry.write(xml, language);
+		
+		if (!entry.translation.isEmpty()
+		    || !legacy_symbol_sets.contains(context))
+		{
+			entry.write(xml);
+		}
 	}
 	if (!context.isEmpty())
 	{
+		writeObsoleteEntries(xml, translations_from_ts, context);
 		xml.writeEndElement();  // context
 	}
 	
@@ -739,22 +1029,17 @@ void SymbolSetTool::processSymbolSetTranslations()
 	buffer.close();
 	
 	// Weblate uses lower-case "utf-8".
-	static auto lower_case_xml = "<?xml version=\"1.0\" encoding=\"utf-8\"?>";
+	static auto lower_case_xml = R"(<?xml version="1.0" encoding="utf-8"?>)";
 	if (!new_data.startsWith(lower_case_xml))
 	{
 		auto pos = new_data.indexOf('>');
 		new_data.replace(0, pos+1, lower_case_xml);
 	}
 	
-	if (new_data != existing_data)
-	{
-		new_data.replace("'", "&apos;");
-		QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Truncate));
-		file.write(new_data);
-		QVERIFY(file.flush());
-		file.close();
-		QCOMPARE(file.error(), QFileDevice::NoError);
-	}
+	// Weblate uses different formatting
+	new_data.replace("    <", "<");
+	
+	saveIfDifferent(file, new_data, existing_data);
 }
 
 
@@ -781,7 +1066,7 @@ void SymbolSetTool::processExamples()
 	
 	Map map;
 	MapView view{ &map };
-	map.loadFrom(source_path, nullptr, &view, false, false);
+	map.loadFrom(source_path, &view);
 	
 	const int num_symbols = map.getNumSymbols();
 	QStringList previous_numbers;
@@ -823,7 +1108,7 @@ void SymbolSetTool::processTestData()
 	
 	Map map;
 	MapView view{ &map };
-	map.loadFrom(source_path, nullptr, &view, false, false);
+	map.loadFrom(source_path, &view);
 	
 	map.undoManager().clear();
 	saveIfDifferent(source_path, &map, &view);
@@ -838,7 +1123,7 @@ void SymbolSetTool::processTestData()
  */
 #ifndef Q_OS_MACOS
 namespace {
-	auto qpa_selected = qputenv("QT_QPA_PLATFORM", "minimal");  // clazy:exclude=non-pod-global-static
+	auto Q_DECL_UNUSED qpa_selected = qputenv("QT_QPA_PLATFORM", "minimal");  // clazy:exclude=non-pod-global-static
 }
 #endif
 
